/*				    				      HTEvent.c
**	MULTITHREADED ACCESS MANAGER
**
**	(c) COPYRIGHT CERN 1994.
**	Please first read the full copyright statement in the file COPYRIGH.
**
** Authors:
**	HFN    	Henrik Frystyk Nielsen <frystyk@dxcern.cern.ch>
**
** History:
**      05 Jul 95 Written from scratch
**
** Bugs
**
*/

/* Implementation dependent include files */
#include "tcp.h"

/* Library include files */
#include "HTUtils.h"
#include "HTAccess.h"
#include "HTError.h"
#include "HTThread.h"
#include "HTList.h"
#include "HTEvent.h"					 /* Implemented here */
#include "tk.h"

PRIVATE HTList *UserList = NULL;	      /* List of call back functions */
PRIVATE	fd_set	HTfd_user;		   /* Bit array of sockets from user */

/* Type definitions and global variables etc. local to this module */

/*							        HTEventRegister
**
**	Registers a socket as active for READ together with a callback
**	function to call when select returns that the socket is pending.
*/

PRIVATE void HTEventProcessTkEvent(ClientData user_fd, int mask) 
{
  HTEventState state;
  HTRequest *cur_request = NULL;
  int status = 0;
  HTEventCallBack *pres = (HTEventCallBack *) user_fd;
  state = (*(pres->callback))(&cur_request);
}

PUBLIC BOOL HTEventRegister ARGS1(HTEventCallBack *, user_fd)
{
  static BOOL done=NO;
  if (!user_fd) {
    if (THD_TRACE)
      fprintf(stderr, "EventRegist. Bad argument\n");
    return NO;
  }
  if (!done) {
    FD_ZERO(&HTfd_user);
    done = YES;
  }

  FD_SET(user_fd->sockfd, &HTfd_user);
  HTThreadState(user_fd->sockfd, THD_SET_INTR);
  Tk_CreateFileHandler(user_fd->sockfd, TK_READABLE, 
		       HTEventProcessTkEvent, (ClientData) user_fd);

  return YES;
}


/*							         HTEventCleanup
**
**	Clears the list of registered user sockets and call back functions
*/
PUBLIC void HTEventCleanup NOARGS
{
}


/*							      HTEventCheckState
**
**	This function checks the return code from the client call back 
**	function HTEventHandler() and HTEventRequestTerminate().
**
**	Returns:
**		YES	If we stay in eventloop
**		NO	If we return to client application
*/
PRIVATE BOOL HTEventCheckState ARGS2(HTRequest *, request, HTEventState, ret)
{
    switch (ret) {
      case EVENT_TERM:					   /* Ignore it here */
      case EVENT_OK:
	break;
	
      case EVENT_INTR:
 	{
	    int sockfd;
	    if (request && request->net_info &&
		(sockfd = request->net_info->sockfd) >= 0)
		HTThreadState(sockfd, THD_SET_INTR);
	}
	break;

      case EVENT_INTR_ALL:
	HTThreadMarkIntrAll(&HTfd_user);
	break;

      case EVENT_QUIT:
	return NO;
	break;	       
	
      default:
	if (THD_TRACE)
	    fprintf(stderr, "HTEventLoop. Invalid return code\n");
    }
    return YES;
}    


/*								HTEventLoop
**
**	This is the internal socket event loop that is executed when the
**	client doesn't want to keep the event loop. The client can interrupt
**	this loop by sending an event on on of the registered sock descriptors
**	in HTThreadInit() in the HTThread.c module
**
**	The input parameters are for the first load, i.e. the home page.
**	The keywords can be NULL. The Home Anchor can easily be found using
**	HTHomeAnchor().
**
**	Returns:
**		HT_ERROR	if can't get first document or error in select
**				(only if it failed with no blocking)
**		HT_OK		On normal exit
*/
PUBLIC int HTEventLoop ARGS3(HTRequest *,	homerequest,
			     HTParentAnchor *,	homeanchor,
			     CONST char *,	homekeywords)
{
  return HT_ERROR;
}

