/* TkWWWCmds.c Adds World Wide Web commands to a Tcl interpreter
** ===============
**
** Authors:
** Joseph Wang, Department of Astronomy, University of Texas at Austin
** (joe@astro.as.utexas.edu)
**
** Copyright:
** Copyright (C) 1992-1994
** Globewide Network Academy
** Macvicar Institute for Educational Software Development
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* Include c stuff
** ---------------
*/

#include <assert.h>
#include <stdio.h>
#include <ctype.h>

#if STDC_HEADERS
#include <sys/types.h>
#include <stdlib.h>
#endif 

/* Include World Wide Web stuff
** ----------------------------
*/

#include <HTAnchor.h>
#include <HTParse.h>
#include <HTAccess.h>
#include <HTML.h>
#include <HText.h>
#include <HTTCP.h>
#include <HTStream.h>
#include <HTFile.h>
#include <HTFWriter.h>
#include <HTextDef.h>
#include <HTAtom.h>
#include <HTPlain.h>
#include <HTMLGen.h>
#include <HTFormat.h>
#include <HTMIME.h>
#include <HTList.h>
#include <HTInit.h>
#include <HTAlert.h>
#include <TkWWWCmds.h>
#include <TkWWWVersion.h>

/* Some Global Variables
** ---------------------
*/

PUBLIC char * HTAppName = "tkWWW";	/* Application name */
PUBLIC char * HTAppVersion = SERVER_VERSION; 	/* Application version */
PUBLIC char * HTTkSaveDir = "/usr/tmp";
PUBLIC Tcl_Interp *HtTclInterp = NULL;
EXTERN char * HTCacheDir;

PRIVATE char cache_dir[256];

EXTERN HTPresentation* default_presentation;
					   
/* Macro to check arguments
** ------------------------
** Assumes interpreter is in interp and number of arguments is in argc
*/

#define HtCheckArgc(min,max,function_name) \
    if ((argc < (min)) || (argc > (max))) { \
       Tcl_AppendResult(interp, (function_name), \
			": Incorrect number of arguments" , NULL); \
       return (TCL_ERROR); \
       }

/* Procedures called by tkWWW functions
** ------------------------------------
*/

PRIVATE int 
HtLoadCmd(dummy, interp, argc, argv)
     ClientData dummy;
     Tcl_Interp *interp;
     int argc;
     char **argv;
{
  HTRequest *request;
  HTTkW3Context *context;
  int error_code;

  HtCheckArgc(2,3, "HtLoad");

  request = HTRequest_new();
  context = (HTTkW3Context *) malloc(sizeof(HTTkW3Context));
  context->interp = interp;
  request->context = context;
  HtTclInterp = interp;
  context->error_code = TCL_OK;
  
  if (argc == 3)
    StrAllocCopy(context->target, argv[2]);
   else
    context->target = NULL;
  

  if (!HTLoadAbsolute(argv[1], request)) {
    if (*interp->result == '\0') {
      HTLoadError(request, 500, "Unable to access document.");
    }
  }

  context = request->context;
  if (context->target) {
    free(context->target);
  }
  error_code = context->error_code;
  free(context);
  HTRequest_delete (request);
  return (error_code);
}

PRIVATE int 
HtParseNameCmd(dummy, interp, argc, argv)
     ClientData dummy;
     Tcl_Interp *interp;
     int argc;
     char **argv;
{
  char *current_address = NULL;
  
  HtCheckArgc(2,3,"HtParseName");
  if (argc == 3 && argv[2] && *(argv[2])) 
    StrAllocCopy(current_address, argv[2]);
  else
    current_address = HTFindRelatedName();

  Tcl_SetResult(interp, HTParse(argv[1], current_address, PARSE_ALL), 
		TCL_DYNAMIC);
  free(current_address);
  
  return (TCL_OK);
}

PRIVATE int 
HtUncacheCmd (dummy, interp, argc, argv) 
     ClientData dummy;
     Tcl_Interp *interp;
     int argc;
     char **argv;
{
  HTParentAnchor *parent_anchor;
  HTAnchor *anchor;
  HText *document;
  HtCheckArgc(2, 2, "HtUncache");

  anchor = HTAnchor_findAddress(argv[1]);
  if (anchor) {
    parent_anchor = HTAnchor_parent(anchor);
    document = (HText *)HTAnchor_document(parent_anchor);
    if (document) 
      HText_free(document);
  }
  return (TCL_OK);
}

PRIVATE int 
HtCleanupCmd(dummy, interp, argc, argv)
     ClientData dummy;
     Tcl_Interp *interp;
     int argc;
     char **argv;
{
  HTLibTerminate();
  return (TCL_OK);
}

PUBLIC BOOL
HTTkSetOutputFile(request, fnam)
     HTRequest *request;
     CONST char *fnam;
 {
  char *address;
  HTTkW3Context *context = (HTTkW3Context *) request->context;
  HText *text = HText_new(request->anchor);
  HTChunkPuts(text->output, "tkW3ConfigDisplayFile ");
  if (context->target)
    HTChunkPuts(text->output, context->target);
  else
    HTChunkPuts(text->output, "{}");
  HTChunkPuts(text->output, " ");
  HTChunkPuts(text->output, HTAtom_name(request->content_type));
  HTChunkPuts(text->output, " ");
  HTChunkPuts(text->output, fnam);
  HTChunkPuts(text->output, " ");
  address = HTAnchor_address((HTAnchor *) request->anchor);
  HTChunkPuts(text->output, address);
  if (request->content_encoding) {
    HTChunkPuts(text->output, " ");
    HTChunkPuts(text->output, HTAtom_name(request->content_encoding));
  }
  HTChunkPuts(text->output, "\n");
  free(address);
  HTChunkTerminate(text->output);
  context->error_code = Tcl_Eval(context->interp, text->output->data);
  return 0;
}

PUBLIC HTStream* HTTkDisplay ARGS5(
        HTRequest *,            request,
        void *,                 param,
        HTFormat,               input_format,
        HTFormat,               output_format,
        HTStream *,             output_stream)

{
  HTStream *me;
  char *old_cache;
  old_cache = HTCacheDir;
  HTCacheDir = HTTkSaveDir;
  request->callback = HTTkSetOutputFile;
  if (!request->content_type) {
    request->content_type = input_format;
  }
  me = HTSaveAndCallBack (request, param, input_format, output_format,
			  output_stream);
  HTCacheDir = old_cache;
  return me;
}



PRIVATE int 
HtAddEncodingCmd(dummy, interp, argc, argv)
     ClientData dummy;
     Tcl_Interp *interp;
     int argc;
     char **argv;
{
  float priority;
  HtCheckArgc(3, 4, "HtAddEncoding");
  priority  = (argc==3) ? 1.0 : atof(argv[3]);
  HTAddEncoding(argv[1], argv[2], priority);
  return (TCL_OK);
}

PRIVATE int 
HtAddTypeCmd(dummy, interp, argc, argv)
     ClientData dummy;
     Tcl_Interp *interp;
     int argc;
     char **argv;
{
  float priority;
  HtCheckArgc(4, 5, "HtAddType");
  priority  = (argc==4) ? 1.0 : atof(argv[4]);
  HTAddType(argv[1], argv[2], argv[3], priority);
  return (TCL_OK);
}

/* Add tkWWW commands to a tcl interpreter init_tkWWW(interp)
** -------------
*/

int 
WWW_AppInit(interp)
     Tcl_Interp *interp;
{
  if (!HTConversions) 
    HTConversions = HTList_new();
  Tcl_CreateCommand(interp, "HtAddType", HtAddTypeCmd,
		    (ClientData) NULL, (void (*)()) NULL);
  Tcl_CreateCommand(interp, "HtAddEncoding", HtAddEncodingCmd,
		    (ClientData) NULL, (void (*)()) NULL);
  Tcl_CreateCommand(interp, "HtLoad", HtLoadCmd, 
		    (ClientData) NULL, (void (*)()) NULL);
  Tcl_CreateCommand(interp, "HtParseName", HtParseNameCmd,
		    (ClientData) NULL, (void (*)()) NULL);
  Tcl_CreateCommand(interp, "HtUncache", HtUncacheCmd,
		    (ClientData) NULL, (void (*)()) NULL);
  Tcl_CreateCommand(interp, "HtCleanup", HtCleanupCmd,
		    (ClientData) NULL, (void (*)()) NULL);


  HTSetConversion(HTConversions, "www/mime",  "*/*",           
		   HTMIMEConvert, 	1.0, 0.0, 0.0);
  HTSetConversion(HTConversions, "text/html", "text/x-c",    
		   HTMLToC, 	        0.5, 0.0, 0.0);
  HTSetConversion(HTConversions, "text/html", "text/plain",  
		   HTMLToPlain, 	0.5, 0.0, 0.0);
  HTSetConversion(HTConversions, "text/html", "www/present", 
		   HTMLPresent, 	1.0, 0.0, 0.0);
  HTSetConversion(HTConversions, "text/plain", "text/html",  
		   HTPlainToHTML,	1.0, 0.0, 0.0);
  HTSetConversion(HTConversions, "*/*", "www/present",
		   HTTkDisplay,      0.3, 0.0, 0.0);

  HTLibInit();
  if (HTClientHost) HTSecure = YES;
  return (TCL_OK);
}
