## bookmarks.tcl tkW3 bookmarks facility
## ==============
## Copyright (C) 1992-1993
## Globewide Network Academy
## Macvicar Institute for Educational Software Development
##
## See the file COPYRIGHT for conditions

## This file contains code to implement the bookmarks feature of tkW3

## Conventions:
##   all global visible objects in this file should begin with 
##   tkW3Bookmarks

# tkW3BookmarksList contains bookmarks in 
# {{title address} {title address}} format

proc tkW3BookmarksInitialize {} {
    global env
     tkW3BookmarksClass tkw3bookmarks \
	 -file $env(HOME)/.mosaic-hotlist-default \
	 -goto_command tkW3NavigateRecordAndGoto \
	 -clone_command tkW3NavigateClone
}

tixClass tkW3BookmarksClass {
    -superclass {}
    -classname TkW3BookmarksClass
    -shortname tkW3Bookmarks
    -method {
	popup goto_selected get_selected \
	    clone_selected delete_selected edit_title \
	    do_edit_title add load save
    }
    -flag {
	-file -goto_command -clone_command
    }
    -readonly {
	-widget -bookmarks -listbox
    }
    -configspec {
	{-file ""}
	{-goto_command ""}
	{-clone_command ""}
    }
}

proc tkW3Bookmarks::Constructor {w} {
    upvar #0 $w data
    set data(-widget) .bookmarks
    $w load

    tixDialogShell $data(-widget)
    tkW3OutputMakeButtons $data(-widget).button_frame {
	{current "Add current" "tkW3BookmarksAdd"}
	{remove "Remove" "tkW3BookmarksDeleteSelected"}
	{edit "Edit Title" "tkW3BookmarksEditTitle"}
    }

    pack append $data(-widget) \
	$data(-widget).button_frame top \
	[tixScrolledListbox $data(-widget).list] {top expand fill} \
	[tixDlgBtns $data(-widget).btns ] {bottom fill}
    set btns $data(-widget).btns
    $btns add goto -text "Goto" -command "$w goto_selected"
    $btns add clone -text "Clone" -command "$w clone_selected"
    $btns add cancel -text "Dismiss" -command "$data(-widget) popdown"

    set data(-listbox) [$data(-widget).list listbox]

    foreach item $data(-bookmarks) {
	$data(-listbox) insert end [lindex $item 0]
    }

    bind $data(-listbox) <Double-1> "$btns button goto flash
%W select set \[%W nearest %y\]
$w goto_selected"

    bind $data(-listbox) <Double-2> "$btns button goto flash
%W select set \[%W nearest %y\]
$w clone_selected"

}

proc tkW3Bookmarks::popup {w} {
    upvar #0 $w data
#    $data(-widget).btns focus_in goto
    $data(-widget) popup
}

proc tkW3Bookmarks::get_selected {w} {
    upvar #0 $w data
    lindex [lindex $data(-bookmarks) [$data(-listbox) curselection]] 1
}

proc tkW3Bookmarks::goto_selected {w} {
    upvar #0 $w data
    $data(-goto_command) [$w get_selected]
}

proc tkW3Bookmarks::clone_selected {w} {
    upvar #0 $w data
    $data(-clone_command) [$w get_selected]
}

proc tkW3Bookmarks::delete_selected {} {
    upvar #0 $w data
    set i [$data(-listbox) curselection]
    set data(-bookmarks) [ lreplace $data(-bookmarks) $i $i]
    $data(-listbox) delete $i
    $w save
}

proc tkW3Bookmarks::edit_title {w} {
    upvar #0 $w data
    set i [$data(-listbox) curselection]
    set title [lindex [lindex $data(-bookmarks) $i] 0]

    DLG:entry . .edit_dialog "Bookmark title" \
	{"Title:"} "Edit" "$w do_edit_title $i"
    DLG:set_entry_value .edit_dialog 1 $title
    .edit_dialog popup
}

proc tkW3Bookmarks::do_edit_title {w i title} {
    upvar #0 $w data
    set address [lindex [lindex $data(-bookmarks) $i] 1]
    set data(-bookmarks) \
	[lreplace $data(-bookmarks) $i $i [list $title $address]]
    $data(-listbox) delete $i
    $data(-listbox) insert $i $title
    $w save
}

## tkW3BookmarksAdd: Add a bookmark
## -----------
## Arguments
##   title - Title of the bookmark
##   address - Address bookmark refers to
##
## If called with no arguments it sets title and address to current values
## No return value
## 
## This procedure adds a bookmark to the list of bookmarks
##

proc tkW3Bookmarks::add {w title address} {
    upvar #0 $w data
    lappend data(-bookmarks) [list $title $address]
    $w save
    $data(-listbox) insert end $title
}

# Load bookmarks from file

proc tkW3Bookmarks::load {w} {
    upvar #0 $w data

# open the file
# trap for errors
    if {[catch {open $data(-file) r} file]} {
	tkW3OutputError $file
	return
    }

# clear the bookmarks
    set data(-bookmarks) ""

    gets $file format
    gets $file name

# while there is stuff in the file
    while {[gets $file string] != -1} { 
	# the line contains a hypertext anchor load it
	set address [lindex [split $string " \t"] 0]
	gets $file title
	lappend data(-bookmarks) [list $title $address]
    }
    close $file
}

# save bookmarks in "filename"
proc tkW3Bookmarks::save {w} {
    upvar #0 $w data

    if {[catch {open $data(-file) w} file]} {
	tkW3OutputError $file
	return
    }

    puts $file "ncsa-xmosaic-hotlist-format-1"
    puts $file "default"

    foreach item $data(-bookmarks) {
	puts $file "[lindex $item 1] [exec date]"
	puts $file "[lindex $item 0]"
    }
    close $file
}
