## edit.tcl main file for tkWWW  user interface
## ==============
## Copyright (C) 1992-1994
## Globewide Network Academy
## Macvicar School of Education and Technology
##
## See the file COPYRIGHT for conditions

## The procedures in this file are responsible for editing inside of tkWWW.

## Conventions:
##   All procedures in this file begin with tkWWWEdit

## Bugs and Todo:
##   Some abstraction questions

# This is the style that the editor is current writing in

set tkW3EditStyle(para) ""
set tkW3EditStyle(char) ""

# This is the cut buffer
set tkW3EditSelection {}

## ********************
## Initialization procedures
## ********************

tkW3ConversionClass tkw3style_table -list {
    {"" "default"}
    {H1 heading1}
    {H2 heading2}
    {H3 heading3}
    {H4 heading4}
    {H5 heading5}
    {H6 heading6}
    {ADDRESS address}
    {PRE preformatted}
    {XMP example}
    {TT typewriter}
    {B bold}
    {I italics}
    {U underline}
    {EM emphasis}
    {STRONG "strong emphasis"}
    {SAMP sample}
    {KBD keyboard}
    {VAR variable}
    {DFN definition}
    {CITE citation}
    {CODE code}
    {unknown "unknown tag"}
}

tkW3ConversionClass tkw3list_table -list {
    {"" "none"}
    {UL "Itemized"}
    {OL "Numbered"}
    {DL "Glossary Term"}
    {DD "Glossary Definition"}
}


proc tkW3EditInitialize {} {
    global tkW3EditChar tkW3EditTags tkW3EditVars

    # divide tags into classes
    set tkW3EditTags(para) {H1 H2 H3 H4 H5 H6 PRE ADDRESS}
    set tkW3EditTags(char) {TT B I U EM STRONG SAMP KBD VAR DFN CITE}
    set tkW3EditTags(bull) {LI DD DT P HR}

    tkW3EditBindText .f.msg
    tkW3EditDisplayStatus

    set tkW3EditVars(anchorStart) {}
    set tkW3EditVars(href) {}
    set tkW3EditVars(name) {}

# if a key possibly starts a 2-key sequence, to be translated according to
# tkW3EditCharTranslate, store this pending char here:
    set tkW3EditChar(pending) ""
    

    foreach tag $tkW3EditTags(para) {
	.style subwidget para insert end [tkw3style_table forward $tag]
    }

    foreach tag $tkW3EditTags(char) {
	.style subwidget char insert end [tkw3style_table forward $tag]
    }
    .style subwidget para  insert 0 [tkw3style_table forward ""]
    .style subwidget char  insert 0 [tkw3style_table forward ""]

    .style subwidget para config -command "tkW3EditSetStyleFromName para"
    .style subwidget char config -command "tkW3EditSetStyleFromName char"

}

set tkW3ConfigEditTk {
    .edit.menu.file.m delete "*Quit*"
    .edit.menu.file.m entryconfigure "Save" -command "tkW3EditSave"
    .edit.menu.edit.m add command -label "Spell Check" -command \
	{ispell_text .edit.main.t}
    set button_list {
	{generate "Generate Source" "tkW3EditGenerate"}
	{save "Save As..." "tkW3EditSave 1"}
	{close "Close Window" "tkW3EditClose"}
    }
    pack append .edit \
	[ tkW3OutputMakeButtons .edit.button_box $button_list] {bottom fillx}
}

proc tkW3EditBindText {w} {
    global tkW3EditStyle

    bind $w <KeyPress> {tkW3EditInsert %W %A}
    bind $w <Shift-KeyPress> {tkW3EditInsert %W %A}
    bind $w <Control-y> {tkW3EditInsert [tkW3EditSelectionGet]}

    bind $w <ButtonPress-2> {tkW3HtButtonPress %W @%x,%y %b %t}
    bind $w <ButtonPress-3> {tkW3HtButtonPress %W @%x,%y %b %t}
    bind $w <ButtonRelease-1> {tkW3HtButtonRelease %W @%x,%y %b %t}
    bind $w <ButtonRelease-2> {tkW3HtButtonRelease %W @%x,%y %b %t}
    bind $w <ButtonRelease-3> {tkW3HtButtonRelease %W @%x,%y %b %t}

    bind $w <ButtonPress-1> {
	tkW3EditSelectionClear %W
	set tk_priv(selectMode) char
	%W mark set insert @%x,%y
	%W mark set insert [tkW3EditDisplace %W [%W index insert]]
	%W mark set anchor insert
	tkW3EditSelectionClear %W
	if {[lindex [%W config -state] 4] == "normal"} {focus %W}
	tkW3EditUseStyle %W
	tkW3HtButtonPress %W @%x,%y %b %t
    }

    bind $w <Double-Button-1> {
	tkW3EditSelectionClear %W
	set tk_priv(selectMode) word
	%W mark set insert @%x,%y
	%W mark set insert [tkW3EditDisplace %W [%W index insert]]
	%W mark set anchor insert
	%W tag add sel "insert wordstart" "insert wordend"
	if {[lindex [%W config -state] 4] == "normal"} {focus %W}
	tkW3EditUseStyle %W
    }

    bind $w <Return> {
	# first cut any selected text
	if {[%W tag ranges sel] != {}} {
	  tkW3EditCut %W
	}
        set taglist [%W tag names "insert linestart"]
        if {[lsearch $taglist LI] != -1} {
	  set parastyle LI
	} else { set parastyle $tkW3EditStyle(para)}

	case $parastyle {
	    {LI} { tkW3EditAddListItem %W 0 }
	    {H1 H2 H3 H4 H5 H6} {
		tkW3EditInsertClean %W \n
		set tkW3EditStyle(para) ""
		%W yview -pickplace insert
		tkW3EditDisplayStatus
		return
	    }
	    {PRE XMP} {%W insert insert \n}
	    {default} {
		tkW3EditInsertBullet %W "P"
	    }
	}
	%W yview -pickplace insert
	tkW3EditUseStyle %W
    }

    bind $w <Tab> { tkW3EditAddNewList %W UL }
    bind $w <Shift-Tab> { tkW3EditOutListItem %W }

    bind $w <BackSpace> {tkW3EditBackspace %W}
    bind $w <Delete>  {tkW3EditBackspace %W}
    bind $w <Control-h> {tkW3EditBackspace %W}
    bind $w <Control-d> {tkW3EditDelChar %W}
    
    bind $w <Control-f> {tkW3EditMoveInsertTo %W "insert + 1 c" 1}
    bind $w <Right>     {tkW3EditMoveInsertTo %W "insert + 1 c" 1}

    bind $w <Control-b> {tkW3EditMoveInsertTo %W "insert - 1 c" 0}
    bind $w <Left>      {tkW3EditMoveInsertTo %W "insert - 1 c" 0}

    bind $w <Control-a> {tkW3EditMoveInsertTo %W "insert linestart" 1}
    
    bind $w <Control-e> {tkW3EditMoveInsertTo %W "insert lineend" 0}

    bind $w <Control-n> {tkW3EditMoveInsertTo %W "insert + 1 lines"}
    bind $w <Down>      {tkW3EditMoveInsertTo %W "insert + 1 lines"}

    bind $w <Control-p> {tkW3EditMoveInsertTo %W "insert - 1 lines" 0}
    bind $w <Up>        {tkW3EditMoveInsertTo %W "insert - 1 lines" 0}

    bind $w <Meta-f>    {tkW3EditMoveInsertTo %W "insert + 1 c wordend" 1}
    bind $w <Meta-b>    {tkW3EditMoveInsertTo %W "insert - 1 c wordstart" 0}
    bind $w <F10>  {tk_firstMenu %W}

  bind $w <Shift-Delete> "tkW3EditCut $w"
  bind $w <Control-Insert> "tkW3EditCopy $w"
  bind $w <Shift-Insert> "tkW3EditPaste $w"

  # used by PER for debugging purposes; not for final product
  bind $w <F11>  {foreach f {edit.tcl edit.generate.tcl hypertext.tcl init.tcl output.tcl} {source $f} }
  bind $w <F12>  {source maptext.tcl; maptext %W}
  bind $w <Shift-F11>  tkW3OutputCursorWaitNew
  bind $w <Shift-F12>  tkW3OutputCursorNormalNew

}

## ********************
## Keybinding actions
## ********************

proc tkW3EditInsert {w a} {
    global tkW3EditStyle tkW3EditChar

    # first cut any selected text
    if {[string length $a] && [$w tag ranges sel] != {}} {
      tkW3EditCut $w
    }

    set start [$w index insert]
    set tags [$w tag names "insert - 1 c"]
    set add_tag ""
    
    set anchor_number [lsearch $tags "h*"]
    if {$anchor_number != -1} {
	lappend add_tag Anchor
	lappend add_tag [lindex $tags $anchor_number]
    }
    
    if {$tkW3EditStyle(para) != ""} {
	lappend add_tag $tkW3EditStyle(para)
    }

    if {$tkW3EditStyle(char) != ""} {
	lappend add_tag $tkW3EditStyle(char)
    }
    
    if {$a != ""} {
	# this is the regular case of "Modified" :
	tkW3EditModifiedSet
	if { $tkW3EditChar(pending) != ""} {
	    # try to do the translation:
	    set translator [lindex $tkW3EditChar(translate) \
	     [expr [lsearch $tkW3EditChar(translate) $tkW3EditChar(pending)]+1] ]
	    set substit [lindex $translator [lsearch $translator "$a?"]]
	    set char [string index $substit 1]
	    if { $char == "" } {
		if {$a == "\033"} {
		    # ESC = cancell pending char.
		    set a ""
		} {
		    set a "$tkW3EditChar(pending)$a"
		}
	    } {
		set a $char
	    }
	    set tkW3EditChar(pending) ""
	    tkW3OutputSetMessage ""
	}
    }
    if {$a != "\033"} {
	if {[lsearch $tkW3EditChar(translate) $a] == -1} {
	    $w insert insert $a
	    foreach tag [.f.msg tag names $start] {
		.f.msg tag remove $tag $start insert
           }
	    foreach tag $add_tag {
		.f.msg tag add $tag $start insert
	    }
	} {
	    # this key possible starts a 2-key sequence.
	    tkW3OutputSetMessage "char compose for $a"
	    set tkW3EditChar(pending) $a
	}
    }
}

proc tkW3EditBackspace {w} {

    if {[$w tag ranges sel] != {}} {
      tkW3EditCut $w
    } else {
      set i [$w index insert]
      $w delete [tkW3EditDisplace $w [$w index "$i-1c"] 0] $i
      $w yview -pickplace insert
      tkW3EditUseStyle $w
      tkW3EditModifiedSet
    }
}

proc tkW3EditDelChar {w} {
     $w delete insert [tkW3EditDisplace $w [$w index "insert +1c"] 1]
     $w yview -pickplace insert
     tkW3EditUseStyle $w
     tkW3EditModifiedSet
}


proc tkW3EditMoveInsertTo {w loc {dir {}}} {
     # 3. parameter 'dir' sets direction to forward (1) or backward (0);
     #    this applies, if the desired location is not allowed and the
     #    insertion cursor needs to be displaced in either direction.

     $w mark set insert $loc
     $w mark set insert [tkW3EditDisplace $w [$w index insert] $dir]
     $w yview -pickplace insert
     tkW3EditUseStyle $w
}

# displace insertion from "index" to next allowed location, if necessary.
# Insertion is not allowed within the representation of a bullet.
# dir 1 means search forward, 0 backward.

proc tkW3EditDisplace {w index {dir {}}} {
    global tkW3EditTags
    set inbul {}
    set names [$w tag names $index]
    foreach bultag $tkW3EditTags(bull) {
	if {[lsearch $names $bultag] != -1} {
	    set inbul $bultag
	    break
	}
     }
    if {$inbul == {}} {return $index}
    set range [tkW3EditCurRange $w $inbul $index]
    if {$dir != {} } {
	return [lindex $range $dir]
    }
    set start [lindex $range 0]
    set finish [lindex $range 1]
    scan $start  "%d" l1
    scan $finish "%d" l2
    scan $index  "%d" li
    if {($li == $l1) && ($li < $l2)} {
	return $start
    }
    return $finish
}


proc tkW3EditUseStyle {w} {
    global tkW3EditStyle tkW3EditTags
    set tags [$w tag names "insert - 1 c"]
    set tkW3EditStyle(char) ""
    set tkW3EditStyle(para) ""


    foreach tag $tags {
	if {[lsearch $tkW3EditTags(para) $tag] != -1} {
	    set tkW3EditStyle(para) $tag
	}
	if {[lsearch $tkW3EditTags(char) $tag] != -1} {
	    set tkW3EditStyle(char) $tag
	}
    }
    tkW3EditDisplayStatus
}

proc tkW3EditDisplayStatus {} {
    global tkW3EditStyle 
    .style print char [tkw3style_table forward $tkW3EditStyle(char)]
    .style print para [tkw3style_table forward $tkW3EditStyle(para)]

    set taglist [.f.msg tag names "insert linestart"]
    foreach tag [tkw3list_table source_list] {
	if {[lsearch $taglist type.$tag] != -1} {
	    .style print list [tkw3list_table forward $tag]
	    return
	}
    }
    .style print list [tkw3list_table forward ""]
}

proc tkW3EditSetStyleFromName {style} {
    tkW3EditSetStyle char [tkw3list_table back $style]
}

proc tkW3EditSetStyle {type style} {
    global tkW3EditStyle
    
    tkW3EditTagSelected .f.msg  $style
    set tkW3EditStyle($type) $style
    tkW3EditDisplayStatus
}

## ********************
## Procedures to show or hide the edit window
## ********************

proc tkW3EditOpen {{display 1}} {
    global tkW3ConfigEditTk
    if {![winfo exists ".edit"]} {
	edittkmain "" "" toplevel "" $tkW3ConfigEditTk
	
    } 

    if {$display} {
	wm deiconify .edit
    }
}

proc tkW3EditClose {} {
    if [winfo exists .edit] {
	wm withdraw .edit
    }
}


## ********************
## These procedure are used to edit the display window
## ********************

proc tkW3EditQueryChangeTitle {w} {
    global tkW3HtPage
    set w .change_title
    if {![winfo exists $w]} {
	tixEntryDialog $w -prompt {"Title:"} -command \
	    tkW3EditChangeTitle 
    
	tkW3OutputMakeToggles $w.toggle_frame {
	    {base "Include BASE tag"}
	    {is_index "Is index"}
	} 0 {expand fillx left}
	pack append $w \
	    $w.toggle_frame {top fillx } 
    }
    .change_title set_value 1 $tkW3HtPage(title)

    tkW3OutputToggleSet $w.toggle_frame.base $tkW3HtPage(base.use)
    tkW3OutputToggleSet $w.toggle_frame.is_index $tkW3HtPage(is_index)


    .change_title popup
}    


proc tkW3EditChangeTitle {title} {
    global tkW3HtPage base is_index
    set tkW3HtPage(title) [lindex $title 0]
    set tkW3HtPage(base.use) $base
    set tkW3HtPage(is_index) $is_index
    tkW3ConfigDisplay
    $w popdown
}

proc tkW3EditTagSelected {w tag {begin 0} {end 0}} {
    set list [tkW3EditClearTagsFromSelected $w]
    if {$list != {}} {
	if {$begin != 0} {
	    $w mark set insert [lindex $list 0]
	    
	}
	$w tag add $tag [lindex $list 0] [lindex $list 1]
	$w mark set insert [lindex $list 1]
	if {$end != 0} {
	    tkW3HtBlankLines $begin
	    $w mark set insert [lindex $list 1]
	}
	tkW3EditModifiedSet
    }
}

proc tkW3EditBeginGlossaryItem {w} {
    tkW3HtAddBul DT
    set tmp [$w index insert]
    tkW3HtAddBul DD
    $w mark set insert $tmp
}

proc tkW3EditClearTagsFromSelected {w} {
    global tkW3EditCurrentStyle tkW3EditTags

    if {[$w tag ranges sel] != {}} {
	set start [$w index sel.first]
	set finish [$w index sel.last]
	foreach tag [.f.msg tag names $start] {
	  if {[lsearch -exact $tkW3EditTags(para) $tag] > -1} {
	    $w tag remove $tag $start $finish
	  } elseif {[lsearch -exact $tkW3EditTags(char) $tag] > -1} {
	    $w tag remove $tag $start $finish
	  }
	}
	return [list $start $finish]
    }
    set tkW3EditCurrentStyle Normal
    return {}
}

proc tkW3EditCut {w} {
    tkW3EditCopy $w
    tkW3EditDelete $w
}

proc tkW3EditCopy {w} {
    global tkW3EditSelection
    set tkW3EditSelection [$w get sel.first sel.last]
}

proc tkW3EditDelete {w} {
    $w delete sel.first sel.last
}

proc tkW3EditPaste {w} {
    global tkW3EditSelection
    if {"" != [$w tag ranges sel]} {
	$w delete sel.first sel.last
	$w set mark insert sel.first
    }
    tkW3EditInsert $w $tkW3EditSelection
}

proc tkW3EditClear {w} {
    $w delete 1.0 end
}

proc tkW3EditInsertBullet {w type} {
    tkW3HtAddBul $type
}

proc tkW3EditCurRange {w tag index} {
    set ranges [$w tag ranges $tag]
    if {$ranges == {}} {
        return {}
    }
    set i 0
    while {[$w compare [lindex $ranges $i] <= $index] == 1} {
        if {[$w compare [lindex $ranges [expr $i+1]] >= $index] == 1} {
            return [lrange $ranges $i [incr i]]
        }
        incr i 2
        if {[llength $ranges] < $i} {
            return {}
        }
    }
    return {}
}

proc tkW3EditSelectAll {w} {
    $w tag add sel 1.0 end
}

proc tkW3EditDeselectAll {w} {
    $w tag remove sel 1.0 end
}

## ********************
## Procedures dealing with image tags
## ********************

proc tkW3EditConvertImages {w} {
    global tkW3HtPage
    foreach mark [$w mark names] {
	if [regexp {window.*\.image([0-9]+)} $mark {} number] {
	    set src $tkW3HtPage(image.$number)
	    tkW3EditInsertClean $w "\n<IMG SRC=\"$src\">" $mark
	    $w mark unset $mark
	}
    }
}

proc tkW3EditAddImage {w} {
    if ![winfo exists .image_dialog] {
	tixEntryDialog .image_dialog \
	    -title "Image Dialog" \
	    -prompt {"Source:"} \
	    -command "tkW3HtStartElement IMG -SRC "
    }
    .image_dialog popup
}


proc tkW3EditChangeImage {i} {
    global tkW3HtPage
    if ![winfo exists .change_image_dialog] {
	tixEntryDialog .change_image_dialog \
	    -title "Image Dialog" \
	    -prompt {"Source:"} \
	    -command "set tkW3HtPage(image.$i) "
    }
    .change_image_dialog 1 $tkW3HtPage(image.$i)
    .change_image_dialog popup
}

proc tkW3EditSave {{saveas 0}} {
    tkW3FileSaveTextWidget .edit.main.t $saveas
}

# This procedure collapses tags
proc tkW3EditCollapseTag {w tag} {
    global tkW3HtPage

    set i 0
    $w mark set tmp 1.0
    while {[set range [$w tag nextrange $w tmp]] != ""} {
	$w mark set tmp [lindex $range 0]
	set tkW3HtPage(tag.$tag.$i) [$w get tmp [lindex $range 1]]
	$w delete tmp [lindex $range 1]
	$w mark set tag.$tag.$i tmp
	incr 1
    }
}

proc tkW3EditExpandTag {w tag} {
    global tkW3HtPage
    foreach name [$w mark names] {
	if [regexp {^tag\.([^ \.]+\.[0-9]+)$} name {} mark] {
	        set start [$w index $mark]
		$w insert $mark $tkW3HtPage($mark)
		unset tkW3HtPage($mark)
		foreach tag [$w tag names $start] {
		    $w tag remove $tag $start $mark
		}
		foreach tag $list {
		    $w tag add $tag $start $mark
		}
		$w mark unset $mark
	    }
    }
}

## **********************************
## New list addtion procedures by PER
## **********************************

proc tkW3EditGetListLevel {w {tabvar {}} {numvar {}}} {
  global tkW3HtText

  set line [$w get "insert linestart" "insert lineend"]
  if [regexp "^$tkW3HtText(inittab)(\t*)(\[0-9\]*)" $line trash tabbing num] {
    set level [expr [string length $tabbing]+1]
  } else { 
    set level 1
    set tabbing {}
    set num {}
  }
  if {[string length $tabvar]} {
    uplevel "set $tabvar {$tabbing}"
  }
  if {[string length $numvar]} {
    uplevel "set $numvar {$num}"
  }
  return $level
}

proc tkW3EditRenumberList {w level {amt 1}} {
  global tkW3HtText
  
  $w mark set renumber insert
  while {[$w compare insert < end]} {
    $w mark set insert [$w index "insert linestart +1 line"]
    if {[lsearch [$w tag names insert] LI] == -1} break
    if {[lsearch [$w tag names insert] type.OL] == -1} {
      if {[tkW3EditGetListLevel $w] <= $level} break
      continue
    }
    set linenum [lindex [split [$w index insert] .] 0]
    set line [$w get "insert linestart" "insert lineend"]
    if [regexp -indices "^$tkW3HtText(inittab)(\t*)(\[0-9\]+)" \
	    $line trash tabbing numndx] {
      set curlevel [expr [lindex $tabbing 1]-[lindex $tabbing 0]+2]
    } else { break }
    if {$curlevel < $level} break
    if {$curlevel == $level} {
      set num [string range $line [lindex $numndx 0] [lindex $numndx 1]]
      incr num $amt
      $w insert $linenum.[expr [lindex $numndx 1]+1] $num
      $w delete $linenum.[lindex $numndx 0] $linenum.[expr [lindex $numndx 1]+1]
    }
  }
  $w mark set insert renumber
}

proc tkW3EditAddListItem {w {before 0}} {
  global tkW3HtText

  # make sure there is at least one blank line after the current one
  if {[$w compare "insert lineend" == end]} {
    set curndx [$w index insert]
    tkW3HtInsert "\n" blank
    $w mark set insert $curndx
  }

  $w mark set insert [$w index "insert linestart"]
  set taglist [$w tag names insert]
  if {[lsearch $taglist LI] == -1} {
    puts stderr "\007"
    return
  }

  if {[set ndx [lsearch $taglist type.*]] > -1} {
    set type [lindex $taglist $ndx]
  } else {
    set type "type.UL"
  }

  set level [tkW3EditGetListLevel $w tabbing num]
  
  if {!$before} {
    while {[$w compare insert < end]} {
      $w mark set insert [$w index "insert +1 line"]
      if {[lsearch [$w tag names insert] LI] == -1} break
      if {[tkW3EditGetListLevel $w] <= $level} break
    }
    $w mark set insert [$w index "insert linestart -1c"]
    tkW3HtInsert "\n" blank
  } else {
    tkW3HtInsert "\n" blank
    $w mark set insert [$w index "insert -1 line"]
  }

  if {$type == "type.OL"} {
    if {![string length $num]} {set num 1}
    if {!$before} {incr num}
    tkW3HtInsert "$tkW3HtText(inittab)$tabbing$num " "LI $type"
  } else {
    tkW3HtInsert "$tkW3HtText(inittab)$tabbing* " "LI $type"
  }
  if {[lsearch $taglist type.OL] != -1} {
    tkW3EditRenumberList $w $level
  }

  $w yview -pickplace insert
  tkW3EditUseStyle $w
  tkW3EditModifiedSet
}

proc tkW3EditAddNewList {w {type UL}} {
  global tkW3HtText

  # make sure there is at least one blank line after the current one
  if {[$w compare "insert lineend" == end]} {
    set curndx [$w index insert]
    tkW3HtInsert "\n" blank
    $w mark set insert $curndx
  }

  $w mark set insert [$w index "insert linestart"]
  set taglist [$w tag names insert]
  if {[lsearch $taglist LI] == -1} {
    set tabbing {}
  } else { 
    set level [tkW3EditGetListLevel $w tabbing]
    append tabbing \t
  }
  
  $w mark set insert [$w index "insert lineend +1c"]
  set taglist [$w tag names insert]
  if {[lsearch $taglist P] != -1} {
    $w mark set insert [$w index "insert lineend +1c"]
  }
  if {$type == "OL"} {
    tkW3HtInsert "$tkW3HtText(inittab)${tabbing}1 " "LI type.$type"
  } else {
    tkW3HtInsert "$tkW3HtText(inittab)$tabbing* " "LI type.$type"
  }
  set final [$w index insert]
  tkW3HtInsert "\n" blank
  if {![string length $tabbing]} {
    tkW3HtInsert "\n" LIend
  }
  $w mark set insert $final

  $w yview -pickplace insert
  tkW3EditUseStyle $w
  tkW3EditModifiedSet
}

# insert new list item after parent of current item
proc tkW3EditOutListItem { w } {

  set curlvl [tkW3EditGetListLevel $w]

  $w mark set insert "insert linestart -1 line"
  set taglist [$w tag names "insert linestart"]
  while {[lsearch $taglist LI] != -1 &&
	 [tkW3EditGetListLevel $w] >= $curlvl} {
    if {[$w compare "insert linestart" == 1.0]} {
      set taglist {}
      break
    }
    $w mark set insert "insert linestart +1 line"
    set taglist [$w tag names "insert linestart"]
  }

  if {[lsearch $taglist LI] != -1} {
    tkW3EditAddListItem $w 0
  } else {
    tkW3EditEndOfList $w
  }
}

# put insert cursor after current list
proc tkW3EditEndOfList { w } {
  set taglist [$w tag names "insert linestart"]
  while {[lsearch $taglist LI] != -1} {
    $w mark set insert "insert linestart +1 line"
    set taglist [$w tag names "insert linestart"]
    if {[$w compare "insert lineend" == end]} {
      break
    }
  }
  if {[lsearch $taglist LIend] != -1} {
    $w mark set insert "insert linestart +1 line"
  }
  if {[$w compare "insert lineend" == end]} {
    $w mark set insert end
    tkW3HtInsert "\n" blank
  }
}
