## file.tcl file menu items
## ==============
## Copyright (C) 1992-1994
## Globewide Network Academy
## Macvicar School of Education and Technology
##
## See the file COPYRIGHT for conditions

proc tkW3FileInitialize {} {
    global tkW3ConfigViewer
    tixEntryDialog .mail_dialog \
	-title "Enter the To and Subject fields" \
	-prompt {"To:" "Subject:"} \
	-command "tkW3NavigateMailText"
    tixEntryDialog .find_dialog \
	-title "Enter the keywords to find" \
	-prompt	{"Find Keywords:"} \
	-command "tkW3NavigateFind"
    tixEntryDialog .goto_dialog \
	-title "Enter the link you wish to go to" \
	-prompt {"Goto Page:"} \
	-command "tkW3NavigateRecordAndGoto"
    tixEntryDialog .print_dialog \
	-title "Enter the command to print with" \
	-prompt {"Print Command:"} \
	-command "tkW3NavigatePrintUsingCommand"
    tixEntryDialog .tcl_command \
	-title "Run TCL Command" \
	-prompt {"Tcl Command:"} \
	-command "eval"
    .print_dialog set_value 1 $tkW3ConfigViewer(printer)
    tixFileSelectDialog .open_file -command tkW3FileDoOpenFile
    tixFileSelectDialog .save_text -command tkW3NavigateSaveText
    tixFileSelectDialog .save_html 
    tixFileSelectDialog .move_file
    tixFileSelectDialog .chdir -command cd
}


proc tkW3FileNewPage {{address ""}} {
    global tkW3HtPage
    if {[tkW3EditModifiedCheck] == 0} {
	return
    }
    
    tkW3HtBeginDoc
    set tkW3HtPage(address) $address
    set tkW3HtPage(title) ""
    tkW3HtEndDoc
    tkW3EditModifiedSet 0
    tkW3EditQueryChangeTitle .f.msg
}

proc tkW3FileOpenFile {} {
    .open_file popup
}

proc tkW3FileDoOpenFile {file} {
    tkW3NavigateGoto "file:$file"
}


proc tkW3FileOpenLink {} {
    .goto_dialog popup
}

proc tkW3FileQueryCloseWindow {} {
    if {[DLG:msg \
	 "Are you sure you want to close this window?" question "Yes" "No"] \
	     == 1} {
	     tkW3NavigateCloseThisWindow
	 }
}

proc tkW3FileQueryExit {} {
    if {[DLG:msg \
	 "Are you sure you want to close ALL tkWWW windows?" question \
	     "Yes" "No"] == 1} {
	     tkW3NavigateCloseAllWindows
	 }
}


proc tkW3FileSave {{saveas 0}} {
    global tkW3HtPage

    switch -- $tkW3HtPage(type) {
	{text/html} {
	    tkW3EditGenerate 0 .f.msg .edit.main.t
	    tkW3FileSaveTextWidget .edit.main.t $saveas
	}
	{default} {
	    tkW3FileSaveTextWidget .f.msg $saveas
	}
    }
}   

proc tkW3FileSaveAs {} {
    tkW3FileSave 1
}   
    
proc tkW3FileSaveText {} {
    .save_text popup
}

proc tkW3FileRunTclCommand {} {
     .tcl_command popup
}

proc tkW3FileChangeDirectory {} {
    .cd popup
}


## ********************
## Save Generated HTML to a file
## ********************

proc tkW3FileSaveTextWidget {w {saveas 0}} {
    global fsBox
    regexp {^([^:]+)*:(.*)$} [tkW3NavigateGetAddress] {} type name 

    set file [file tail $name]
    set fsBox(path) ""

    if {"$type" == "file" && ![string match $name "//*"]} {
	set fsBox(path) [file dirname $name]
	if {$saveas == 0} {
	    tkW3EditClose
	    tkW3FileOutputTextWidget $w $fsBox(path)/$file $saveas
	    return
	}
    }

    tkW3EditClose
    .save_html configure -command "tkW3FileOutputTextWidget $w $saveas"
    .save_html popup
}

proc tkW3FileOutputTextWidget {w filename {saveas 0}} {
    set dir [file dirname $filename]
    set has_rcs [tkW3rcs file_exists $filename]
 
    if {![file exists $dir]} {
        if {[DLG:msg "directory $dir does not exist" question \
                "Create" "Cancel"] == 1} {
            exec mkdir $dir
        } else {
            return
        }
    }

     if {[file exists $filename]} {
         if {[DLG:msg "File $filename already exists." question \
                 "Ovewrite" "Cancel"] != 1} {
             return
         }
     }
  


    # If file is not writable and is RCS'ed attempt to check out
    if {![file writable $filename] && $has_rcs} {
	if [tkW3OutputPrompt "RCS file exists, attempt to checkout?" \
	    "Check out file"] {
		 tkW3rcs checkout $filename
	     } {
		 tkW3OutputError "Checkout aborted"
	     }
    }

    if {[catch "open $filename w" file] != 0} {
        tkW3OutputError "couldn't open $filename for writing"
        return
    }
    puts $file [$w get 1.0 end]
    close $file

    # file is saved:
    tkW3EditModifiedSet 0

    # Ask user about checking file into RCS
    if {$has_rcs || [tkW3rcs dir_exists $filename]} {
	if [tkW3OutputPrompt "Attempt to check file into RCS?" \
	    "Check in file"] {
		 tkW3rcs checkin $filename
	     }
    }

    # RELOAD:
    # make sure, file and view are identical (this is important for
    # page caching, which would otherwise keep the old page).
    if {$saveas} {
        tkW3NavigateRecordAndGoto file:$filename
    } {
        tkW3NavigateReload
    }
}



proc tkW3FileMoveFile {file_name address} {
    tkW3OutputClearBody
    set name [file tail $address]
    .move_file configure -command "exec mv $file_name"
    .move_file show
}
