## hypertext.tcl Procedures for processing the hypertext
## ==============
## Copyright (C) 1992-1995
## Globewide Network Academy
## Macvicar School of Education and Technology
##
## See the file COPYRIGHT for conditions

## Conventions:
## The procedures in this procedure have names which begin with tkW3Ht

# This file contains TCL procedures which are executed by the application
# wwwish

# tkW3HtPage is an array that contains characteristics of the hypertext page
# It is used throughout the application, and should be the only connection
# between this file and the rest of the application

# tkW3HtText is an array that contains internal variable used to create the
# page.  tkW3HtText should not be called outside of this file.

# Set initial values for the array.  These values are set when this file is
# autoloaded.

set tkW3HtText(image.max) 0
set tkW3HtPage(is_index) 0
set tkW3HtPage(base) ""
set tkW3HtPage(modified) 0
set tkW3HtPage(type) ""
set tkW3HtPage(encoding) ""
set tkW3HtPage(has_forms) {}

# Set entities
tkW3ConversionClass tkw3entities -list {
    { "lt" "<" }
    { "gt" ">" }
    { "amp" "&" }
    { "Agrave" "\x0c0" }
    { "Aacute" "\x0c1" }
    { "Acirc" "\x0c2" }
    { "Atilde" "\x0c3" }
    { "Auml" "\x0c4" }
    { "Aring" "\x0c5" }
    { "AElig" "\x0c6" }
    { "Ccedil" "\x0c7" }
    { "Egrave" "\x0c8" }
    { "Eacute" "\x0c9" }
    { "Ecirc" "\x0ca" }
    { "Euml" "\x0cb" }
    { "Igrave" "\x0cc" }
    { "Iacute" "\x0cd" }
    { "Icirc" "\x0ce" }
    { "Iuml" "\x0cf" }
    { "ETH" "\x0d0" }
    { "Ntilde" "\x0d1" }
    { "Ograve" "\x0d2" }
    { "Oacute" "\x0d3" }
    { "Ocirc" "\x0d4" }
    { "Otilde" "\x0d5" }
    { "Ouml" "\x0d6" }
    { "Oslash" "\x0d8" }
    { "Ugrave" "\x0d9" }
    { "Uacute" "\x0da" }
    { "Ucirc" "\x0db" }
    { "Uuml" "\x0dc" }
    { "Yacute" "\x0dd" }
    { "THORN" "\x0de" }
    { "szlig" "\x0df" }
    { "agrave" "\x0e0" }
    { "aacute" "\x0e1" }
    { "acirc" "\x0e2" }
    { "atilde" "\x0e3" }
    { "auml" "\x0e4" }
    { "aring" "\x0e5" }
    { "aelig" "\x0e6" }
    { "ccedil" "\x0e7" }
    { "egrave" "\x0e8" }
    { "eacute" "\x0e9" }
    { "ecirc" "\x0ea" }
    { "euml" "\x0eb" }
    { "igrave" "\x0ec" }
    { "iacute" "\x0ed" }
    { "icirc" "\x0ee" }
    { "iuml" "\x0ef" }
    { "eth" "\x0f0" }
    { "ntilde" "\x0f1" }
    { "ograve" "\x0f2" }
    { "oacute" "\x0f3" }
    { "ocirc" "\x0f4" }
    { "otilde" "\x0f5" }
    { "ouml" "\x0f6" }
    { "oslash" "\x0f8" }
    { "ugrave" "\x0f9" }
    { "uacute" "\x0fa" }
    { "ucirc" "\x0fb" }
    { "uuml" "\x0fc" }
    { "yacute" "\x0fd" }
    { "thorn" "\x0fe" }
    { "yuml" "\x0ff" }
}


# ADDED BY PAUL RAINES
set tkW3HtText(b2time) 0
set tkW3HtText(depressed) {}
set tkW3HtText(inittab) \t

# The next procedure is a big bottleneck in tkWWW
# It should be rewriiten in C
proc tkW3HtWidgetInsert {widget {list ""}} {
#    set w .f.msg
#     set start [$w index insert]
     .f.msg window create insert -window $widget
#     foreach tag [$w tag names $start] {
#         $w tag remove $tag $start insert
#     }
#     foreach tag $list {
# 	$w tag add $tag $start insert
#     }
 }

proc tkW3HtInsert {text {list ""}} {
    .f.msg insert insert $text $list
}

proc tkW3HtSetAddress {address} {
    global tkW3HtPage
    set tkW3HtPage(address) $address
    if {$tkW3HtPage(base) == ""} {
	set tkW3HtPage(base) $tkW3HtPage(address)
    }
}
 
proc tkW3HtBeginDoc {} {
    global tkW3HtPage tkW3HtText


    unset tkW3HtPage
    set tkW3HtPage(anchor.index) 1
    set tkW3HtPage(image.index) 1
    set tkW3HtPage(is_index) 0
    set tkW3HtPage(modified) 0
    set tkW3HtPage(base) ""
    set tkW3HtPage(base.use) 0
    set tkW3HtPage(next_id) ""
    set tkW3HtPage(index.href) ""
    set tkW3HtPage(link) ""
    set tkW3HtPage(type) "text/html"
    set tkW3HtPage(encoding) "8bit"

    set tkW3HtText(in_anchor) 0
    set tkW3HtText(current_anchor) ""
    set tkW3HtText(list.stack) {}
    set tkW3HtText(para.stack) {}
    set tkW3HtText(char.stack) {}
    set tkW3HtText(background) [.f.msg cget -background]
    set tkW3HtText(target) ""
    set tkW3HtText(in_title) 0
    set tkW3HtText(foreground)  #00000000a5a5

  global tkW3NumList
  set tkW3NumList {}

    tkW3OutputClearBody
}


proc tkW3HtAdd {string {styles ""}} {
    global tkW3HtText tkW3HtPage
    if $tkW3HtText(in_title) {
	append tkW3HtPage(title) $string
	return
    }

    if $tkW3HtText(in_anchor) {
	lappend styles Anchor
	lappend styles h$tkW3HtPage(anchor.index)
    }

    lappend styles [tkW3HtListLast tkW3HtText(para.stack)]
  

    if [llength $tkW3HtText(char.stack)] {
	lappend styles [tkW3HtListLast tkW3HtText(char.stack)]
    }

  # add spaces around an Anchor
  if {[lsearch $styles Anchor] > -1} {
    set string " $string "
  }

    tkW3HtInsert $string $styles
}

proc tkW3HtBegin {stack elem} {
    global tkW3HtText
    
    switch -regexp -- $elem {
	{^H[1-9]+$} -
	{^ADDRESS$} {
	    tkW3HtBlankLines 2
	}
        {^XMP$} -
	{^PRE$} {
	    tkW3HtBlankLines 1
	}
	{^Q$} {
	    tkW3HtInsert "``" blank
	}
    }
    lappend tkW3HtText($stack.stack) $elem

  if {$stack == "list" && $elem == "OL"} {
    global tkW3NumList
    lappend tkW3NumList 0
  }
  
}

proc tkW3HtEnd {stack elem} {
    global tkW3HtText tkW3NumList

    tkW3HtListPop tkW3HtText($stack.stack)

    switch -regexp -- $elem {
	{^H[1-9]+$} -
	{^ADDRESS$} {
	    tkW3HtBlankLines 1
	}
	{^Q$} {
	    tkW3HtInsert "''" blank
	}
    }

    if {$stack == "list" && $tkW3HtText(list.stack) == {}} {
	tkW3HtBlankLines 1
	tkW3HtInsert "\n" LIend
    }

  if {$stack == "list" && $elem == "OL"} {
    global tkW3NumList
    tkW3HtListPop tkW3NumList
  }
}

proc tkW3HtEndDoc {} {
    update idletasks
}

proc tkW3HtAddBul {type} {
    global tkW3HtText

    switch -exact -- $type {
	BR {
	    tkW3HtInsert "\n" BR
	    return
	}

	HR {
	    tkW3HtInsert "\n_______________________________________________________________________________\n" HR
	    return
	}

	P {
	    if {[lindex [split [.f.msg index insert] .] 1] == 0} {
	      tkW3HtInsert "\n" P
	    } else {
	      tkW3HtInsert "\n\n" P
	    }
	    return
	}
	
	DT {
	    tkW3HtBlankLines 1
	}
    }

    if {[lindex [split [.f.msg index insert] .] 1] != 0} {
      tkW3HtInsert "\n" blank
    }

    set tabbing $tkW3HtText(inittab)
    for {set i 1} {$i < [llength $tkW3HtText(list.stack)]} {incr i} {
        append tabbing "\t"
    }

    case $type {
	"LI" {

	  set litype [tkW3HtListLast tkW3HtText(list.stack)]
	  if {$litype == "OL"} {
	    global tkW3NumList
	    set num [tkW3HtListPop tkW3NumList]
	    lappend tkW3NumList [incr num]
	    # tkW3HtInsert "$tabbing[join $tkW3NumList .] " "LI type.$litype"
	    tkW3HtInsert "$tabbing$num " "LI type.$litype"
	  } else {
	    tkW3HtInsert "$tabbing* " "LI type.$litype"
	  }

	}
	"DD" {
	    tkW3HtInsert "$tabbing   * " "DD"
	}
	"DT" {
	    tkW3HtInsert "$tabbing* " "LI type.DL"
	}
    }
}


proc tkW3HtBlankLines {n} {
    set text [regexp {^[ ]*$} \
	      [.f.msg get {insert linestart} {insert - 1 char lineend}]]
    if {!$text} {
	for {set i 0} {$i<$n} {incr i} {
	    tkW3HtInsert "\n" blank
	}
    }
}

proc tkW3HtListPop {in_list} {
    upvar $in_list list
    set index [llength $list]
    incr index -1
    set item [lindex $list $index]
    incr index -1
    set list [lrange $list 0 $index]
    return $item
}

proc tkW3HtListLast {in_list} {
    upvar $in_list list
    set index [llength $list]
    incr index -1
    lindex $list $index
}

proc tkW3HtButtonPress {w loc b tm} {
    global tkW3HtPage tkW3HtText
    set tag_list [.f.msg tag names $loc]
    set index [lsearch -regexp $tag_list  {^i[0-9]+$}]
 
    if {$index == -1} {
	set index [lsearch -regexp $tag_list  {^h[0-9]+$}]
    }
    if {$index != -1} {
	# undepress any depressed tag
	if {[string length $tkW3HtText(depressed)]} {
	    $w tag configure $tkW3HtText(depressed) -relief raised
	    set $tkW3HtText(depressed) {}
	    $w tag raise sel
	    update idletasks
	}
        set tkW3HtText(depressed) [lindex $tag_list $index]
	$w tag configure [lindex $tag_list $index] -relief sunken
        tkW3OutputSetMessage "" 
	update idletasks
    }
    if {$b == 2} {
      set y [lindex [split $loc ,] 1]
      $w scan mark $y
      set tkW3HtText(b2time) $tm
    }
}

proc tkW3HtButtonRelease {w loc b tm} {
    global tkW3HtPage tkW3HtText
    set tag_list [.f.msg tag names $loc]

    # undepress any depressed tag
    if {[string length $tkW3HtText(depressed)]} {
	$w tag configure $tkW3HtText(depressed) -relief raised
        set $tkW3HtText(depressed) {}
        $w tag raise sel
	update idletasks
    }

    # Search for active tag
    # image tags override hypertext anchor tags 
    set index [lsearch -regexp $tag_list  {^i[0-9]+$}]
    if {$index == -1} {
	set index [lsearch -regexp $tag_list  {^h[0-9]+$}]
    }

    if {$index != -1} {
	set tag [lindex $tag_list $index]
	$w tag configure $tag -relief raised
	update idletasks
	regexp {([hi])([0-9]+)} $tag {} tag_type i
	switch -- $b {
	    "1" {
		tkW3NavigateRecordAndGoto $tkW3HtPage(anchor.href.$i)
	    }
	    "2" {
		tkW3NavigateClone $tkW3HtPage(anchor.href.$i)
	    }
	    "3" {
		tkW3EditSetupAnchorDialog $w $i
	    }
	}
    } else {
	if {$b == 2 && [expr $tm-$tkW3HtText(b2time)]<500 &&
	    ![catch {selection get} s]} {
		tkW3EditInsert $w $s
	    }
    }
}

proc tkW3HtProgress {msg} {
    tkW3OutputSetMessage $msg
    update idletasks
}

proc tkW3HtAlert {msg} {
    if {[regexp {404} $msg] &&
	[regexp {file:[^ 
]+} $msg page_name ]} {
	    if {[DLG:msg "$msg
Do you wish to create $page_name?" question "Yes" "No"]==1} {
    tkW3FileNewPage $page_name
} {
    tkW3HistoryRestore
}
	    return
	}
	# Undo any changes we have made to the history list
    tkW3HistoryRestore
    # Tell the user about the error
    tkW3OutputError $msg
}

proc tkW3HtStartElement {tag args} {
    global tkW3HtPage tkW3HtText tkW3ImageCache
    tkW3HtHandleOptions attr $args
    puts stdout "Start: $tag"
    switch -- $tag {
	A {
	    set name ""
	    set i $tkW3HtPage(anchor.index)
	    set tkW3HtText(in_anchor) 1
	    set tkW3HtText(current_anchor) ""
	    set tkW3HtPage(anchor.name.$i) ""
	    set tkW3HtPage(anchor.href.$i) ""

	    if [info exists attr(-NAME)] {
		set tkW3HtPage(anchor.name.$i) $attr(-NAME)
	    }
	    if [info exists attr(-ID)] {
		set tkW3HtPage(anchor.href.$i) $attr(-ID)
	    }
	    if [info exists attr(-HREF)] {
		set tkW3HtPage(anchor.href.$i) $attr(-HREF)
	    }

	    set tkW3HtText(current_anchor) $tkW3HtPage(anchor.href.$i)
	    set tkW3HtPage(id.$name) h$i
	    .f.msg tag configure h$i -relief raised -borderwidth 2 \
		-background $tkW3HtText(background)

	}
	IMG {
	    if [info exists attr(-SRC)] {
		set i $tkW3HtPage(image.index)
		set w .f.msg.image$i


		set tkW3HtPage(image.$i) $attr(-SRC)
		set address \
		    [HtParseName $tkW3HtPage(image.$i) $tkW3HtPage(base)]
		set anchor $tkW3HtText(current_anchor)

		if [info exists tkW3ImageCache($address)] {
		    update idletasks
		    button $w -image $tkW3ImageCache($address)
		}  else {
		    button $w
		    update idletasks
		    HtLoad $address $w
		}

		if {$anchor != ""} {
		    if [info exists attr(-ISMAP)] {
			$w config -command \
			    "tkW3NavigateRecordAndGoto $anchor?%x,%y"
		    } {
			$w config -command \
			    "tkW3NavigateRecordAndGoto $anchor"
		    }
		}

		tkW3HtWidgetInsert $w
		incr tkW3HtPage(image.index)
	    }
	}
	TITLE {
	    set tkW3HtText(in_title) 1
	}
	NEXTID {
	    if [info exists attr(-N)] {
		set tkW3HtPage(next_id) $attr(-N)
	    }
	}
	BASE {
	   if [info exists attr(-HREF)] {
	       set tkW3HtPage(base.use) 1
	       set tkW3HtPage(base) $attr(-HREF)
	   }
       }
	ISINDEX {
	    set i $tkW3HtText(form_count.isindex)
	    set w .f.msg.isindex$i
	    entry $w -width $size -relief sunken 
	    bind $w <Return> {eval tkW3NavigateFind [%W get] }
	    tkW3HtInsert \
		"This is a searchable index. Enter search keywords:" blank
	    tkW3HtWidgetInsert $w
	    incr tkW3HtText(form_count.isindex)

	    set tkW3HtPage(is_index) 1
	    set tkW3HtPage(index.href) $href
	}
	DL -
	UL -
	OL -
	MENU -
	DIR {
	    tkW3HtBegin list $tag
	}
	DT -
	DD -
	LI -
	P -
	BR -
	HR {
	    tkW3HtAddBul $tag
	}
	LISTING -
	XMP -
	PLAINTEXT -
	PRE -
	LIT -
	ABSTRACT -
	BYLINE -
	H1 -
	H2 -
	H3 -
	H4 -
	H5 -
	H6 -
	H7 -
	ADDRESS -
	BLOCKQUOTE -
	FOOTNOTE -
	MARGIN {
	    tkW3HtBegin para $tag
	}
	TT -
	B -
	I -
	U -
	EM -
	STRONG -
	CODE -
	SAMP -
	KBD -
	VAR -
	DFN -
	CITE -
	Q -
	PERSON -
	ACRONYM -
	ABBREV -
	CMD -
	ARG {
	    tkW3HtBegin char $tag
	}
	LINK {
	    if {[info exists attr(-REL)] && [info exists attr(-HREF)]} {
		lappend tkW3HtPage(link) $attr(-REL)
		set tkW3HtPage(link.$attr(-REL)) $attr(-HREF)
	    }
	}
	FORM {
	    set i $tkW3HtPage(form.count)
	    set tkW3HtPage(form.$i.action) $attr(-ACTION)
	    set tkW3HtPage(form.$i.method) $attr(-METHOD)
	    incr $tkW3HtPage(form.count)
	}
    }
}

proc tkW3HtEndElement {tag args} {
    global tkW3HtPage tkW3HtText
    puts stdout "End: $tag"
    switch -- $tag {
	TITLE {
	    set tkW3HtText(in_title) 0
	}
	A {
	    set tkW3HtText(in_anchor) 0
	    incr tkW3HtPage(anchor.index)
	}
	TT -
	B -
	I -
	U -
	EM -
	STRONG -
	CODE -
	SAMP -
	KBD -
	VAR -
	DFN -
	CITE -
	Q -
	PERSON -
	ACRONYM -
	ABBREV -
	CMD -
	ARG {
	    tkW3HtEnd char $tag
	}
	DL -
	UL -
	OL -
	MENU -
	DIR {
	    tkW3HtEnd list $tag
	}
	LISTING -
	XMP -
	PLAINTEXT -
	PRE -
	LIT -
	ABSTRACT -
	BYLINE -
	H1 -
	H2 -
	H3 -
	H4 -
	H5 -
	H6 -
	H7 -
	ADDRESS -
	BLOCKQUOTE -
	FOOTNOTE -
	MARGIN {
	    tkW3HtEnd para $tag
	}
	FORM {
	}
	INPUT {
	}
    }
}

proc tkW3HtAddEntity {entity} {
    if [tkw3entities in_source $entity] {
	tkW3HtInsert [tkw3entities forward $entity]
    } else {
	puts stdout "ENTITY: $entity"
    }
}

proc tkW3HtHandleOptions {record arglist} {
    upvar $record data
    set len [expr [llength $arglist]-2 ]
    set i 0

    while {$i <= $len} {
	set option [lindex $arglist $i]
	incr i
	set arg [lindex $arglist $i]
	incr i
	set data($option) $arg
    }
}
