## output.tcl: output for tkWWW user interface
## ==============
## Copyright (C) 1992-1994
## Globewide Network Academy
## Macvicar School of Educational and Technology
##
## See the file COPYRIGHT for conditions

# All output should go through these procedures

proc tkW3OutputInitialize {} {
}

proc tkW3OutputSetMessage {message {duration ""}} {
    tkW3OutputEntryPrint .message $message
    if {$duration != ""} {
	after $duration {tkW3OutputSetMessage {}}
    }
}

proc tkW3OutputEntryPrint {w message} {
    $w configure -state normal
    $w delete 0 end
    $w insert 0 $message
    $w configure -state disabled
}

proc tkW3OutputSetBodyToFile {filename} {
    tkW3OutputClearBody
    set file [open $filename "r"]
    while {[gets $file line] != -1} {
	.f.msg insert insert "$line\n"
    }
    close $file
    .f.msg tag add PRE 1.0 end
}

proc tkW3OutputGetScrollPosition {} {
    lindex [.vs get] 2
}

proc tkW3OutputSetScrollPosition {value} {
    .f.msg yview $value
}

## ************************
## Cursor procedures
## ************************

proc tkW3OutputCursorWait {} {
  .f.msg configure -cursor watch
}

proc tkW3OutputCursorNormal {} {
  .f.msg configure -cursor xterm
}

proc tkW3OutputMakeFrame {w} {
    if [winfo exists $w] {
	foreach child [winfo children $w] {
	    destroy $child
	}
    } {
	frame $w -borderwidth 0
    }
}


proc tkW3OutputMakeButtons {w list} {
    tkW3OutputMakeFrame $w
    foreach item $list {
	pack append $w \
	    [button $w.[lindex $item 0] -padx 0 \
	     -text [lindex $item 1] -command [lindex $item 2]] \
	    {expand fill left}
    }
    return $w
}

proc tkW3OutputDoNothing {} {
}

proc tkW3OutputMakeToggles {w list {static 1} {pos right}} {
    tkW3OutputMakeFrame $w
    foreach item $list {
	set name [lindex $item 0]
	pack append $w \
	    [ checkbutton $w.$name -text [lindex $item 1] -variable $name \
-relief flat] \
	    $pos
	if {$static} {
	    bind $w.$name <Any-Enter> "tkW3OutputDoNothing"
	    bind $w.$name <Any-Leave> "tkW3OutputDoNothing"
	    bind $w.$name <1> "tkW3OutputDoNothing"
	    bind $w.$name <ButtonRelease-1> "tkW3OutputDoNothing"
	}
    }
    if {$static} {
	pack append $w \
	    [label .message_label -text "Message:"] {left} \
	    [entry .message -state disabled] {left  expand fillx}
    }
    return $w
}

proc tkW3OutputMakeText {w} {
    frame $w -relief flat -borderwidth 5

    pack [ text $w.msg -yscrollcommand { .vs set } \
	  -height 40 ] -expand 1 -fill both -padx 2 -pady 2 -side top
    pack before $w.msg \
	[scrollbar .vs \
	 -command "$w.msg yview" -orient vertical] { right filly }
    return $w
}

proc tkW3OutputMakeScrollbars {} {
}

proc tkW3OutputClearBody {} {
    .f.msg delete 1.0 end
    foreach w [winfo children .f.msg] {
	destroy $w
    }
}

proc tkW3OutputText {msg} {
    .f.msg insert insert "$msg\n"
}

proc tkW3OutputToggleSet {w var} {
    if {$var == 0} {
	$w deselect
    } {
	$w select
    }
}

proc tkW3OutputMenuSetSensitive {w entry var} {
    if {$var == 0} {
	$w entryconfigure $entry -state disabled
    } {
	$w entryconfigure $entry -state normal
    }
}

proc tkW3OutputButtonSetSensitive {w var} {
    if {$var == 0} {
	$w configure -state disabled
    } {
	$w configure -state normal
    }
}

proc tkW3OutputError {message} {
    DLG:msg $message error "OK"
}

proc tkW3OutputPrompt {message action {help ""}} {
    DLG:msg $message question $action "Cancel"
}


proc tkW3OutputMail {to {subject ""}} {
    set w .f.mail
    if ![winfo exists $w] {
	frame $w
	pack append $w \
            [DLG:draw_entries $w.frame {To: Subject:} ] {top fill} \
	    [frame $w.f -relief raised] {top fill expand} \
	    [tixDlgBtns $w.btns] {fill}

	pack append $w.f \
	    [label $w.label \
	     -text "Message:" ] { top } \
	    [scrollbar $w.vs -command "$w.text yview" -orient vertical ] \
	    { right filly expand } \
	    [text $w.text -yscrollcommand "$w.vs set" -height 10] \
	      {fill top expand} 
	    

	$w.btns add ok -text "Send Mail" -width 6
	[$w.btns button ok] config -command "tkW3ConfigSendMail \
        \[DLG:get_entry_value $w 1\] \[DLG:get_entry_value $w 2\] \
        \[$w.text get 0.0 end\]"
    }
    place .f.mail -in .f -relx 0.0 -rely 0.0 -relwidth 1.0 -relheight 1.0
    raise .f.mail
    DLG:set_entry_value $w 1 $to
    DLG:set_entry_value $w 2 $subject
}

tixWidgetClass tkW3OutputStyleBarClass {
    -superclass tixPrimitive
    -classname TkW3OutputStyleBar 
    -shortname tkW3Osb
    -method {print get}
    -flag {
	-entries -combos -entry_width -orientation
    }
    -configspec {
	{-entries entries Entries ""}
	{-combos combos Combos ""}
	{-entry_width entryWidth EntryWidth 8}
	{-orientation orientation Orientation horizontal}
    }
}

proc tkW3Osb::ConstructWidget {w} {
    upvar #0 $w data

    set data(entries_list) ""
    set data(combos_list) ""

    if {$data(-orientation) == "horizontal"} {
	set flags {expand fill left}
    } {
	set flags {expand fill bottom}
    }

    foreach item $data(-entries) {
	set name [lindex $item 0]
	lappend data(entries_list) $name

	pack append $w \
	    [tixLabelFrame $w.$name -label [lindex $item 1]] $flags
	    
 	pack append [$w.$name frame] \
	    [entry $w.$name.entry -width $data(-entry_width)] {expand fill}
	set data(w:$name) $w.$name.entry
     }

    foreach item $data(-combos) {
	set name [lindex $item 0]
	lappend data(combos_list) $name

	pack append $w \
	    [tixLabelFrame $w.$name -label [lindex $item 1]] $flags

	set data(w:$name) [tixCombobox $w.$name.combo]
	$w.$name.combo entry configure -width $data(-entry_width)
 	pack append [$w.$name frame] $w.$name.combo {expand fill}
    } 
    return $w
}

proc tkW3Osb::print {w item text} {
    upvar #0 $w data
    if {[lsearch $data(combos_list) $item] != -1} {
	tkW3OutputEntryPrint [$w.$item.combo entry] $text
    } elseif {[lsearch $data(entries_list) $item] != -1} {
	tkW3OutputEntryPrint $w.$item.entry $text
    }
}



