## tix.tcl: history for tkWWW  user interface
## ==============
## Copyright (C) 1993
## Globewide Network Academy
## MacVicar Institute for Educational Software Development
##
## See the file COPYRIGHT for conditions

## This file contains things that should be bug fixes to TiX

proc tixInitialize {} {
}

tixClass tkW3ConversionClass {
    -superclass {}
    -classname TkW3ConversionClass
    -shortname tkW3Conversion
    -method {forward back in_source in_dest source_list dest_list} 
    -flag {-list}
    -configspec {
	{-list ""}
    }
}

proc tkW3Conversion::Constructor {w} {
    upvar #0 $w data
    
    foreach item $data(-list) {
	set item1 [lindex $item 0]
	set item2 [lindex $item 1]
	set data(forward.$item1) $item2
	set data(back.$item2) $item1
    }
}

proc tkW3Conversion::forward {w item} {
    upvar #0 $w data
    set data(forward.$item)
}

proc tkW3Conversion::back {w item} {
    upvar #0 $w data
    set data(back.$item)
}

proc tkW3Conversion::in_source {w item} {
    upvar #0 $w data
    info exists data(forward.$item)
}

proc tkW3Conversion::in_dest {w item} {
    upvar #0 $w data
    info exists data(back.$item)
}

proc tkW3Conversion::source_list {w} {
    upvar #0 $w data
    set return_string ""
    foreach item $data(-list) {
	lappend return_string [lindex $item 0]
    }
    return $return_string
}

proc tkW3Conversion::dest_list {w} {
    upvar #0 $w data
    set return_string ""
    foreach item $data(-list) {
	lappend return_string [lindex $item 1]
    }
    return $return_string
}


tixWidgetClass tixEntryDialog {
    -superclass tixStdDialogShell
    -classname  TixEntryDialog
    -shortname  tixED
    -method {
	get_values set_value bind_entry
    }
    -flag {
	-command -prompt -title
    }
    -configspec {
	{-command command Command {}}
	{-prompt prompt Prompt {Entry:}}
	{-title title Title {}}
    }
}

proc tixED::ConstructTopFrame {w frame} {
    upvar #0 $w data
    tixChainMethod $w ConstructTopFrame $frame
    set index 1

    foreach label $data(-prompt) {
	set name ef$index
	tixLabelFrame $frame.$name -label "$label"
	set data(w:entry$index) \
	    [entry $frame.$name.entry -relief sunken]
	# Add the entry widget
	pack append [$frame.$name frame] \
	    $data(w:entry$index) \
	    {bottom expand fillx}
	$w bind_entry $index "$data(w:btns) subwidget ok invoke"
	incr index 1
	pack $frame.$name -expand 1 -fill both
    }
}

proc tixED::SetBindings {w} {
    upvar #0 $w data
    tixChainMethod $w SetBindings

    $data(w:btns) subwidget ok config -command  "$w popdown
eval $data(-command) \[$w get_values\]"
    $data(w:btns) subwidget cancel config -command "$w popdown"
    pack unpack [$data(w:btns) subwidget apply]
    return $w
}

proc tixED::get_values {w} {
    upvar #0 $w data
    set return_string ""
    for {set i 1} {$i <= [llength $data(-prompt)]} {incr i 1} {
	lappend return_string [$data(w:entry$i) get]
    }
    return $return_string
}

proc tixED::bind_entry {w index binding} {
    upvar #0 $w data
    bind $data(w:entry$index) <Return> $binding
}


proc tixED::get_value {w i} {
    upvar #0 $w data
    $data(w:entry$index) get
}

proc tixED::set_value {w index message} {
    upvar #0 $w data
    $data(w:entry$index) delete 0 end
    $data(w:entry$index) insert 0 $message
}


# ********************
# bug fixes for fileselection box
# ********************

# subwidget: filter, selection

tixWidgetClass tixFileSelectbox {
    -superclass tixPrimitive
    -classname  TixFileSelectbox
    -shortname  tixFsb
    -method {
	filter invoke
    }
    -flag {
	-browsecmd -command -filter -grab -pattern -selection
    }
    -static {}
    -configspec {
	{-browsecmd browseCmd BrowseCmd {}}
	{-command command Command {}}
	{-grab grab Grab global}
	{-filter filter Filter {}}
	{-pattern pattern Pattern *}
	{-selection selection Selection {}}
        {-width width Width 240}
	{-height height Height 280}
    }
    -alias {}
}

proc tixFsb::InitWidgetRec {w} {
    upvar #0 $w data

    tixChainMethod $w InitWidgetRec

    set data(pwd) [pwd]
    if {$data(-filter) == {}} {
	set data(-filter) $data(pwd)/$data(-pattern)
    }
    set data(exposed)   0
    set data(flag)      0
    set data(w_dirmenu) $w.dirmenu
}

proc tixFsb::SetBindings {w} {
    upvar #0 $w data

    tixChainMethod $w SetBindings

    bind $w <Expose> "tixFsb::Expose $w"
}

#----------------------------------------------------------------------
#		Construct widget
#----------------------------------------------------------------------
proc tixFsb::ConstructWidget {w} {
    upvar #0 $w data

    tixChainMethod $w ConstructWidget

    set frame1 [tixFsb::CreateFrame1 $w]
    set frame2 [tixFsb::CreateFrame2 $w]
    set frame3 [tixFsb::CreateFrame3 $w]

    pack $frame1 -in $w -side top -fill x
    pack $frame3 -in $w -side bottom -fill x
    pack $frame2 -in $w -side top -fill both -expand yes

    # Binding of List box 1 (dir)
    #----------------------------
    set list1 [$data(box1) listbox]
    bind $list1 <1>         "tixFsb::SelectDir $w %W %y"
    bind $list1 <B1-Motion> "tixFsb::SelectDir $w %W %y"
    bind $list1 <Double-ButtonPress-1> "tixFsb::InvokeDir $w %W %y"
    global tix_priv
    if [info exists tix_priv(as_loaded)] {
	tixAutoS_BindListboxSingle $list1
    }

    # Binding of List box 2 (file)
    #-----------------------------
    set list2 [$data(box2) listbox]
    bind $list2 <1>         "tixFsb::SelectFile $w %W %y"
    bind $list2 <B1-Motion> "tixFsb::SelectFile $w %W %y"
    bind $list2 <Double-ButtonPress-1> "$w invoke"
    if [info exists tix_priv(as_loaded)] {
	tixAutoS_BindListboxSingle $list2
    }

    # kludge. This can be removed when "pack recalculate" is introduced
    #
    pack propagate $w 0
    $data(rootCmd) config -width  $data(-width)
    $data(rootCmd) config -height $data(-height)
}

proc tixFsb::CreateFrame1 {w} {
    upvar #0 $w data

    frame $w.f1 -border 10
    label $w.f1.l1 -text "Filter"
    tixCombobox $w.f1.filter -history true\
	-listheight 5 \
	-command "$w filter" -anchor e \
	-options {
	    slistbox.scrollbar auto
	}
    set data(w:filter) $w.f1.filter
    $data(w:filter) insert 0 $data(-filter)

    pack $w.f1.l1 $data(w:filter) -side top -expand yes -fill both
    return $w.f1
}

proc tixFsb::CreateFrame2 {w} {
    upvar #0 $w data

    tixPanedWindow $w.f2 -orientation horizontal -width 200 -height 300
    #     THE LEFT FRAME
    #-----------------------
    set dir [$w.f2 add directory]
    $dir config -relief flat
    label $dir.lab -text "Directories"
    tixScrolledListbox $dir.box -options {listbox.geometry 8x6}

    pack $dir.lab -side top                -fill x    -padx 10
    pack $dir.box -side bottom -expand yes -fill both -padx 10

    #     THE RIGHT FRAME
    #-----------------------
    set file [$w.f2 add file]
    $file config -relief flat
    label $file.lab -text "Files"
    tixScrolledListbox $file.box -options {listbox.geometry 8x6}

    pack $file.lab -side top                -fill x    -padx 10
    pack $file.box -side bottom -expand yes -fill both -padx 10

    set data(box1)  $dir.box
    set data(box2)  $file.box
    set data(dummy) $dir.lab

    return $w.f2
}

proc tixFsb::CreateFrame3 {w} {
    upvar #0 $w data

    frame $w.f3 -border 10
    label $w.f3.l1 -text "Selection"
    tixCombobox $w.f3.selection -history true\
	-command "$w invoke" \
	-anchor e \
	-listheight 5 \
	-options {
	    slistbox.scrollbar auto
	}

    set data(w:selection) $w.f3.selection

    pack $w.f3.l1 $data(w:selection) -side top -fill both

    return $w.f3
}

#----------------------------------------------------------------------
#                           CONFIG OPTIONS
#----------------------------------------------------------------------
# Set the filter to the real  directory (resolve all symlinks)
#
proc tixFsb::config-filter {w arg} {
    upvar #0 $w data

    set tmp [pwd]
    catch {
	cd [file dirname $arg]
	set data(pwd) [pwd]
    }
    cd $tmp

    set data(-pattern) [file tail $arg]
    if {$data(pwd) == "/"} {
	set data(-filter) /$data(-pattern)
    } else {
	set data(-filter) $data(pwd)/$data(-pattern)
    }

    $data(w:filter) config -value $data(-filter)
}

proc tixFsb::config-selection {w arg} {
    upvar #0 $w data
    $data(w:selection) config -value $arg
}

proc tixFsb::config-pattern {w arg} {
    upvar #0 $w data
    set dir [file dirname [$data(w:filter) entry get]]
    if {$dir == "/"} {
	set data(-filter) /$arg
    } else {
	set data(-filter) $dir/$arg
    }
    $data(w:filter) config -value $data(-filter)
}

#----------------------------------------------------------------------
#                    INTERNAL METHODS
#----------------------------------------------------------------------
proc tixFsb::LoadDirIntoLists {w} {
    upvar #0 $w data

    $data(box1) listbox delete 0 end
    $data(box2) listbox delete 0 end

    set tmp [pwd]
    cd $data(pwd)

    foreach i [lsort [glob -nocomplain * .*]] {
	set fname [file tail $i]
	if [file isdirectory $data(pwd)/$fname] {
	    $data(box1) listbox insert end $fname
	}
    }

    set data(-filter) [$data(w:filter) entry get]

    set top 0

    # force glob to list the .* files. However, since the use might not
    # be interested in them, put the "normal" files at the top of the listbox

    if {$data(-pattern) == "*"} {
	foreach i [lsort [glob -nocomplain * .*]] {
	    set fname [file tail $i]
	    if {![file isdirectory $data(pwd)/$fname]} {
		$data(box2) listbox insert end $fname
		if {[string index $fname 0] == "."} {
		    incr top
		}
	    }
	}
    } else {
	foreach i [lsort [glob -nocomplain $data(-filter)]] {
	    set fname [file tail $i]
	    if {![file isdirectory $data(pwd)/$fname]} {
		$data(box2) listbox insert end $fname
	    }
	}
    }

    $data(box2) listbox yview $top
    cd $tmp
}

proc tixFsb::LoadDir {w} {
    upvar #0 $w data

    # Sets a grab to $data(dummy), which is a label, so that the 
    # application won't respond to any event during the time the file names
    # are being loaded in. I use a nasty global grab here. I will try to 
    # do it with a localk grab but so far I haven't succeeded.
    update idletasks

    case $data(-grab) {
	global {
	    grab -global $data(dummy)
	    $data(dummy) config -cursor \
		"[tix getbitmap hourglass] \
		 [string range [tix getbitmap hourglass] 1 end]\
		 red white"
	}
	local {
	    grab $data(dummy)
	}
    }

    catch {
	# Just in case some error happens inside grab, don't
	# want to make the X server inaccessible

	update idletasks
	catch {
	    tixFsb::LoadDirIntoLists $w
	} err

	# If the user enters an invalid directory, clean up the mess, restore
	# the filter and attempt to reload the files
	#--------------------------------------------------------------------
	if {$err != {}} {
	    set data(-filter) [$data(w:filter) entry get]
	    set ftail [file tail $data(-filter)]
	    if {$data(pwd) == "/"} {
		set data(-filter) /$ftail
	    } else {
		set data(-filter) $data(pwd)/$ftail
	    }
	    $data(w:filter) config -value $data(-filter)
	    catch {
		tixFsb::LoadDirIntoLists $w
	    } err
	}

	tixFsb::MkDirMenu $w

	# process (and discard) all events when the file names were being 
	# loaded in, then release the grab

	update
	$data(dummy) config -cursor {}
	grab release $data(dummy) 
    } err

    if {$err != {}} {
	grab release $data(dummy) 
	$data(dummy) config -cursor {}
	error $err
    }
}

proc tixFsb::MkDirMenu {w} {
    upvar #0 $w data

    if [winfo exists $data(w_dirmenu)] {
	destroy $data(w_dirmenu)
    }
    tixPopupMenu $data(w_dirmenu) -title Directories 
    $data(w_dirmenu) bind [$data(w:filter) entry]
}

proc tixFsb::MkDirMenuProc {w m} {
    upvar #0 $w data

    set f $data(pwd)

    menu $m
    while {$f != "/"} {
	set name "$f/$data(-pattern)"
	$m add command -label $name -command "tixFsb::MenuInvokeDir $w $name"
	set f [file dirname $f]
    }
    $m add command -label $f/$data(-pattern)
}

proc tixFsb::SelectDir {w dir_list y} {
    upvar #0 $w data

    set data(-filter) [$data(w:filter) entry get]
    set data(-pattern) [file tail $data(-filter)]

    $dir_list select from [$dir_list nearest $y]
    $dir_list select to [$dir_list nearest $y]
    set subdir [$dir_list get [$dir_list nearest $y]]
    if {$data(pwd) == "/"} {
	set data(-filter) /$subdir/$data(-pattern)
    } else {
	set data(-filter) $data(pwd)/$subdir/$data(-pattern)
    }
    $data(w:filter) config -value $data(-filter)
}

proc tixFsb::InvokeDir {w dir_list y} {
    upvar #0 $w data

    set data(-filter) [$data(w:filter) entry get]
    set data(-pattern) [file tail $data(-filter)]

    $dir_list select from [$dir_list nearest $y]
    $dir_list select to [$dir_list nearest $y]
    set tmp [pwd]
    catch {
	cd $data(pwd)
	cd [$dir_list get [$dir_list nearest $y]]
	set data(pwd) [pwd]
    } err

    cd $tmp

    if {$data(pwd) == "/"} {
	set data(-filter) /$data(-pattern)
    } else {
	set data(-filter) $data(pwd)/$data(-pattern)
    }
    $data(w:filter) config -value $data(-filter)
    tixFsb::LoadDir $w
}

proc tixFsb::MenuInvokeDir {w filter} {
    upvar #0 $w data

    set data(-filter) $filter
    set data(pwd) [file dirname $filter]
    $data(w:filter) config -value $filter

    tixFsb::LoadDir $w
}

proc tixFsb::filter {w args} {
    upvar #0 $w data

    set data(-filter)  [$data(w:filter) entry get]
    set data(-pattern) [file tail $data(-filter)]
    set tmp [pwd]
    catch {
	cd $data(pwd)
	cd [file dirname $data(-filter)]
	set data(pwd) [pwd]
    } err

    cd $tmp

    tixFsb::LoadDir $w
}

proc tixFsb::SelectFile {w file_list y} {
    upvar #0 $w data

    $file_list select from [$file_list nearest $y]
    $file_list select to [$file_list nearest $y]
    set selected [$file_list get [$file_list nearest $y]]

    if {$data(pwd) == "/"} {
	$data(w:selection) config -value /$selected
	set data(-selection) /$selected
    } else {
	$data(w:selection) config -value $data(pwd)/$selected
	set data(-selection) $data(pwd)/$selected
    }
    if {$data(-browsecmd) != {}} {
	eval $data(-browsecmd) [$data(w:selection) get]
    }
}

proc tixFsb::invoke {w args} {
    upvar #0 $w data

    $data(w:filter) insert 0 $data(-filter)
    set data(-filter) [$data(w:filter) entry get]

    $data(w:selection) insert 0 [$data(w:selection) entry get]
    set data(-selection) [$data(w:selection) entry get]

    if {$data(-command) != {}} {
	eval $data(-command) [list [$data(w:selection) entry get]]
    }
}

# Load the directory during its first exposure
#
proc tixFsb::Expose {w} {
    upvar #0 $w data

    if {$data(exposed) == 0} {
	set data(exposed) 1
	tixFsb::LoadDir $w
	$data(w:filter) align
    }
}

#----------------------------------------------------------------------
#
#
#              C O N V E N I E N C E   R O U T I N E S 
#
#
#----------------------------------------------------------------------
proc tixMkFileDialog {w args} {
    set option(-okcmd)    {}
    set option(-helpcmd)  {}

    tixHandleOptions option {-okcmd -helpcmd} $args

    toplevel $w
    wm minsize $w 10 10

    tixStdDlgBtns $w.btns
    
    if {$option(-okcmd) != {}} {
	tixFileSelectbox $w.fsb -command "wm withdraw $w; $option(-okcmd)"
    } else {
	tixFileSelectbox $w.fsb -command "wm withdraw $w"
    }

    $w.btns button ok     config -command "$w.fsb invoke"
    $w.btns button apply  config -command "$w.fsb filter" -text Filter
    $w.btns button cancel config -command "wm withdraw $w"

    if {$option(-helpcmd) == {}} {
	$w.btns button help config -state disabled
    } else {
	$w.btns button help config -command $option(-helpcmd)
    }
    wm protocol $w WM_DELETE_WINDOW "wm withdraw $w"
    pack $w.btns  -side bottom -fill both
    pack $w.fsb   -fill both -expand yes

    return $w.fsb
}


