tixClass tixVAutoScroll {
    -superclass {}
    -classname  TixVAutoScroll
    -shortname  tixVASc
    -method {
	bind
    }
    -flag {
	-client
    }
    -static {
	-client
    }
    -readonly {
	-haspixmap

    }
    -configspec {
	{-client	{}}
	{-bitmapdir	{}}
	{-beta       	true}
	{-binding    	TK}
	{-debug      	false}
	{-fontset    	TK}
	{-grabmode 	global}
	{-haspixmap 	0}
	{-libdir     	/usr/local/lib/tix}
	{-scheme     	TK}
    }
}

proc tixVASc::bind {w} {
    upvar #0 $w data

    bind $data(-client) <B1-Leave> 	  "+tixVASc::B1Leave   $w %x %y"
    bind $data(-client) <B1-Motion> 	  "+tixVASc::B1Motion  $w %x %y"
    bind $data(-client) <B1-Enter>        "+tixVASc::B1Enter   $x %x %y"
    bind $data(-client) <ButtonRelease-1> "+tixVASc::B1Release $x %x %y"
}

proc tixWidgetBindAutoScroll {w} {
    global tix_priv
    set tix_priv(as_ignore) 0 
    set tix_priv(as_state)  STOPPED

    set class [winfo class $w]

    bind $w <B1-Leave>         "+[bind $class <B1-Leave>]"
    bind $w <B1-Enter>         "+[bind $class <B1-Enter>]"
    bind $w <B1-Motion>        "+[bind $class <B1-Motion>]"
    bind $w <ButtonRelease-1>  "+[bind $class <ButtonRelease-1>]"
}

proc tixVASc::B1Leave {w x y} {
    global tix_priv

    set data(xcount) 0
    set data(ycount) 0
    set data(state)  STARTED
    set data(width)  [winfo width  $w]
    set data(height) [winfo height $w]
    set data(unit)   $unit
    set data(limit)  $limit

    if {$data(ignore) == 0} {
	tixAutoS_Calc $w $x $y
	tixAutoS_ReSend $w $time $func
    } else {
	puts conf
    }
}

proc tixAutoS_Stop {w} {
    global tix_priv

    set data(state) STOPPED
}

proc tixAutoS_Motion {w time func x y} {
    global tix_priv

    if {$data(state) != "STOPPED"} {
       tixAutoS_Calc $w $x $y
       tixAutoS_Send $w $func
    }
}

proc tixAutoS_Send {w func} {
    global tix_priv

    if {$data(dx) < $data(unit)} {
	incr data(xcount) $data(dx)
	if {$data(xcount) >= $data(unit)} {
	    set dx 1
	    set data(xcount) 0
	} else {
	    set dx 0
	}
    } else {
	set dx [expr "$data(dx) / $data(unit)"]
    }

    if {$data(dy) < $data(unit)} {
	incr data(ycount) $data(dy)
	if {$data(ycount) >= $data(unit)} {
	    set dy 1
	    set data(ycount) 0
	} else {
	    set dy 0
	}
    } else {
	set dy [expr "$data(dy) / $data(unit)"]
    }

    if {$dx > $data(limit)} {
	set dx $data(limit)
    }
    if {$dy > $data(limit)} {
	set dy $data(limit)
    }

    eval $func $w $data(dir) $dx $dy

}

proc tixAutoS_ReSend {w time func} {
    global tix_priv

    if {$data(state) != "STOPPED"} {
	tixAutoS_Send $w $func
	after $time tixAutoS_ReSend $w $time $func
    }
}

# This tells you what the direction is, and the amount of each direction
#
#
proc tixAutoS_Calc {w x y} {
    global tix_priv

    # Calculate the X direction
    #
    if {$x <= 0} {
	set dirx -1
    } elseif {$x >= $data(width)} {
	set dirx  1
    } else {
	set dirx  0
    }

    # Calculate the Y direction
    #
    if {$y <= 0} {
	set diry -1
    } elseif {$y >= $data(height)} {
	set diry  1
    } else {
	set diry  0
    }

    if {$x <= 0} {
	set dx [expr "0 - $x"]
    } elseif {$x >= $data(width)} {
	set dx [expr "$x - $data(width)"]
    } else {
	set dx 0
    }

    if {$dx > 0} {
	set dx 1
    }

    if [catch {set data(dx) [expr "round($dx * log($dx))"]}] {
	set data(dx) 0
    }

    if {$y <= 0} {
	set dy [expr "0 - $y"]
    } elseif {$y >= $data(height)} {
	set dy [expr "$y - $data(height)"]
    } else {
	set dy 0
    }

    if {$dy < 1} {
	set dy 1
    }

    if [catch {set data(dy) [expr "round($dy * log($dy))"]}] {
	set data(dy) 0
    }
}

proc tixAutoS_Update {} {
    global tix_priv

    set data(ignore) 1
    update idletasks
    set data(ignore) 0
}

#----------------------------------------------------------------------
# Scroll for text
#----------------------------------------------------------------------
proc tixAutoS_Text {w dir dx dy} {
    global tix_priv

    if {$dy == 0} {
	return
    }

    case $dir {
	{n nw ne} {
	    tk_textSelectTo $w "@0,0 -$dy line"
	    $w yview [$w index "@0,0 -$dy line"]
	    tixAutoS_Update
	}
	{s sw se} {
	    tk_textSelectTo $w \
		"@$data(width),$data(height) +$dy line"
	    $w yview [$w index "@0,0 +$dy line"]
	    tixAutoS_Update
	}
    }
}


proc tixAutoS_Listbox {w dir dx dy} {
    global tix_priv

    case $dir {
	{n} {
	    if {$dy == 0} {
		return
	    }
	    set sel [expr "[$w nearest 0]-$dy"]
	    $w yview $sel
	    $w select to $sel
	    tixAutoS_Update	
	}
	{s} {
	    if {$dy == 0} {
		return
	    }
	    set height [winfo height $w]
	    set sel [expr "[$w nearest $height]+$dy"]
	    $w yview [expr "[$w nearest 0]+$dy"]
	    $w select to $sel
	    tixAutoS_Update
	}
	{e} {

	}
	{w} {

	}
    }
}


# New bindings
proc tixDefaultAutoBind {} {
    global tix_priv

    tixClassBindAutoScroll Entry \
	$tix_priv(entRate) \
	$tix_priv(entUnit) \
	$tix_priv(entLimit)\
	tixAutoS_Entry
    tixClassBindAutoScroll Listbox \
	$tix_priv(listRate) \
	$tix_priv(listUnit) \
	$tix_priv(listLimit)\
	tixAutoS_Listbox
    tixClassBindAutoScroll Text \
	$tix_priv(textRate) \
	$tix_priv(textUnit) \
	$tix_priv(textLimit)\
	tixAutoS_Text
}

proc tixAutoS_BindListbox {w} {
    tixWidgetBindAutoScroll $w
}

proc tixAutoS_BindListboxSingle {w} {
  global tix_priv

  set time $tix_priv(listRate)
  set unit $tix_priv(listUnit)
  set limit $tix_priv(listLimit)
  set func tixAutoS_ListboxSingle

  set tix_priv(ignore) 0 
  set tix_priv(state)  STOPPED
  set tix_priv(loaded) yes

  bind $w <B1-Leave>     "+tixAutoS_Start  %W $time $unit $limit $func %x %y"
  bind $w <B1-Motion>    "+tixAutoS_Motion %W $time $func %x %y"
  bind $w <B1-Enter>         "+tixAutoS_Stop %W"
  bind $w <ButtonRelease-1>  "+tixAutoS_Stop %W"
}

proc tixAutoS_BindEntry {w} {
    tixWidgetBindAutoScroll $w
}

proc tixAutoS_BindText {w} {
    tixWidgetBindAutoScroll $w
}

set tix_priv(listRate)  30
set tix_priv(listUnit)  130
set tix_priv(listLimit) 10

set tix_priv(textRate)  30
set tix_priv(textUnit)  130
set tix_priv(textLimit) 45

set tix_priv(entRate)   30
set tix_priv(entUnit)   130
set tix_priv(entLimit)  100


# These are functions that serve for Listbox scroll
#
#
proc tixLbSc::Scroll {w which amount} {



}
