tixWidgetClass tixBalloon {
    -superclass tixShell
    -classname  TixBalloon
    -shortname  tixBln
    -method {
	bind unbind
    }
    -flag {
	-initwait -state -statusbar
    }
    -configspec {
	{-initwait initWait InitWait 200}
	{-state state State both}
	{-statusbar statusBar StatusBar {}}

 	{-cursor corsor Corsur left_ptr}
   }
}

proc tixBln::InitWidgetRec {w} {
    upvar #0 $w data

    tixChainMethod $w InitWidgetRec

    set data(popped)    0
    set data(statusSet) 0
    set data(serial)    0
    set data(fakeEnter) 0
    set data(curWidget) {}
}

proc tixBln::ConstructWidget {w} {
    upvar #0 $w data

    tixChainMethod $w ConstructWidget

    wm overrideredirect $w 1
    wm withdraw $w

    # Frame 1 : arrow
    frame $w.f1 -bd 0
    set data(w:label) [label $w.f1.lab -bd 0 -relief flat \
		       -bitmap [tix getbitmap balArrow]]
    pack $data(w:label) -side left
    
    # Frame 2 : Message
    frame $w.f2 -bd 0
    set data(w:message) [message $w.f2.message -padx 0 -pady 0 -bd 0]
    pack $data(w:message) -side left -expand yes -fill both -padx 10 -pady 1

    # Pack all
    pack $w.f1 -fill both
    pack $w.f2 -fill both

}

proc tixBln::SetBindings {w} {
    upvar #0 $w data

    tixChainMethod $w SetBindings

    bind $w <Leave> "tixBln::Leave $w $w %X %Y"
}

#----------------------------------------------------------------------
# PrivateMethods:
#----------------------------------------------------------------------

proc tixBln::UnBind {w client} {
    upvar #0 $w data

    unset data(m:$client)
    unset data(s:$client)
}


# Handle the mouse pointer entering the client widget
#
proc tixBln::Enter {w client} {
    upvar #0 $w data

    if {$data(fakeEnter) == 1} {
	# The mouse pointer just left either the balloon window or the
	# client window: do nothing; otherwise the balloon will flash
	#
	set data(fakeEnter) 0
	return
    }
    if {$data(-state) != "none"} {
    	set data(popped)    0
    	set data(statusSet) 0
	set data(curWidget) $client
	incr data(serial)
    	after $data(-initwait) tixBln::Activate $w $data(serial)
    }
}

proc tixBln::Leave {w client rootX rootY} {
    upvar #0 $w data

    set cw [winfo containing $rootX $rootY]
    if {$cw == "." && $data(curWidget) == "."} {
	# The root window have this problem: [winfo containing] may
	# return "." even the mouse is outside the root window
	#
	set rx1 [winfo rootx .]
	set ry1 [winfo rooty .]
	set rw [winfo width .]
	set rh [winfo height .]
	set rx2 [expr $rx1+$rw]
	set ry2 [expr $ry1+$rh]

	if {$rootX > $rx1 && $rootX < $rx2 && $rootY > $ry1 && $rootY < $ry2} {
	    # mouse moves from balloon to the root window (current client)
	    # This causes a fake enter to the root window, let's
	    # do nothing.
	    #
	    set data(fakeEnter) 1
	    return 
	}
    } elseif [string match $data(curWidget).* $cw] {
	# mouse moves from balloon to current client widget
	# This causes a fake enter to the client widget, let's
	# do nothing.
	#
	set data(fakeEnter) 1
	return
    }

    if [string match $w* $cw] {
	# mouse moves from current client widget to balloon
	#
	set data(fakeEnter) 1
	return
    }

    if {$data(popped) == 1 || $data(statusSet) == 1} {
	if {$data(popped) == 1} {
	    set data(fakeEnter) 0
	    tixBln::Popdown $w
	}
	if {$data(statusSet) == 1} {
	    tixBln::ClearStatus $w
	}
    } elseif {$client == $data(curWidget) || $client == $w} {
	# have to make sure that previous popup's are cancelled
	# just make sure previous
	#
	incr data(serial)
    }
}

proc tixBln::Activate {w serial} {
    upvar #0 $w data

    if {$serial != $data(serial)} {
	# a new balloon will be activated by the latest call
	#
	return	
    } else {
	if {$data(-state) == "both" || $data(-state) == "balloon"} {
	    tixBln::Popup $w
	}
	if {$data(-state) == "both" || $data(-state) == "status"} {
	    tixBln::SetStatus $w
	}
    }
}

proc tixBln::Popup {w} {
    upvar #0 $w data

    # trick: the following lines allow the balloon window to
    # acquire a stable width and height when it is finally
    # put on the visible screen
    #
    set client $data(curWidget)
    $data(w:message) config -text $data(m:$client)
    wm geometry $w +10000+10000
    wm deiconify $w
    raise $w
    update

    # Put it on the visible screen
    #
    set x [expr [winfo rootx $client]+[winfo width  $client]/2]
    set y [expr int([winfo rooty $client]+[winfo height $client]/1.3)]

    wm geometry $w +$x+$y

    set data(popped) 1
}

proc tixBln::Popdown {w} {
    upvar #0 $w data

    wm withdraw $w
    set data(popped) 0

}

proc tixBln::SetStatus {w} {
    upvar #0 $w data

    if {$data(-statusbar) != {}} {
	$data(-statusbar) config -text $data(s:$data(curWidget))
    }
    set data(statusSet) 1
}

proc tixBln::ClearStatus {w} {
    upvar #0 $w data

    # Clear the StatusBar widget
    #
    if {$data(-statusbar) != {}} {
	$data(-statusbar) config -text ""
    }
}

#----------------------------------------------------------------------
# PublicMethods:
#----------------------------------------------------------------------

# %% if balloon is already popped-up for this client, change mesage
#
proc tixBln::bind {w client args} {
    upvar #0 $w data

    if [info exists data(m:$client)] {
	# This client widget is already bound
	#
	return
    }

    set opt(-balloonmsg) {}
    set opt(-statusmsg)  {}

    tixHandleOptions opt {-balloonmsg -statusmsg} $args

    set data(m:$client) $opt(-balloonmsg)
    set data(s:$client) $opt(-statusmsg)

    # Set up the bindings of the widget, in which the balloon should appear
    #
    set class [winfo class $client]

    bind $client <Any-Enter>   +[bind $class <Any-Enter>]
    bind $client <Any-Enter>  "+tixBln::Enter $w $client"
    bind $client <Any-Leave>   +[bind $class <Any-Leave>]
    bind $client <Any-Leave>  "+tixBln::Leave $w $client %X %Y"
    bind $client <Destroy>     +[bind $class <Destroy>]
    bind $client <Destroy>    "+tixBln::UnBind $client"
}
