# BitmapButton
#
#
#
# public widgets

tixWidgetClass tixBitmapButton {
    -superclass tixPrimitive
    -classname  TixBitmapButton
    -shortname  tixBmBtn
    -method {
	-invoke
    }
    -flag {
	-bitmap -command -padx -pady -relief -state -text
    }
    -configspec {
	{-bitmap bitmap Bitmap {}}
	{-command command Command {}}
	{-padx padX Pad 2}
	{-pady padY Pad 2}
	{-relief relief Relief raised}
	{-state state State normal}
	{-text text Text {}}


	{-borderwidth borderWidth BorderWidth 2}
	{-width width Width 0}
	{-width width Width 0}
    }
}


proc tixBmBtn::InitWidgetRec {w} {
    upvar #0 $w data

    tixChainMethod $w InitWidgetRec

    set data(active)    inactive
}

proc tixBmBtn::ConstructWidget {w} {
    upvar #0 $w data

    tixChainMethod $w ConstructWidget

    set data(w_f1)      [frame $w.f1]
    set data(w_f2)      [frame $w.f1.f2]
    set data(w:bitmap)  [label $data(w_f2).bitmap -bitmap $data(-bitmap)]
    set data(w:label)   [label $data(w_f2).label -text $data(-text)]

    tixBmBtn::Pack $w

    # If user specifies a fixed size, the don't cosider the required size
    #
    if {$data(-width) != 0 && $data(-height) != 0} {
	$data(rootCmd) config -width $data(-width) -height $data(-height)
	pack propagate $w false
    }
}

proc tixBmBtn::Pack {w} {
    upvar #0 $w data

    pack $data(w:bitmap) -side top \
	-padx $data(-padx) -pady $data(-pady) -expand yes
    pack $data(w:label)  -side top \
	-padx $data(-padx) -pady $data(-pady)

    place forget $data(w_f2)
    pack $data(w_f2) -in $data(w_f1) -expand yes -fill both
    pack $data(w_f1) -expand yes -fill both
}

proc tixBmBtn::SetBindings {w} {
    upvar #0 $w data

    tixChainMethod $w SetBindings

    foreach wid  "$w $data(w_f1) $data(w_f2) $data(w:label) $data(w:bitmap)" {
	bind $wid <1>               "tixBmBtn::B1 $w"
	bind $wid <ButtonRelease-1> "tixBmBtn::B1Release $w"
    }
    bind $w <B1-Enter>        "tixBmBtn::B1Enter $w"
    bind $w <B1-Leave>        "tixBmBtn::B1Leave $w %X %Y"
}

proc tixBmBtn::B1 {w} {
    upvar #0 $w data

    if {$data(-state) != "normal"} {
	return
    }

    set data(active) active

    set data(oldrelief) $data(-relief)
    if {$data(oldrelief) == "raised"} {
	tixBmBtn::config-relief $w sunken
    }

    grab $w
}

proc tixBmBtn::B1Leave {w rootX rootY} {
    upvar #0 $w data

    if {$data(-state) != "normal"} {
	return
    }

    if [string match $w.* [winfo containing $rootX $rootY]] {
	return
    }

    if {$data(active) == "active"} {
	set data(active) left
    }
}

proc tixBmBtn::B1Enter {w} {
    upvar #0 $w data

    if {$data(-state) != "normal"} {
	return
    }

    if {$data(active) == "left"} {
	set data(active) active
    }
}

proc tixBmBtn::B1Release {w} {
    upvar #0 $w data

    if {$data(-state) != "normal"} {
	return
    }    

    grab release $w

    if {$data(oldrelief) == "raised"} {
	tixBmBtn::config-relief $w raised
    }

    if {$data(active) == "active"} {
	if {$data(-command) != {}} {
	    eval $data(-command)
	}
    }
    set data(active) inactive
}

#----------------------------------------------------------------------
#                           CONFIG OPTIONS
#----------------------------------------------------------------------
proc tixBmBtn::config-background {w value} {
    upvar #0 $w data

    $data(rootCmd)  config -bg $value
    $data(w_f1)     config -bg $value
    $data(w_f2)     config -bg $value
    $data(w:bitmap) config -bg $value
    $data(w:label)  config -bg $value
}


proc tixBmBtn::config-relief {w value} {
    upvar #0 $w data

    $data(rootCmd) config -relief $value

    if {$value == "sunken"} {
	# pack propagate prevent the button from shrinking
	#
	pack propagate $data(w_f1) false
	set width  [winfo width  $data(w_f2)]
	set height [winfo height $data(w_f2)]
	pack forget $data(w_f2)
	place $data(w_f2) -in $data(w_f1) \
	    -x 2 -y 2 -width $width -height $height 
    } else {
	pack propagate $data(w_f1) true
	tixBmBtn::Pack $w
    }
}

proc tixBmBtn::config-state {w value} {
    upvar #0 $w data

    case $value {
	{disabled normal} {
	    $data(w:bitmap) config -state $value
	    $data(w:label)  config -state $value
	}
    }
}
