# Documented subwidgets
#
#     button, cross, entry, listbox, shell, slistbox, tick
#
# Undocumented subwidgets
#
#
#

# The content of the entry is always the last item inserted. To choose
# another item, you have to use "config -value {= 0}"
#

# align: align the combobox to either right or left
# entry, listbox: hook to subwidget
# insert: insert into the listbox


tixWidgetClass tixCombobox {
    -superclass tixLabelWidget
    -classname  TixCombobox
    -shortname  tixCbx
    -method {
	align entry insert listbox
    }
    -flag {
	-arrowbitmap
	-anchor -command -crossbitmap -disabledforeground -dropdown -editable
	-fancy -listlimit -listheight -history -state -tickbitmap -value
    }
    -static {
	-anchor -fancy -type
    }
    -configspec {
	{-arrowbitmap arrowBitmap ArrowBitmap [tix getbitmap cbxarrow]}
	{-anchor anchor Anchor w}
        {-command command Command {}}
	{-crossbitmap crossBitmap CrossBitmap [tix getbitmap cross]}
	{-disabledforeground disabledForeground DisabledForeground #606060}
	{-dropdown dropDown DropDown true}
	{-editable editable Editable false}
	{-fancy fancy Fancy false}
	{-listheight listHeight ListHeight 4}
	{-listlimit listLimit ListLimit 20}
	{-history history History false}
        {-state state State normal}
	{-type type Type static}
	{-value value Value ""}
	{-tickbitmap tickBitmap TickBitmap [tix getbitmap tick]}
    }
}

proc tixCbx::InitWidgetRec {w} {
    upvar #0 $w data

    tixChainMethod $w InitWidgetRec

    set data(virgin)       1
    set data(grabbed)      0
    set data(flag)         0
    set data(listCount)    0

    if {$data(-history)} {
        set data(-editable) true
    }
}

proc tixCbx::ConstructFramedWidget {w frame} {
    upvar #0 $w data
    global tix_priv

    tixChainMethod $w ConstructFramedWidget $frame

    if {$data(-dropdown)} {
	tixCbx::ConstructEntryFrame $w $frame
	tixCbx::ConstructListShell  $w
    } else {
	set f1 [frame $frame.f1]
	set f2 [frame $frame.f2]

	tixCbx::ConstructEntryFrame $w $f1
	tixCbx::ConstructListFrame  $w $f2
	pack $f1 -side top -pady 2 -fill x
	pack $f2 -side top -pady 2 -fill both -expand yes
    }
}

proc tixCbx::ConstructEntryFrame {w frame} {
    upvar #0 $w data
    global tix_priv

    # (1) The entry
    #
    set data(w:entry)  [entry $frame.entry]

    if {$data(-editable)} {
	# do nothing
    } else {
	global tixOption
	$data(w:entry) config -bg $tixOption(bg)
    }

    # This is used during "config-state"
    #
    set data(entryfg) [lindex [$frame.entry config -fg] 4]

    # (2) The dropdown button, not necessary when not in dropdown mode
    #
    if {$data(-dropdown)} {
	set data(w:button) [button $frame.button -bitmap $data(-arrowbitmap)]
    } else {
	set xframe [frame $frame.xframe -width 19]
    }

    # (3) The fancy tick and cross buttons
    #
    if {$data(-fancy)} {
	if {$data(-editable)} {
           set data(w:cross)  [button $frame.cross -bitmap $data(-crossbitmap)]
	   set data(w:tick)   [button $frame.tick  -bitmap $data(-tickbitmap)]

	   pack $frame.cross -side left -padx 1
	   pack $frame.tick  -side left -padx 1
	} else {
	   set data(w:tick)   [button $frame.tick  -bitmap $data(-tickbitmap)]
	   pack $frame.tick  -side left -padx 1
	}
    }

    if {$data(-dropdown)} {
	pack $data(w:button) -side right -padx 1
    } else {
	pack $xframe -side right -padx 1
    }
    pack $frame.entry -side right -fill x -expand yes -padx 1
}

proc tixCbx::ConstructListShell {w} {
    upvar #0 $w data

    # Create the shell and the list
    #------------------------------
    set data(w:shell) [toplevel $w.shell -bd 2 -relief raised]
    wm overrideredirect $data(w:shell) 1
    wm withdraw $data(w:shell)

    set data(w:slistbox) [tixScrolledListbox $data(w:shell).slistbox \
			  -anchor $data(-anchor)]

    set data(w:listbox) [$data(w:slistbox) listbox]

    pack $data(w:slistbox) -expand yes -fill both
}

proc tixCbx::ConstructListFrame {w frame} {
    upvar #0 $w data

    set data(w:slistbox) [tixScrolledListbox $frame.slistbox \
			  -anchor $data(-anchor)]

    set data(w:listbox) [$data(w:slistbox) listbox]

    pack $data(w:slistbox) -expand yes -fill both
}

proc tixCbx::SetBindings {w} {
    upvar #0 $w data

    tixChainMethod $w SetBindings

    bind $data(w:entry) <Down>     "tixCbx::EntArrowKeyDown $w"
    bind $data(w:entry) <Up>       "tixCbx::EntArrowKeyUp   $w"
    bind $data(w:entry) <Return>   "tixCbx::EntReturn $w"
    bind $data(w:entry) <Double-ButtonPress-1> "tixCbx::EntReturn $w"
    bind $data(w:entry) <1>        "+focus %W"
    bind $data(w:entry) <1>        "+[bind Entry <1>]"

    if [info exists tix_priv(as_loaded)] {
	# kludge
	#
	tixAutoS_BindEntry $data(w:entry)
    }

    $data(w:slistbox) config -browsecmd "tixCbx::LbBrowse $w"
    $data(w:slistbox) config -command   "tixCbx::LbInvoke $w"

    if 0 {
    bind $data(w:listbox) <Down>      "tixCbx::LbArrowKeyDown $w"
    bind $data(w:listbox) <Up>        "tixCbx::LbArrowKeyUp   $w"
    bind $data(w:listbox) <Return>    "tixCbx::LbReturn $w"

    bind $data(w:listbox) <B1-Motion> "tixCbx::LbButton $w %y"
    bind $data(w:listbox) <1>         "tixCbx::LbButton $w %y"
    bind $data(w:listbox) <ButtonRelease-1> "tixCbx::LbRelease $w %x %y"
    }

    # Note that <1> is treated the same as escape if it happens
    # outsides of the combobox
    #
    bind $data(w:listbox) <Escape> "tixCbx::EscKey $w"
    bind $data(w:entry)   <Escape> "tixCbx::EscKey $w"
    bind $w               <Escape> "tixCbx::EscKey $w"
    bind $w               <1>      "tixCbx::EscKey $w"

    # Command of buttons
    #
    if {$data(-dropdown) } {
	$data(w:button) config -command "tixCbx::BtnUp $w"
	bind $data(w:button) <Escape> "tixCbx::EscKey $w"
    }

    if {$data(-fancy)} {
	if {$data(-editable)} {
	    $data(w:tick)  config -command "tixCbx::EntReturn  $w"
	    $data(w:cross) config -command "tixCbx::ClearEntry $w"
	} else {
	    $data(w:tick)  config -command "tixCbx::EntReturn  $w"
	}
    }
}

#----------------------------------------------------------------------
#                           CONFIG OPTIONS
#----------------------------------------------------------------------
proc tixCbx::config-state {w arg} {
    upvar #0 $w data

    if {$arg == "normal"} {
	$data(w:button) config -state normal
	$data(w:tick)   config -state normal
	$data(w:cross)  config -state normal
	if {$data(-editable)} {
	    $data(w:entry) config -fg $data(entryfg) -state normal
	} else {
	    $data(w:entry) config -fg $data(entryfg)
	}
    } else {
	$data(w:button) config -state disabled
	$data(w:tick)   config -state disabled
	$data(w:cross)  config -state disabled
	if {$data(-editable)} {
	   $data(w:entry) config -fg $data(-disabledforeground) -state disabled
       } else {
	   $data(w:entry) config -fg $data(-disabledforeground) 
       }
    }
}

# If value is {= index}
#    set the value to be the list item at $index
# otherwise
#    set the value to be $value
#
proc tixCbx::config-value {w value} {
    upvar #0 $w data

    if {[llength $value] == 2 && [lindex $value 0] == "="} {
	set index [lindex $value 1]
	# Get the text indexed by this number
	#
	set text [$data(w:listbox) get $index]
	# also highlight the listbox to indicate selection
	#
	$data(w:listbox) select set $index
	$data(w:listbox) yview       $index
    } else {
	# %% todo: if is not editable, check if $value is valid
	#          also highlight the list properly
	set text $value
    }

    tixSetEntry $data(w:entry) $text

    if {$data(-anchor) == "e"} {
	tixCbx::EntryAlignEnd $w
    }

    return $text
}


# Sets the height of the listbox
#
proc tixCbx::config-listheight {w value} {
    upvar #0 $w data

    # the "10" here is just a dummy. The width is determined by the width
    # of the combobox
    $data(w:listbox) config -geometry 10x$value
}

#----------------------------------------------------------------------
#                     WIDGET COMMANDS
#----------------------------------------------------------------------
proc tixCbx::listbox {w args} {
    return [eval tixCallMethod $w subwidget entry]
}

proc tixCbx::entry {w args} {
    return [eval tixCallMethod $w subwidget entry $args]
}

proc tixCbx::insert {w index newitem} {
    upvar #0 $w data

    $data(w:listbox) insert $index $newitem
    $data(w:listbox) select set $index
    $data(w:listbox) yview $index

    if {$data(listCount) == $data(-listlimit)} {
	$data(w:listbox) delete 0
    } else {
	incr data(listCount)
    }

    set data(-value) $newitem
    tixSetEntry $data(w:entry) $newitem
}

proc tixCbx::align {w} {
    upvar #0 $w data

    if {$data(-anchor) == "e"} {
	tixCbx::EntryAlignEnd $w
    }
}

#----------------------------------------------------------------------
#                   E V E N T   B I N D I N G S
#----------------------------------------------------------------------
proc tixCbx::BtnUp {w} {
    upvar #0 $w data

    if {$data(grabbed) == 0} {
	tixCbx::PopupShell $w
    } else {
	tixCbx::RestoreValue $w
	tixCbx::PopdownShell $w
    }
}

proc tixCbx::EntArrowKeyUp {w} {
    upvar #0 $w data

    if {$data(grabbed) == 0} {
	tixCbx::PopupShell $w
	tixCbx::SetEntry $w
    } else {
	tixCbx::LbArrowKeyUp $w
    }
}

proc tixCbx::EntArrowKeyDown {w} {
    upvar #0 $w data

    if {$data(grabbed) == 0} {
	tixCbx::PopupShell $w
	tixCbx::SetEntry $w
    } else {
	tixCbx::LbArrowKeyDown $w
    }
}

proc tixCbx::EntReturn {w} {
    upvar #0 $w data

    if {$data(grabbed) != 0} {
	tixCbx::PopdownShell $w
    }

    set old_bg [lindex [$data(w:entry) config -bg] 4]
    set old_fg [lindex [$data(w:entry) config -fg] 4]
    $data(w:entry) config -fg $old_bg
    $data(w:entry) config -bg $old_fg
    update idletasks
    $data(w:entry) config -fg $old_fg
    $data(w:entry) config -bg $old_bg

    if {$data(-command) != {} && $data(-command) != "" } {
	eval $data(-command) [list [$data(w:entry) get]]
    }

    if {$data(-history)} {
	tixCbx::insert $w 0 [$data(w:entry) get]
    }
}

proc tixCbx::LbInvoke {w} {
   upvar #0 $w data
   tixCbx::SetEntry $w
   if {$data(-command) != {} && $data(-command) != "" } {
       eval $data(-command) [list [$data(w:entry) get]]
   }
}

proc tixCbx::LbButton {w y} {
   upvar #0 $w data

   $data(w:listbox) select clear
   $data(w:listbox) select set [$data(w:listbox) nearest $y]
   tixCbx::SetEntry $w
}

proc tixCbx::LbReturn {w} {
    upvar #0 $w data

    tixCbx::PopdownShell $w
    if {$data(-command) != {} && $data(-command) != "" } {
	eval $data(-command) [list [$data(w:entry) get]]
    }
}

proc tixCbx::LbRelease {w x y} {
    upvar #0 $w data

    if {$x < 0 || $y < 0} {
	return
    }

    if {$x > [winfo width $data(w:listbox)]} {
	return
    }
    if {$y > [winfo height $data(w:listbox)]} {
	return
    }

    tixCbx::PopdownShell $w
    if {$data(-command) != {}} {
	eval $data(-command) [list [$data(w:entry) get]]
    }
}


proc tixCbx::LbArrowKeyUp {w} {
    upvar #0 $w data

    set indices [$data(w:listbox) curselection]
    if {$indices == "" } {
	$data(w:listbox) select set 0
	$data(w:listbox) yview 0
    } else {
	set i [lindex $indices 0]
	if {$i > 0} {
	    incr i -1
	    $data(w:listbox) select set $i
	    set height [$data(w:slistbox) info height]
	    set yview  [$data(w:slistbox) info yview]
	    if {$i < $yview || $i >= [expr "$yview+$height"]} {
		$data(w:listbox) yview $i
	    }
	}
    }
    tixCbx::SetEntry $w
}


proc tixCbx::LbArrowKeyDown {w} {
    upvar #0 $w data

    set indices [$data(w:listbox) curselection]
    if {$indices == "" } {
	$data(w:listbox) select set 0
	$data(w:listbox) yview 0
    } else {
	set i [lindex $indices 0]
	incr i
	if {$i < [$data(w:slistbox) info totalheight]} {
	    $data(w:listbox) select set $i
	    set height [$data(w:slistbox) info height]
	    set yview  [$data(w:slistbox) info yview]
	    if {$i < $yview || $i >= [expr "$yview+$height"]} {
		$data(w:listbox) yview [expr "$i - $height + 1"]
	    }
	}
    }
    tixCbx::SetEntry $w
}

proc tixCbx::EscKey {w} {
    upvar #0 $w data
    
    if {$data(grabbed) == "1"} {
	tixCbx::RestoreValue $w
	tixCbx::PopdownShell $w
    }
}

#----------------------------------------------------------------------
# Internal commands
#----------------------------------------------------------------------
proc tixCbx::SetEntry {w} {
    upvar #0 $w data

    set indices [$data(w:listbox) curselection]
    if {$indices != ""} {
	set i [lindex $indices 0]
    } else {
	set i 0
    }

    set data(-value)  [$data(w:listbox) get $i]
    tixSetEntry $data(w:entry) [$data(w:listbox) get $i]

    # kludge
    #
    focus $data(w:entry)
}

proc tixCbx::ClearEntry {w} {
    upvar #0 $w data

    $data(w:entry) delete 0 end
}

proc tixCbx::PopdownShell {w} {
    upvar #0 $w data

    wm withdraw $data(w:shell)
    focus $data(w:entry)
    $data(rootCmd) config -cursor {}
    grab release $data(w:root)
    set data(grabbed) 0
}

proc tixCbx::RestoreValue {w} {
    upvar #0 $w data

    tixSetEntry $data(w:entry) $data(oldvalue)
    set data(-value) $data(oldvalue)
}

# Calculating the geometry of the combo box
#
# +4 in the height because of the frames surrounding the listbox
proc tixCbx::PopupShell {w} {
    upvar #0 $w data

    if {$data(-dropdown)} {

	# pop up the shell
	set  y [winfo rooty $data(w:entry)]
	incr y [winfo height $data(w:entry)]
	incr y 3

	set height [expr "[winfo reqheight $data(w:slistbox)]+4"]

	set x1 [winfo rootx $data(w:entry)]
	set x2  [winfo rootx $data(w:button)]
	incr x2 [winfo width $data(w:button)]

	set width  [expr "$x2 - $x1"]

	# If the listbox is below bottom of screen, put it upwards
	set scrheight [winfo screenheight .]
	set bottom [expr $y+$height]
	if {$bottom > $scrheight} {
	    set y [expr $y-$height-[winfo height $data(w:entry)]-5]
	}

	wm geometry $data(w:shell) $width\x$height+$x1+$y
	wm deiconify $data(w:shell)
	raise $data(w:shell)
	focus $data(w:entry)

	if {[$data(w:listbox) curselection] == ""} {
	    $data(w:listbox) select set 0
	}
	$data(rootCmd) config -cursor arrow

	set data(grabbed) 1
	set data(oldvalue) $data(-value)
	catch {
	    grab -global $data(w:root)
	}
    }
}

# The following two routines can emulate a "right align mode" for the
# entry in the combo box.
proc tixCbx::MonitorEntry {w total max begin end} {
    upvar #0 $w data

    if {$data(flag) == 0} {
	set data(flag) 1
	if {$total > $max && $total > [expr $end +1]} {
	    $data(w:entry) view [expr "$total - $max + 1"]
	}
	set data(flag) 0
    }
}

proc tixCbx::EntryAlignEnd {w} {
    upvar #0 $w data

    set data(flag) 0
    $data(w:entry) config \
	-scrollcommand "tixCbx::MonitorEntry $w"
    update idletasks
    $data(w:entry) config -scrollcommand {}
}
