# TixControl Widget
#

tixWidgetClass tixControl {
    -superclass tixLabelWidget
    -classname  TixControl
    -shortname  tixCtl
    -method {
	incr decr invoke
    }
    -flag {
	-autorepeat -command -disabledforeground
	-initwait -llimit -repeatrate 
	-step -state -value -variable -ulimit
    }
    -forcecall {
	-variable
    }
    -configspec {
	{-autorepeat autoRepeat AutoRepeat true}
	{-command command Command {}}
	{-disabledforeground disabledForeground DisabledForeground #606060}
	{-initwait initWait InitWait 500}
	{-llimit llimit Llimit {}}
	{-repeatrate repeatRate RepeatRate 50}
	{-step step Step 1}
	{-state state State normal}
	{-value value Value 0}
	{-variable variable Variable {}}
	{-ulimit ulimit Ulimit {}}
    }
}

proc tixCtl::InitWidgetRec {w} {
    upvar #0 $w data

    tixChainMethod $w InitWidgetRec

    set data(serial)	0
}

proc tixCtl::ConstructFramedWidget {w frame} {
    upvar #0 $w data

    tixChainMethod $w ConstructFramedWidget $frame

    set data(w:entry)  [entry $frame.entry]

    set data(w:incr) [button $frame.incr -bitmap [tix getbitmap incr]]
    set data(w:decr) [button $frame.decr -bitmap [tix getbitmap decr]]

    pack $data(w:entry) -side left  -expand yes -fill both
    pack $data(w:decr)  -side bottom -fill both
    pack $data(w:incr)  -side top    -fill both

    $data(w:entry) delete 0 end
    $data(w:entry) insert 0 $data(-value)

    # This value is used to configure the disable/normal fg of the ebtry
    set data(entryfg) [lindex [$data(w:entry) config -fg] 4]
}

proc tixCtl::SetBindings {w} {
    upvar #0 $w data

    tixChainMethod $w SetBindings

    bind $data(w:incr) <ButtonPress-1> \
	"tk_butDown %W; tixCtl::StartRepeat $w  1"
    bind $data(w:decr) <ButtonPress-1> \
	"tk_butDown %W; tixCtl::StartRepeat $w -1"

    # These bindings will stop the button autorepeat when the 
    # mouse button is up
    foreach btn "$data(w:incr) $data(w:decr)" {
	bind $btn <ButtonRelease-1> "tk_butUp %W; tixCtl::StopRepeat $w"
    }

    # If user press <return>, verify the value and call the -command
    #
    bind $data(w:entry) <Return> \
	"tixCtl::invoke $w"
}


#----------------------------------------------------------------------
#                           CONFIG OPTIONS
#----------------------------------------------------------------------
proc tixCtl::config-state {w arg} {
    upvar #0 $w data

    if {$arg == "normal"} {
	$data(w:incr)  config -state $arg
	$data(w:decr)  config -state $arg
	$data(w:entry) config -state $arg -fg $data(entryfg)
    } else {
	$data(w:incr)  config -state $arg
	$data(w:decr)  config -state $arg
	$data(w:entry) config -state $arg -fg $data(-disabledforeground)
    }
}

proc tixCtl::config-value {w value} {
    upvar #0 $w data

    tixCtl::SetValue $w $value

    # This will tell the Intrinsics: "Please use this value"
    # because "value" might be altered by SetValues
    #
    return $data(-value)
}

proc tixCtl::config-variable {w arg} {
    upvar #0 $w data

    puts hi

    if {$data(-variable) != {}} {
	upvar #0 $data(-variable) oldVar
	trace vdelete oldVar w "tixCtl::TraceProc $w"
    }

    if {$arg != {}} {
	upvar #0 $arg variable
	if [info exists $arg] {
	    # This global variable exists, we use its value
	    #
	    tixCtl::SetValue $w $variable
	} else {
	    # This global variable does not exist; let's set it 
	    #
	    set variable $data(-value)
	}
	trace variable variable w "tixCtl::TraceProc $w"
    }
}

#----------------------------------------------------------------------
#                         User Commands
#----------------------------------------------------------------------
proc tixCtl::incr {w} {
    tixCtl::ChangeValue $w  1
}

proc tixCtl::decr {w} {
    tixCtl::ChangeValue $w -1
}

proc tixCtl::invoke {w} {
    upvar #0 $w data

    if {[catch {$data(w:entry) index sel.first}] == 0} {
	# THIS ENTRY OWNS SELECTION --> TURN IT OFF
	#
	$data(w:entry) select from end
	$data(w:entry) select to   end
    }

    tixCtl::SetValue $w [$data(w:entry) get]
}

#----------------------------------------------------------------------
#                       Internal Commands
#----------------------------------------------------------------------

# The three functions StartRepeat, Repeat and StopRepeat make use of the
# data(serial) variable to discard spurious repeats: If a button is clicked
# repeatedly but is not hold down, the serial counter will increase
# successively and all "after" time event handlers will be discarded
#
proc tixCtl::StartRepeat {w amount} {
    upvar #0 $w data

    incr data(serial)

    tixCtl::ChangeValue $w $amount

    if {$data(-autorepeat)} {
	after $data(-initwait) tixCtl::Repeat $w $amount $data(serial)
    }
}

proc tixCtl::Repeat {w amount serial} {
    upvar #0 $w data

    if {$serial == $data(serial)} {
	tixCtl::ChangeValue $w $amount

	if {$data(-autorepeat)} {
	    after $data(-repeatrate) tixCtl::Repeat $w $amount $data(serial)
	}
    }
}

proc tixCtl::StopRepeat {w} {
    upvar #0 $w data

    incr data(serial)
}

# Change the value by a multiple of the data(-step)
#
proc tixCtl::ChangeValue {w amount} {
    upvar #0 $w data

    tixCtl::SetValue $w [expr "$data(-value) + $amount * $data(-step)"]
}

proc tixCtl::SetValue {w newvalue} {
    upvar #0 $w data

    set oldvalue $data(-value)

    # If the new value is not a valid number, then ignore it
    #
    if [catch {set data(-value) [format "%d" $newvalue]}] {
	if [catch {set data(-value) [format "%f" $newvalue]}] {
	    set data(-value) $oldvalue
	}
    }

    if {$data(-ulimit) != {} && $data(-value) > $data(-ulimit)} {
	set data(-value) $data(-ulimit)
    }
    if {$data(-llimit) != {} && $data(-value) < $data(-llimit)} {
	set data(-value) $data(-llimit)
    }

    $data(w:entry) delete 0 end
    $data(w:entry) insert 0 $data(-value)

    tixCtl::UpdateVariable $w

    if {$data(-command) != {}} {
	eval $data(-command) $data(-value)
    }
}

proc tixCtl::UpdateVariable {w} {
    upvar #0 $w data

    if {$data(-variable) != {}} {
	upvar #0 $data(-variable) variable

	trace vdelete  variable w "tixCtl::TraceProc $w"
	set variable $data(-value)
	trace variable variable w "tixCtl::TraceProc $w"
    }
}

proc tixCtl::TraceProc {w name1 name2 op} {
    upvar #0 $name1 var

    if {$name2 == {}} {
	$w config -value $var
    } else {
	$w config -value $var($name2)
    }
}
