
# File DlgTop.tcl
# This procedure creates a tix DialogToplevel widget
# It is a toplevel widget which interacts with the window manger as a dialog
# box
#
# Written by Joseph Wang (joe@mit.edu)
# Part of the Globewide Network Academy
# Macvicar School of Education and Technology
#
# Among other things:
#   It popups up automatically with requiring the user to hit the 
#      mouse another time
#   It centers itself over a parent widget
#   It iconifies itself when the parent widget is iconifies

proc tixDialogToplevel {w args} {
    eval tixInt_CreateWidget $w tixDlgTop TixDialogToplevel $args
    return $w
}

proc tixDlgTop::CreateClassRec {} {
    global tixDlgTop
    
    # OPTIONS
    set tixDlgTop(superClass) tixPmtv
    set tixDlgTop(options) {-parent -title -mapped -minwidth -minheight}
    set tixDlgTop(staticOptions) {}

    # DEFAULT VALUES
    set tixDlgTop(-parent) {-parent parent Parent .}
    set tixDlgTop(-title)  {-title title Title ""}
    set tixDlgTop(-mapped) {-mapped mapped Mapped 0}
    set tixDlgTop(-minwidth) {-minwidth minWidth MinWidth 0}
    set tixDlgTop(-minheight) {-minheight minHeight MinHeight 0}

    set tixDlgTop(methods) {show hide center}
}

#----------------------------------------------------------------------
#		Construct widget
#----------------------------------------------------------------------
proc tixDlgTop::ConstructWidget {w} {
    upvar #0 $w data

    tixChainMethod $w ConstructWidget

    if {$data(-parent) != ""} {
	wm group $w $data(-parent)
	wm transient $w $data(-parent)
    }

    wm minsize $w $data(-minwidth) $data(-minheight)

    wm title $w $data(-title)
    if {$data(-mapped) == 0} {
	wm withdraw $w
    }
}


proc tixDlgTop::CreateRootWidget {w} {
    upvar #0 $w data
    toplevel $w -class Dialog
}

# The next procedures manage the dialog boxes

proc tixDlgTop::show {w args} {
    upvar #0 $w data

    # First update to make sure the boxes are the right size
    update idletask

    # Then we set the position and update
    tixDlgTop::center $w 
    update idletask

    # and now make it visible. Viola!  Centered over parent.
    wm deiconify $w
}

# This procedure centers a dialog box over a window making sure that the 
# dialog box doesn't appear off the screen

proc tixDlgTop::center {w args} {
    upvar #0 $w data
    # Tell the WM that we'll do this ourselves.
    wm sizefrom $w user
    wm positionfrom $w user

    # Where is my parent and what are it's dimensions
    if {$data(-parent) != ""} {
	set pargeo [split [wm geometry $data(-parent)] "+x"]
	set parwidth [lindex $pargeo 0]
	set parheight [lindex $pargeo 1]
	set parx [lindex $pargeo 2]
	set pary [lindex $pargeo 3]
    } {
	set parwidth [winfo screenwidth $w]
	set parheight [winfo screenheight $w]
	set parx 0
	set pary 0
	set parent [winfo parent $w]
    }

   # What are is the offset of the virtual window
    set vrootx [winfo vrootx $data(-parent)]
    set vrooty [winfo vrooty $data(-parent)]


    # What are my dimensions ?
    set dialogwidth [winfo reqwidth $w]
    set dialogheight  [winfo reqheight $w]

 
    set dialogx [expr $parx+($parwidth-$dialogwidth)/2+$vrootx]
    set dialogy [expr $pary+($parheight-$dialogheight)/2+$vrooty]

    set maxx [expr "[winfo screenwidth $data(-parent)] - $dialogwidth"]
    set maxy [expr "[winfo screenheight $data(-parent)] - $dialogheight"]

# Make sure it doesn't go off screen
    if {$dialogx < 0} {
	set dialogx 0
    } {
	if {$dialogx > $maxx} {
	    set dialogx $maxx
	}
    }
    if {$dialogy < 0} {
	set dialogy 0
    } {
	if {$dialogy > $maxy} {
	    set dialogy $maxy
	}
    }

    # Build my new position (and dimensions)
    wm geometry $w [ format "%dx%d+%d+%d" $dialogwidth $dialogheight \
			 $dialogx $dialogy]
}

proc tixDlgTop::hide {w args} {
    wm withdraw $w
}

