#
# Problems with {args}: you have to use eval and things will get messy:
# especially: some {} will be eliminated by eval.
#
# solution: now all internal procedured use "arglist".
#
#
#
#

# The Tix Intrinsics
#
# Important changes: now support subalcssing

# Performance issue:
#
# Some bottlenecks are eliminated/improved
#
# (1) Checking wether a method is supported by a class, useing "case" is
#     more efficient that using "lsearch"
#
# (2) Now tixCallMethod, tixChainMethod and tixGetMethod all cache the
#     method. This saves a lot of time when the same method of the same
#     class is called again.
#
#


# tixClass
#
# construct for defining a class.
#

proc tixClass {class arglist} {
    tixInt_InitClass $class $arglist 0
}

proc tixWidgetClass {class arglist} {
    tixInt_InitClass $class $arglist 1
}

proc tixInt_InitClass {class arglist isWidget} {
    global tixClasses

    set validOptions {
	-alias -classname -configspec -flag -forcecall -method
	-readonly -shortname -static -superclass
    }
    set opt(-alias)      {}
    set opt(-configspec) {}
    set opt(-classname)  {}
    set opt(-flag)       {}
    set opt(-forcecall)  {}
    set opt(-method)     {}
    set opt(-readonly)   {}
    set opt(-shortname)  {}
    set opt(-static)     {}
    set opt(-superclass) {}

    tixHandleOptions opt $validOptions $arglist
    
    # Record this new class in the list of all classes
    #
    set tixClasses($class,csn) $opt(-shortname)
 
    # Load the superclass if it is not loaded yet
    #
    if {$opt(-superclass) != {}} {
	if {![info exists tixClasses($opt(-superclass),csn)]} {
            $opt(-superclass)::AutoLoad
	}
    }

    upvar #0 $opt(-shortname) classRec

    if {$opt(-superclass) != {}} {
	set classRec(supercsn) $tixClasses($opt(-superclass),csn)   
    } else {
	set classRec(supercsn) {}
    }
    set classRec(className)     $opt(-classname)
    set classRec(options)       $opt(-flag)
    set classRec(forceCall)     $opt(-forcecall)
    set classRec(methods)       $opt(-method)
    set classRec(staticOptions) $opt(-static)

    foreach spec $opt(-configspec) {
	set classRec([lindex $spec 0]) [eval tixInt_Expand $spec]
    }

    foreach spec $opt(-alias) {
	set classRec([lindex $spec 0]) "= [lindex $spec 1]"
    }

    # Merge the class record with the superclass's class record.
    #
    tixInt_MergeClassRec $opt(-shortname)

    # Make the command that create an instance of a class
    # A widgetclass needs a different command than a normal class
    #
    if {$isWidget} {
       set body "return \[eval tixCreateWidget \$w $opt(-shortname) \$args\]"
    } else {
       set body "return \[eval tixCreateInstance \$w $opt(-shortname) \$args\]"
    }
    proc $class {w args} $body

    # This command is necessary for auto-loading of the CDF properly
    #
    proc $class::AutoLoad {} {}
}

#
# Merge the class record with the superclass's class record.
#
proc tixInt_MergeClassRec {csn} {
    upvar #0 $csn classRec

    if {$classRec(supercsn) != ""} {
	# Merge with the superclass's class record.
	#
	upvar #0 $classRec(supercsn) superClassRec

	foreach option $superClassRec(options) {
	    if {[lsearch $classRec(options) $option] == -1} {
		lappend classRec(options) $option
	    }
	}

	foreach option $superClassRec(staticOptions) {
	    if {[lsearch $classRec(staticOptions) $option] == -1} {
		lappend classRec(staticOptions) $option
	    }
	}

	foreach method $superClassRec(methods) {
	    if {[lsearch $classRec(methods) $method] == -1} {
		lappend classRec(methods) $method
	    }
	}
    }

    set classRec(options)       [lsort $classRec(options)]
    set classRec(staticOptions) [lsort $classRec(staticOptions)]
    set classRec(methods)       [lsort $classRec(methods)]
}

#----------------------------------------------------------------------
# tixCreateWidget
#
# Create an instance of a widget class
#
proc tixCreateWidget {w csn args} {
    upvar #0 $w data
    upvar #0 $csn classRec

    # Initialize the Instance Record. So that the CallMethod calls will
    # be executed properly.
    #
    set data(csn)       $csn
    set data(className) $classRec(className)
    set data(context)   $csn

    eval tixCallMethod $w Constructor $args

    return $w
}

#----------------------------------------------------------------------
# tixCreateInstance
#
# Create an instance of a non-widget class
#
proc tixCreateInstance {w csn args} {
    upvar #0 $w   data
    upvar #0 $csn classRec

    # Initialize the Instance Record. So that the CallMethod calls will
    # be executed properly.
    #
    set data(csn)       $csn
    set data(className) $classRec(className)
    set data(context)   $csn

    tixInt_MkInstanceCmd $w
    eval tixInt_SetDefaultOptions $w $args

    tixCallMethod $w Constructor

    return $w
}

proc tixInt_SetDefaultOptions {w args} {
    upvar #0 $w data
    upvar #0 $data(csn) classRec

    # SET UP THE INSTANCE RECORD ACCORDING TO DEFAULT VALUES IN
    # THE OPTIONS DATABASE
    #
    foreach option $classRec(options) {
	set spec [tixInt_GetOptionSpec $data(csn) $option]

	if {[lindex $spec 0] == "="} {
	    continue
	}

	set data($option) [lindex $spec 1]
    }

    # SET UP THE INSTANCE RECORD ACCORDING TO COMMAND ARGUMENTS FROM
    # THE USER OF THE TIX LIBRARY (i.e. Application programmer:)
    #
    tixForEach {option value} $args {
	if {[lsearch $classRec(options) $option] != "-1"} {
	    set spec [tixInt_GetOptionSpec $data(csn) $option]

	    if {[lindex $spec 0] != "="} {
		set data($option) $value
	    } else {
		set realOption [lindex $spec 1]
		set data($realOption) $value
	    }
	} else {
	    error "unknown option $option. Should be: [tixInt_ListOptions $w]"
	}
    }
}

#----------------------------------------------------------------------
# PrivateMethod: MkWidgetCmd
# 
# Construct and set up the compound widget
#
# Override: sometimes
# Chain   : sometimes, before
#
proc tixInt_MkInstanceCmd {w} {
    upvar #0 $w data

    proc $w {method args} {
	global tix_priv
	# the name of the data structure is the same as the procedure
	#
	set w [lindex [info level 0] 0]
	if [catch {eval tixInt_MethodHook $w $method $args} ret] {
	    if {$tix_priv(debug)} {
		global errorInfo
		error $ret $errorInfo
	    } else {
		return -code error $ret
	    }
	} else {
	    return $ret
	}
    }
}


#------------
proc tixInt_ListOptions {w} {
    upvar #0 $w data
    upvar #0 $data(csn) classRec

    return $classRec(options)
}

proc tixInt_QueryAllOptions {w} {
    upvar #0 $w data
    upvar #0 $data(csn) classRec

    set options {}

    foreach option $classRec(options) {
	set spec [tixInt_GetOptionSpec $data(csn) $option]

	if {[lindex $spec 0] != "="} {
	    lappend spec $data($option)
	    lappend options $spec
	} else {
	    set realoption [lindex $spec 1]
	    set spec [tixInt_GetOptionSpec $data(csn) $realoption]
	    lappend options "$option [lindex $spec 1]"
	}
    }

    return $options
}

proc tixInt_QueryOption {w option} {
    upvar #0 $w data
    upvar #0 $data(csn) classRec


    set spec [tixInt_GetOptionSpec $data(csn) $option]

    if {[lindex $spec 0] != "="} {
	return "$spec $data($option)"
    } else {
	return "$option [lindex $spec 1]"
    }
}


proc tixInt_ChangeOneOption {w option value} {
    upvar #0 $w data
    upvar #0 $data(csn) classRec

    # Can't change it if it is a static option
    #
    case $option \
	"{$classRec(staticOptions)} {tixInt_StaticOptionError $option}"

    # Translate an option alias flag to the flag of the real option
    #
    set spec [tixInt_GetOptionSpec $data(csn) $option]
    if {[lindex $spec 0] == "="} {
	set option [lindex $spec 1]
    }

    # Call the appropriate config method to change this option. If a
    # config method does not exist for this option, then simply 
    # update the widget record
    #

    set method [tixGetMethod $w $data(csn) config$option]
    if {$method != ""} {
	set newValue [$method $w $value]
    } else {
	set method [tixGetMethod $w $data(csn) config]
	if {$method != ""} {
	    set newValue [$method $w $option $value]
	} else {
	    set newValue ""
	}
    }

    if {$newValue != ""} {
	# The config method has override the value
	set data($option) $newValue
    } else {
	set data($option) $value
    }
}

# Change the options of an instance after it was created.
#
proc tixInt_ChangeOptions {w args} {
    upvar #0 $w data
    upvar #0 $data(csn) classRec

    if {[llength $args] == 1} {
	return [tixInt_QueryOption $w $args]
    }

    tixForEach {option value} $args {
	if {[lsearch $classRec(options) $option] != -1} {
	    tixInt_ChangeOneOption $w $option $value
	} else {
	    tixInt_UnknownOptionError $w $option
	}
    }
}

proc tixInt_GetVar {w option} {
    upvar #0 $w data
    upvar #0 $data(csn) classRec

    if {[lsearch $classRec(options) $option] != -1} {
	return $data($option)
    } else {
	tixInt_UnknownOptionError $w $option
    }
}

#----------------------------------------------------------------------
# tixInt_MethodHook
# 
# Redirect the call into method calls
#
proc tixInt_MethodHook {w method {args} } {
    global tix_priv
    upvar #0 $w data
    upvar #0 $data(csn) classRec

    if {[lsearch $classRec(methods) $method] != -1} {
	return [eval tixCallMethod $w $method $args]
    }

    case $method {
	#
	# Config is a special method that needs special attention
	#
	{config configure} {
	    if {$args == {}} {
		# Query all options
		return [tixInt_QueryAllOptions $w]
	    }
	    if {[llength $args] == 1} {
		# query the value of an option
		#
		if [catch {tixInt_QueryOption $w [lindex $args 0]} ret] {
		    global errorInfo
		    error $ret $errorInfo
		} else {
		    return $ret
		}
	    } else {
		eval tixInt_ChangeOptions $w $args
		return
	    }
	}
	getvar {
	    return [tixInt_GetVar $w [lindex $args 0]]
	}
	default {
	    # This method is unknown
	    #
	    global errorInfo
	    error [tixInt_UnknownMethodError $w $method] $errorInfo
	}
    }
}

proc tixSuperClass {class} {
    upvar #0 $class classRec

    return $classRec(supercsn)
}

#----------------------------------------------------------------------
# These functions enables overriding, chaining of methods
#
#----------------------------------------------------------------------

proc tixGetMethod {w class method} {
    upvar #0 $w data
    upvar #0 $class classRec
    set theclass $class
    global tixMCache


    if [info exists tixMCache($class,$method)] {
	if {$tixMCache($class,$method) == ""} {
	    return ""
	} else {
	    return $tixMCache($class,$method)::$method
	}
    }

    while 1 {
	if [catch {info body $class::$method}] {
	    if {$classRec(supercsn) != ""} {
		set class $classRec(supercsn)
		upvar #0 $classRec(supercsn) classRec
	    } else {
		set tixMCache($theclass,$method) ""
		return ""
	    }
	} else {
	    set tixMCache($theclass,$method) $class
	    return $class::$method
	}
    }
}

# This command is manually optimized. It uses "info body" instead of
# "info commands", which gives a 30% boost in performance when there
# are a lot of commands
#
# Two other commands are similarly optimized: tixChainMethod and tixGetMethod 
#
proc tixCallMethod {w method args} {
    upvar #0 $w data
    upvar #0 $data(csn) classRec
    global tixMCache

    set old_context $data(context)
    set data(context) $data(csn)

    # We cached the method last time.
    #
    if [info exists tixMCache($data(csn),$method)] {
	if {$tixMCache($data(csn),$method) == ""} {
	    # We know that this method does not exist
	    #
	    set data(context) $old_context
	    error "cannot call method $method for context $data(csn)"
	} else {
	    set data(context) $tixMCache($data(csn),$method)
	    set ret [eval $tixMCache($data(csn),$method)::$method $w $args]
	    set data(context) $old_context
	    return $ret
	}
    }

    while 1 {

	if {$data(context) == {}} {
	    set tixMCache($data(csn),$method) ""
	    set data(context) $old_context
	    error "cannot call method $method for context $data(csn)"
	}

	# TRICK:
	# catch {info body} returns the negation of [info commands ..],
	# but the former is more effecient than the latter
	#
	if {![catch {info body $data(context)::$method}]} {
	    set tixMCache($data(csn),$method) $data(context)
	    set ret [eval $data(context)::$method $w $args]
	    set data(context) $old_context
	    return $ret
	}
	# Let's go to the immediate parent class
	#
	set data(context) [tixSuperClass $data(context)]
    }
}

# Call the method of a previous class
#
proc tixChainMethod {w method args} {
    upvar #0 $w data
    upvar #0 $data(csn) classRec

    if {$data(context) == {}} {
	error "Cannot chain a method without a context"
    } else {
	set init_context $data(context)
    }

    set data(context) [tixSuperClass $data(context)]

    # We cached the method last time.
    #
    if [info exists tixMCache($data(context),$method)] {
	if {$tixMCache($data(context),$method) == ""} {
	    # We know that this method does not exist
	    #
	    set data(context) $init_context
	    error "cannot chain method \"$method\" for context $init_context"
	} else {
	 set data(context) $tixMCache($data(context),$method)
	 set ret [eval $tixMCache($data(context),$method)::$method $w $args]
	 set data(context) $init_context
	 return $ret
	}
    }

    while 1 {

	if {$data(context) == {}} {
	    set data(context) $init_context
	    error "cannot chain method \"$method\" for context $init_context"
	}

	if {![catch {info body $data(context)::$method}]} {
	    set ret [eval $data(context)::$method $w $args]
	    set data(context) $init_context
	    return $ret
	} else {
	    set data(context) [tixSuperClass $data(context)]
	}
    }
}

# Find the spec for this option from this class or the nearest superclass.
# Subclasses can provide new option spec to override the default value
# of a superclass
#
proc tixInt_GetOptionSpec {class option} {
    upvar #0 $class classRec

    while 1 {
	if [info exists classRec($option)] {
	    return $classRec($option)
	} elseif {$classRec(supercsn) != ""} {
	    set class $classRec(supercsn)
	    upvar #0 $classRec(supercsn) classRec
	} else {
	    error "unknown option $option for class $class"
	}
    }
}

# Give an error of "unknown method". The format of this error message
# is similar to the "unknown command" error given by standard TCL.
#
proc tixInt_UnknownMethodError {w method} {
    upvar #0 $w data
    upvar #0 $data(csn) classRec

    set msg "bad option \"$method\":  must be"
    set len [llength $classRec(methods)]
    set i 1

    foreach m $classRec(methods) {
	if {$i == $len} {
	    incr i -1
	    break
	} else {
	    set msg "$msg $m,"
	}
	incr i
    }

    return "$msg or [lindex $classRec(methods) $i]"
}

proc tixInt_StaticOptionError {option} {
    error "cannot configure static option \"$option\""
}

proc tixInt_UnknownOptionError {w method} {
    error "Unknown method \"$method\". Should be: [tixInt_ListOptions $w]"
}

proc tixInt_Expand {args} {
    return $args
}
