# LabelWidget
#
# This widget class is the base class for all widgets that has a
# label. Most Tix compound widgets will have a label so that
# the app programmer doesn't need to add labels themselvel.
#

# LabelSide : top, left, right, bottom, none, acrosstop
#
#
# public widgets {label frame}

tixWidgetClass tixLabelWidget {
    -superclass tixPrimitive
    -classname  TixLabelWidget
    -shortname  tixLaw
    -flag {
	-framerelief -label -labelside -padx -pady
    }
    -static {-labelside}
    -configspec {
	{-framerelief frameRelief FrameRelief flat}
	{-label label Label ""}
	{-labelside labelSide LabelSide left}
	{-padx padX Pad 0}
	{-pady padY Pad 0}
    }
}

proc tixLaw::ConstructWidget {w} {
    upvar #0 $w data

    tixChainMethod $w ConstructWidget

    set data(w:frame)   [frame $w.frame -bd 2 -relief $data(-framerelief)]
    set data(w_framex1) [frame $w.fx1]
    set data(w_framex2) [frame $w.frame.fx2]
    set data(w:label)   [label $w.label -text $data(-label)]

    tixLaw::Pack $w

    tixCallMethod $w ConstructFramedWidget $data(w:frame)
}

proc tixLaw::ConstructFramedWidget {w frame} {
    # Do nothing
}

proc tixLaw::Pack {w} {
    upvar #0 $w data

    case $data(-labelside) {
	acrosstop {
	    # The following tricks make the label appear across the top frame
	    #

	    set labHalfHeight [expr [winfo reqheight $data(w:label)] / 2]

	    if {$data(-pady) < $labHalfHeight} {
		$data(w_framex1) config -height $labHalfHeight \
		    -width [expr [winfo reqwidth $data(w:label)] + 8]
	    } else {
		$data(w_framex1) config -height 1 \
		    -width [expr [winfo reqwidth $data(w:label)] + 8]
	    }
	    $data(w_framex2) config -height $labHalfHeight
	    pack $data(w_framex1) -side top \
		-fill x -padx $data(-padx) 
	    pack $data(w:frame) -side top \
		-padx $data(-padx) -pady $data(-pady) \
		-expand yes -fill both
	    pack $data(w_framex2) -side top

	    if {$data(-pady) < $labHalfHeight} {
		place $data(w:label) -x [expr $data(-padx) + 4] \
		    -y 1
	    } else {
		place $data(w:label) -x [expr $data(-padx) + 4] \
		    -y [expr $data(-pady) - $labHalfHeight + 1] 
	    }
	}
	top {
	    pack $data(w:label) -side top \
		-padx $data(-padx) -pady $data(-pady) \
		-fill x
	    pack $data(w:frame) -side top \
		-padx $data(-padx) -pady $data(-pady) \
		-expand yes -fill both
	}
	bottom {
	    pack $data(w:label) -side bottom \
		-padx $data(-padx) -pady $data(-pady) \
		-fill x
	    pack $data(w:frame) -side bottom \
		-padx $data(-padx) -pady $data(-pady) \
		-expand yes -fill both
	}
	left {
	    pack $data(w:label) -side left \
		-padx $data(-padx) -pady $data(-pady) \
		-fill y
	    pack $data(w:frame) -side left \
		-padx $data(-padx) -pady $data(-pady) \
		-expand yes -fill both
	}
	right {
	    pack $data(w:label) -side right \
		-padx $data(-padx) -pady $data(-pady) \
		-fill x
	    pack $data(w:frame) -side right \
		-padx $data(-padx) -pady $data(-pady) \
		-expand yes -fill both
	}
	none {
	    pack $data(w:frame)\
		-padx $data(-padx) -pady $data(-pady) \
		-expand yes -fill both
	}
	default {
	    error "unknown -labelside option \"$data(-labelside)\""
	}
    }
}

proc tixLaw::RePack {w} {
    upvar #0 $w data

    set labHalfHeight [expr [winfo reqheight $data(w:label)] / 2]

    if {$data(-pady) < $labHalfHeight} {
	$data(w_framex1) config -height $labHalfHeight \
	    -width [expr [winfo reqwidth $data(w:label)] + 8]
    } else {
	$data(w_framex1) config -height 1 \
	    -width [expr [winfo reqwidth $data(w:label)] + 8]
    }

    if {$data(-pady) < $labHalfHeight} {
	place $data(w:label) -x [expr $data(-padx) + 4] \
	    -y 1
    } else {
	place $data(w:label) -x [expr $data(-padx) + 4] \
	    -y [expr $data(-pady) - $labHalfHeight + 1] 
    }
}

#----------------------------------------------------------------------
#                           CONFIG OPTIONS
#----------------------------------------------------------------------
proc tixLaw::config-label {w value} {
    upvar #0 $w data

    $data(w:label) config -text $value

    if {$data(-labelside) == "acrosstop"} {
	tixLaw::RePack $w
    }
}
