tixWidgetClass tixMultiList {
    -superclass tixPrimitive
    -classname  TixMultiList
    -shortname  tixSlb
    -method {
	listbox info
    }
    -flag {
	-anchor -autoscroll -browsecmd -command -scrollbar -selectmode -state
    }
    -static {
	-anchor -scrollbar -selectmode
    }
    -configspec {
	{-anchor anchor Anchor w}
	{-autoscroll autoScroll AutoScroll true}
	{-browsecmd browseCmd BrowseCmd {}}
	{-command command Command {}}
	{-scrollbar scrollbar Scrollbar both}
	{-selectmode selectMode SelectMode single}
	{-state state State normal}
    }
}

proc tixSlb::InitWidgetRec {w} {
    upvar #0 $w data

    tixChainMethod $w InitWidgetRec

    set data(width)          0
    set data(height)         0
    set data(xview)          0
    set data(yview)          0
    set data(totalwidth)     0
    set data(totalheight)    0
    set data(type)           nil
    set data(flag)           0
}

proc tixSlb::ConstructWidget {w} {
    upvar #0 $w data

    tixChainMethod $w ConstructWidget

    frame $w.f1 -border 0
    frame $w.f2 -border 0

    scrollbar $w.hsb \
	-orient horizontal \
	-command "$w.listbox xview"

    scrollbar $w.vsb \
	-command "$w.listbox yview"

    listbox $w.listbox \
	-xscrollcommand "tixSlb::MonitorSize $w width $w.hsb" \
	-yscrollcommand "tixSlb::MonitorSize $w height $w.vsb"

    set scbWidth [lindex [$w.vsb config -width] 4]
    set scbBd    [lindex [$w.vsb config -bd]    4]

    frame $w.xframe -height $scbWidth -relief flat \
	-width [expr "$scbWidth + 2 * $scbBd"]

    set data(w:listbox) $w.listbox
    set data(w:hsb)     $w.hsb
    set data(w:vsb)     $w.vsb
    set data(w:xframe)  $w.xframe

    tixSlb::Repack $w $data(-scrollbar)

    # some kludge
    #
    bind $w.hsb <1>                "+set [format "%s(flag)" $w] 1"
    bind $w.hsb <ButtonRelease-1>  "+set [format "%s(flag)" $w] 0"
}

proc tixSlb::SetBindings {w} {
    upvar #0 $w data

    tixChainMethod $w SetBindings

}

proc tixSlb::ScrollBarsNeeded {w} {
    upvar #0 $w data

    if {$data(totalwidth) > $data(width)} {
	if {$data(totalheight) > $data(height) || $data(yview) > 0 } {
	    return both
	} else {
	    return x
	}
    } elseif {$data(totalheight) > $data(height) || $data(yview) > 0 } {
	return y
    } else {
	return none
    }
}

proc tixSlb::Repack {w type} {
    upvar #0 $w data

    if {$type == "auto"} {
	set type [tixSlb::ScrollBarsNeeded $w]
    }

    if {$type != $data(type)} {
	pack forget $data(w:listbox)
	pack forget $data(w:hsb)
	pack forget $data(w:vsb)
	pack forget $data(w:xframe)
	pack forget $w.f1
	pack forget $w.f2

	tixSlb::Pack $w $type
	set data(type) $type
    }
}

proc tixSlb::Pack {w type} {
    upvar #0 $w data

    if {$type == "both"} {
	pack $data(w:vsb)     -in $w.f1 -side right -fill y
	pack $data(w:listbox) -in $w.f1 -side left  -fill both -expand yes

	pack $data(w:hsb)     -in $w.f2 -side left -fill x -expand yes
	pack $data(w:xframe)  -in $w.f2 -side left -fill y

	pack $w.f2 -side bottom -fill x
	pack $w.f1 -side top -fill both -expand yes
    } elseif {$type == "x"} {
	pack $data(w:hsb)     -side bottom -fill x
	pack $data(w:listbox) -side top -fill both -expand yes
    } elseif {$type == "y"} {
	pack $data(w:vsb)     -side right -fill y
	pack $data(w:listbox) -side left -fill both -expand yes
    } else {
	pack $data(w:listbox) -fill both -expand yes
    }
}

#----------------------------------------------------------------------
#                     WIDGET COMMANDS
#----------------------------------------------------------------------
proc tixSlb::listbox {w args} {
    upvar #0 $w data

    if {$args != {}} {
	return [eval $data(w:listbox) $args]
    } else {
	return $data(w:listbox)
    }
}

proc tixSlb::info {w spec} {
    upvar #0 $w data

    if {$spec == "width" || $spec == "height"} {
	return $data($spec)
    }
    if {$spec == "totalwidth" || $spec == "totalheight"} {
	return $data($spec)
    }
    if {$spec == "xview" || $spec == "yview"} {
	return $data($spec)
    }
}

#----------------------------------------------------------------------
#                     Internal commands
#----------------------------------------------------------------------
proc tixSlb::MonitorSize {w spec scb total max begin end} {
    upvar #0 $w data

    set data($spec) $max
    set data(total$spec) $total
    if {$spec == "width"} {
	set data(xview) $begin
    } else {
	set data(yview) $begin
    }
    $scb set $total $max $begin $end

    if {$data(-scrollbar) == "auto"} {
	tixSlb::Repack $w auto
    }

    if {$data(flag) == 0} {
	if {$data(-anchor) == "e"} {
	    set data(flag) 1
	    if {$total > $max && $total > [expr $end +1]} {
		$data(w:listbox) xview [expr "$total - $max + 1"]
	    }
	    set data(flag) 0
	}
    }
}
