tixWidgetClass tixMwmClient {
    -superclass tixVResize
    -classname  TixMwmClient
    -shortname  tixMwmCn
    -method {
	maximize
    }
    -flag {
	-borderbg -cursorfg -cursorbg
	-framewidth -handlesize
	-title -titlebd -titleheight
	-hintcolor -hintwidth
	-parent
	-sysbitmap -minbitmap -maxbitmap -resbitmap
    }
    -configspec {
	{-borderbg borderBg BorderBg gray}
	{-cursorfg cursorFg CursorColor white}
	{-cursorbg cursorBg CursorColor red}
	{-foreground foreground Foreground lightgray}
	{-framewidth frameWidth FrameWidth 3}
	{-handlesize handleSize HandleSize 32}
	{-hintcolor hintColor HintColor red}
	{-hintwidth hintWidth HintWidth 1}
	{-parent parent Parent {}}
	{-title title Title {}}
	{-titlebd titleBd BorderWidth 1}
	{-titleheight titleHeight titleHeight {}}
	{-sysbitmap sysBitmap SysBitmap {}}
	{-minbitmap minBitmap MinBitmap {}}
	{-maxbitmap maxBitmap MaxBitmap {}}
	{-resbitmap resBitmap ResBitmap {}}

	{-borderwidth borderWidth BorderWidth 1}
	{-height height Height 150}
	{-minheight minHeight MinHeight 40}
	{-minwidth minWidth MinWidth 120}
        {-width width Width 230}
    }
}

proc tixMwmCn::InitWidgetRec {w} {
    upvar #0 $w data

    tixChainMethod $w InitWidgetRec

    # Dynamically set the default value of the bitmaps
    #
    global TIX_SOURCE_DIR
    set dir $TIX_SOURCE_DIR/bitmaps

    if {$data(-sysbitmap) == {}} {
	set data(-sysbitmap)  @$dir/system.xpm
    }
    if {$data(-minbitmap) == {}} {
	set data(-minbitmap)  @$dir/minimize.xpm
    }
    if {$data(-maxbitmap) == {}} {
	set data(-maxbitmap)  @$dir/maximize.xpm
    }
    if {$data(-resbitmap) == {}} {
	set data(-resbitmap)  @$dir/restore.xpm
    }
}

proc tixMwmCn::ConstructWidget {w} {
    upvar #0 $w data

    tixChainMethod $w ConstructWidget

    $data(rootCmd) config -bd 1 -bg black

    # the four corner handles
    set r00 [frame $w.r00 -relief raised -bd $data(-borderwidth) \
	     -cursor "top_left_corner $data(-cursorbg) $data(-cursorfg)"\
	     -bg $data(-borderbg)]
    set r01 [frame $w.r01 -relief raised -bd $data(-borderwidth)\
	     -cursor "bottom_left_corner $data(-cursorbg) $data(-cursorfg)"\
	     -bg $data(-borderbg)]
    set r10 [frame $w.r10 -relief raised -bd $data(-borderwidth)\
	     -cursor "top_right_corner $data(-cursorbg) $data(-cursorfg)"\
	     -bg $data(-borderbg)]
    set r11 [frame $w.r11 -relief raised -bd $data(-borderwidth)\
	     -cursor "bottom_right_corner $data(-cursorbg) $data(-cursorfg)"\
	     -bg $data(-borderbg)]

    # the four border handles
    set bt [frame $w.bt -relief raised -bd $data(-borderwidth) \
	     -cursor "top_side $data(-cursorbg) $data(-cursorfg)"\
	     -bg $data(-borderbg)]
    set bb [frame $w.bb -relief raised -bd $data(-borderwidth)\
	     -cursor "bottom_side $data(-cursorbg) $data(-cursorfg)"\
	     -bg $data(-borderbg)]
    set br [frame $w.br -relief raised -bd $data(-borderwidth)\
	     -cursor "right_side $data(-cursorbg) $data(-cursorfg)"\
	     -bg $data(-borderbg)]
    set bl [frame $w.bl -relief raised -bd $data(-borderwidth)\
	     -cursor "left_side $data(-cursorbg) $data(-cursorfg)"\
	     -bg $data(-borderbg)]

    # the inner sunken frame
    set sunken [frame $w.sunken -relief sunken -bd $data(-borderwidth)]

    # the top bar
    set bar [frame $sunken.bar]

    # Do the buttons inside the title. they are packed, not placed
    set title [button $bar.title -anchor sw -pady 0 -padx 8\
	       -text $data(-title) -relief raised -bd $data(-titlebd)\
	     -bg $data(-borderbg)]
    set sys [menubutton $bar.sys -relief raised -bd $data(-titlebd)\
	     -bitmap $data(-sysbitmap) -padx 0 -pady 0 -anchor center\
	     -menu $bar.sys.m\
	     -bg $data(-borderbg)]
    set min [button $bar.min -relief raised -bd $data(-titlebd)\
	     -bitmap $data(-minbitmap) -padx 0 -pady 0 -anchor center\
	     -bg $data(-borderbg)]
    set max [button $bar.max -relief raised -bd $data(-titlebd)\
	     -bitmap $data(-maxbitmap) -padx 0 -pady 0 -anchor center\
	     -bg $data(-borderbg)]

    pack $sys -side left -fill y
    pack $max $min -side right -fill y
    pack $title -side left -fill both -expand yes

    tixMwmCn::MkSysMenu $w $bar.sys.m

    # the frame for user to use
    set frame [frame $sunken.frame]

    # The frames to use for resizing and dragging
    #
    set data(w:ht) $w:tix_priv_ht
    set data(w:hb) $w:tix_priv_hb
    set data(w:hx) $w:tix_priv_hx
    set data(w:hl) $w:tix_priv_hl
    set data(w:hr) $w:tix_priv_hr

    frame $data(w:ht) -bg $data(-hintcolor) -height $data(-hintwidth)
    frame $data(w:hb) -bg $data(-hintcolor) -height $data(-hintwidth)
    frame $data(w:hx) -bg $data(-hintcolor) -height $data(-hintwidth)
    frame $data(w:hl) -bg $data(-hintcolor) -width  $data(-hintwidth)
    frame $data(w:hr) -bg $data(-hintcolor) -width  $data(-hintwidth)

    # Record the internal widget pathnames
    set data(w:title) $title
    set data(w:r00)   $w.r00
    set data(w:r01)   $w.r01
    set data(w:r10)   $w.r10
    set data(w:r11)   $w.r11
    set data(w:bt)    $w.bt
    set data(w:bb)    $w.bb
    set data(w:bl)    $w.bl
    set data(w:br)    $w.br
    set data(w:sys)   $sys
    set data(w:min)   $min
    set data(w:max)   $max
    set data(w:frame) $w.sunken.frame
} 

proc tixMwmCn::MkSysMenu {w menu} {
    upvar #0 $w data

    menu $menu

    $menu add command -under 0 -label "Restore    "
    $menu add command -under 0 -label "Move       "
    $menu add command -under 0 -label "Size       "
}

proc tixMwmCn::SetBindings {w} {
    upvar #0 $w data

    tixChainMethod $w SetBindings

    bind $w <Configure> "tixMwmCn::ComposeWindow $w"

    bind $data(w:title) <1> \
	"tixMwmCn::dragstart $w $data(w:title) 1 %X %Y  {1  1  0  0}"
    bind $data(w:r00)   <1> \
	"tixMwmCn::dragstart $w $data(w:r00)   0 %X %Y  {1  1 -1 -1}"
    bind $data(w:r01)   <1> \
	"tixMwmCn::dragstart $w $data(w:r01)   0 %X %Y  {1  0 -1  1}"
    bind $data(w:r10)   <1> \
	"tixMwmCn::dragstart $w $data(w:r10)   0 %X %Y  {0  1  1 -1}"
    bind $data(w:r11)   <1> \
	"tixMwmCn::dragstart $w $data(w:r11)   0 %X %Y  {0  0  1  1}"
    bind $data(w:bt)    <1> \
	"tixMwmCn::dragstart $w $data(w:bt)    0 %X %Y  {0  1  0 -1}"
    bind $data(w:bb)    <1> \
	"tixMwmCn::dragstart $w $data(w:bb)    0 %X %Y  {0  0  0  1}"
    bind $data(w:bl)    <1> \
	"tixMwmCn::dragstart $w $data(w:bl)    0 %X %Y  {1  0 -1  0}"
    bind $data(w:br)    <1> \
	"tixMwmCn::dragstart $w $data(w:br)    0 %X %Y  {0  0  1  0}"

    foreach win [list \
		 $data(w:title)\
		 $data(w:r00)\
		 $data(w:r01)\
		 $data(w:r10)\
		 $data(w:r11)\
		 $data(w:bt)\
		 $data(w:bb)\
		 $data(w:bl)\
		 $data(w:br)\
		 ] {
	bind $win <B1-Motion>       "tixVRe::drag    $w %X %Y"
	bind $win <ButtonRelease-1> "tixVRe::dragend $w $win 0 %X %Y"
	bind $win <Any-Escape>      "tixVRe::dragend $w $win 1  0  0"
    }

    # special attention
    bind $data(w:title) <1> [list +$data(w:title) config \
			     -cursor "fleur $data(-cursorbg) $data(-cursorfg)"]
    bind $data(w:title) <ButtonRelease-1> \
	"+$data(w:title) config -cursor {}"

    # the menu of the system button
    bind $data(w:sys) <1> \
	"[bind Menubutton <1>]; %W config -relief sunken"
    bind $data(w:sys) <Any-ButtonRelease-1> \
	"tk_mbUnpost; %W config -relief raised"
}

#----------------------------------------------------------------------
#                    Internal methods
#----------------------------------------------------------------------
proc tixMwmCn::dragstart {w win depress rootx rooty mrect} {
    upvar #0 $w data

    set wx [winfo x      $w]
    set wy [winfo y      $w]
    set ww [expr [winfo width  $w] - 2]
    set wh [expr [winfo height $w] - 2]

    tixVRe::dragstart $w $win $depress $rootx $rooty \
	[list $wx $wy $ww $wh] $mrect
}


# Place the hint frame to indicate the changes
#
proc tixMwmCn::SetHintFrame {w x y width height} {
    upvar #0 $w data

    # The four sides of the window
    #
    set x2 [expr "$x+$width"]
    set y2 [expr "$y+$height"]

    place $data(w:ht) -x $x  -y $y  -width  $width  -bordermode inside
    place $data(w:hb) -x $x  -y $y2 -width  $width  -bordermode inside
    place $data(w:hl) -x $x  -y $y  -height $height -bordermode inside
    place $data(w:hr) -x $x2 -y $y  -height $height -bordermode inside

    # This ensures that the hint frames appear as quickly as possible
    #
    
    # One additional line to indicate the position of the title
    #
    set hndy [expr "$y+$data(-handlesize)"]
    place $data(w:hx) -x $x -y $hndy -width  $width -bordermode inside
}

proc tixMwmCn::ShowHintFrame {w} {
    upvar #0 $w data

    raise $w
    raise $data(w:ht)
    raise $data(w:hb)
    raise $data(w:hx)
    raise $data(w:hl)
    raise $data(w:hr)
}

proc tixMwmCn::HideHintFrame {w} {
    upvar #0 $w data

    # Hide the hint frame
    #
    place forget $data(w:ht)
    place forget $data(w:hb)
    place forget $data(w:hl)
    place forget $data(w:hr)
    place forget $data(w:hx)
}

proc tixMwmCn::UpdateSize {w x y width height} {
    place $w -x $x -y $y -width [expr $width + 2] -height [expr $height + 2]
}

# TK calls this command each time the window is resized
#
proc tixMwmCn::ComposeWindow {w} {
    upvar #0 $w data

    set width  [expr [winfo width  $w] - 2]
    set height [expr [winfo height $w] - 2]

    # place the corner handlers
    set size $data(-handlesize)

    set rx [expr "$width-$size"]
    set ry [expr "$height-$size"]

    place $w.r00 -x 0   -y 0   -width $size -height $size \
	    -bordermode inside
    place $w.r01 -x 0   -y $ry -width $size -height $size \
	    -bordermode inside
    place $w.r10 -x $rx -y 0   -width $size -height $size \
	    -bordermode inside
    place $w.r11 -x $rx -y $ry -width $size -height $size \
	    -bordermode inside

    # Place the border handlers
    #
    set size2 [expr "2 * $size"]
    set bw [expr "$width-$size2"]
    set bh [expr "$height-$size2"]

    place $w.bt  -x $size -y 0      -width $bw   -height $size \
	    -bordermode inside
    place $w.bb  -x $size -y $ry    -width $bw   -height $size \
	    -bordermode inside
    place $w.bl  -x 0     -y $size  -width $size -height $bh \
	    -bordermode inside
    place $w.br  -x $rx   -y $size  -width $size -height $bh \
	    -bordermode inside

    # Place the sunken frame
    #
    set fw [expr "$data(-framewidth) + $data(-borderwidth)"]
    set sw [expr "$width  - (2 * $fw)"]
    set sh [expr "$height - (2 * $fw)"]
    place $w.sunken -x $fw -y $fw -width $sw  -height $sh \
	    -bordermode inside

    # Place the title bar
    set tw [expr "$data(-framewidth) + (2 * $data(-borderwidth) )"]
    set aw [expr "$width - (2 * $tw)"]

    if {$data(-titleheight) == {}} {
      set data(-titleheight) \
	[expr "$data(-handlesize)-$data(-framewidth)-(2*$data(-borderwidth))"]
    }
    place $w.sunken.bar -x 0 -y 0 -width $aw -height $data(-titleheight)
    $data(w:sys) config -width [expr "$data(-titleheight) - 2*$data(-titlebd)"]
    $data(w:max) config -width [expr "$data(-titleheight) - 3*$data(-titlebd)"]
    $data(w:min) config -width [expr "$data(-titleheight) - 3*$data(-titlebd)"]

    # Place the user frame
    set fh [expr "$sh - $data(-titleheight) - $data(-borderwidth) * 2"]
    place $w.sunken.frame -x 0 -y $data(-titleheight) \
	-width $aw -height $fh
}

#----------------------------------------------------------------------
#                  public methods
#----------------------------------------------------------------------
proc tixMwmCn::maximize {w} {
    upvar #0 $w data

    # should be GetMaximizeInfo
}
