# Documented subwidgets
#
#     listbox, hsv, vsb
#
# Undocumented subwidgets
#
#     xframe
#

# -selectmode : single, multiple, disjoint

tixWidgetClass tixNewScrolledListbox {
    -superclass tixVScroll
    -classname  TixNewScrolledListbox
    -shortname  tixNSlb
    -method {
	listbox info
    }
    -flag {
	-anchor -autoscroll -browsecmd -command -scrollbar -selectmode -state
    }
    -static {
	-anchor -scrollbar -selectmode
    }
    -configspec {
	{-anchor anchor Anchor w}
	{-autoscroll autoScroll AutoScroll true}
	{-browsecmd browseCmd BrowseCmd {}}
	{-command command Command {}}
	{-scrollbar scrollbar Scrollbar both}
	{-selectmode selectMode SelectMode single}
	{-state state State normal}
    }
}

proc tixNSlb::InitWidgetRec {w} {
    upvar #0 $w data

    tixChainMethod $w InitWidgetRec

    set data(x,Total)  0
    set data(x,Window) 0
    set data(x,First)  0
    set data(x,Last)   0

    set data(y,Total)  0
    set data(y,Window) 0
    set data(y,First)  0
    set data(y,Last)   0

    set data(wait)     0
    set data(curIndex) -1
}

proc tixNSlb::ConstructClientWindow {w parent} {
    upvar #0 $w data

    set data(w:listbox) [listbox $parent.window -relief flat -bd 0]
}

proc tixNSlb::SetBindings {w} {
    upvar #0 $w data

    tixChainMethod $w SetBindings

    # bind for auto-scrolling here
    #
    $data(w:listbox) config \
	-xscrollcommand "tixNSlb::LbxScrollCommand $w x" \
	-yscrollcommand "tixNSlb::LbxScrollCommand $w y"

    bind $data(w:listbox) <1> "tixNSlb::Button1 $w %y"
    bind $w <Return> "tixNSlb::ReturnKey $w"
}

proc tixNSlb::GetClientInfo {w xinfo yinfo} {
    upvar #0 $w data

    set defXOffset [lindex $xinfo 2]

    set clntXInfo \
	[list $data(x,Total) $data(x,Window) $data(x,First) $data(x,Last)] 
    set clntYInfo \
	[list $data(y,Total) $data(y,Window) $data(y,First) $data(y,Last)]

    return [list $clntXInfo $clntYInfo]
}

proc tixNSlb::SetClientView {w which first} {
    upvar #0 $w data

    tixNSlb::SaveLbxInfo $w $which \
	$data($which,Total) $data($which,Window) \
	$first \
	$data($which,Last)

    case $which {
	"x" {
	    $data(w:listbox) xview $data(x,First)
	}
	"y" {
	    $data(w:listbox) yview $data(y,First)
	}
    }

    set clientInfo [list \
		    $data($which,Total) \
		    $data($which,Window) \
		    $data($which,First) \
		    $data($which,Last)]
    return $clientInfo
}

# This is the -{x,y}scrollcommand option of the listbox.
# it keeps track of the sizes of the listbox
#
proc tixNSlb::LbxScrollCommand {w which total window first last} {
    upvar #0 $w data

    if {$which == "x"} {
	incr window -1
	incr last -1
    }
    tixNSlb::SaveLbxInfo $w $which $total $window $first $last
#   tixDoWhenIdle tixNSlb::UpdateScroll $w
    tixNSlb::UpdateScroll $w
}

proc tixNSlb::SaveLbxInfo {w which total window first last} {
    upvar #0 $w data

    if {$first < 0} {
	set first 0
    }

    if {$last <= [expr $first + $window]} {
	set last [expr $first + $window - 1]
    }

    if {$window >= $total} {
	set first 0
	set last  [expr $total -1]
    } else {

	if {[expr $window + $first] > $total} {
	    set first [expr $total - $window]
	    set last  [expr $total - 1]
	}

	if {$last > $total} {
	    set first [expr $total - $window]
	    set last  [expr $total - 1]
	}
    }

    set data($which,Total)  $total
    set data($which,Window) $window
    set data($which,First)  $first
    set data($which,Last)   $last
}

proc tixNSlb::UpdateScroll {w} {
    upvar #0 $w data

    if {$data(wait)} {
	return
    }

    set data(wait) 1
    set clntXInfo \
	[list $data(x,Total) $data(x,Window) $data(x,First) $data(x,Last)] 
    set clntYInfo \
	[list $data(y,Total) $data(y,Window) $data(y,First) $data(y,Last)]

    eval tixCallMethod $w ClientRequest [list $clntXInfo $clntYInfo]

    set data(wait) 0
}

#----------------------------------------------------------------------
#     Virtual event handlers
#----------------------------------------------------------------------
proc tixNSlb::HandleLineUp {w} {
    upvar #0 $w data

    if {$data(curIndex) == "-1"} {
	tixChainMethod $w HandleLineUp
	return
    }

    incr data(curIndex) -1
    tixNSlb::ShowCurIndex $w
}

proc tixNSlb::HandleLineDown {w} {
    upvar #0 $w data

    if {$data(curIndex) == "-1"} {
	tixChainMethod $w HandleLineDown
	return
    }

    incr data(curIndex) 1
    tixNSlb::ShowCurIndex $w
}

proc tixNSlb::HandlePageUp {w} {
    upvar #0 $w data

    if {$data(curIndex) == "-1"} {
	tixChainMethod $w HandlePageUp
	return
    }

    incr data(curIndex) -$data(y,Window)
    tixNSlb::ShowCurIndex $w
}

proc tixNSlb::HandlePageDown {w} {
    upvar #0 $w data

    if {$data(curIndex) == "-1"} {
	tixChainMethod $w HandlePageDown
	return
    }

    incr data(curIndex) $data(y,Window)
    tixNSlb::ShowCurIndex $w
}

proc tixNSlb::ShowCurIndex {w} {
   upvar #0 $w data

   if {$data(curIndex) < 0} {
       set data(curIndex) 0
   }
   if {$data(curIndex) >= $data(y,Total)} {
       set data(curIndex) [expr $data(y,Total) -1 ]
   }

   $data(w:listbox) select from $data(curIndex)
   $data(w:listbox) select to   $data(curIndex)

   if {$data(curIndex) > $data(y,Last) || $data(curIndex) < $data(y,First)} {
       set data(y,First) [expr $data(curIndex) - $data(y,Window) / 2]
       if {$data(y,First) < 0} {
	   set data(y,First) 0
       }
       set data(y,Last) [expr $data(y,First) + $data(y,Window) -1]

       tixNSlb::UpdateScroll $w
   }
}

# will be implemented in scroll class
#
proc tixNSlb::Button1 {w y} {
    upvar #0 $w data

    set data(curIndex) [$data(w:listbox) nearest $y]
    $data(w:listbox) select from $data(curIndex)
    $data(w:listbox) select to   $data(curIndex)
}

proc tixNSlb::ReturnKey {w} {
    upvar #0 $w data

    if {$data(-command) != ""} {
	if {$data(curIndex) != -1} {
	    eval $data(-command) [list [$data(w:listbox) get $data(curIndex)]]
	} else {
	    eval $data(-command) {}
	}
    }
}
