tixWidgetClass tixOptionMenu {
    -superclass tixLabelWidget
    -classname  TixOptionMenu
    -shortname  tixOpM
    -method {
	add disable enable 
    }
    -flag {
	-command -value
    }
    -configspec {
	{-command command Command {}}
	{-value value Value {}}

	{-framerelief frameRelief FrameRelief raised}
    }
}

proc tixOpM::InitWidgetRec {w} {
    upvar #0 $w data

    tixChainMethod $w InitWidgetRec
 
    set data(nItems)	0
}

proc tixOpM::ConstructFramedWidget {w frame} {
    upvar #0 $w data

    tixChainMethod $w ConstructFramedWidget $frame

    set data(w:menubutton) [menubutton $frame.menubutton]
    set data(w:menu)       [menu $frame.menubutton.menu]
    set data(w:selector)   [frame $frame.selector]
    pack $data(w:menubutton) -side left -expand yes -fill both
    pack $data(w:selector)   -side left -padx 3 -pady 3

    $data(w:menubutton) config -menu $data(w:menu)
}

proc tixOpM::SetBindings {w} {
    upvar #0 $w data

    tixChainMethod $w SetBindings

    bind $data(w:menubutton) <1> "tixOpM::ButtonDown $w"
    bind $data(w:selector)   <1> "tixOpM::ButtonDown $w"
    bind $data(w:frame)      <1> "tixOpM::ButtonDown $w"
    bind $data(w:menubutton)   <ButtonRelease-1> \
	"tixOpM::ButtonUp $w %X %Y"
    bind $data(w:selector)     <ButtonRelease-1> \
	"tixOpM::ButtonUp $w %X %Y"
    bind $data(w:frame)        <ButtonRelease-1> \
	"tixOpM::ButtonUp $w %X %Y"
    bind $data(w:menubutton) <B1-Motion> "tixOpM::B1Motion $w %X %Y"
    bind $data(w:selector)   <B1-Motion> "tixOpM::B1Motion $w %X %Y"
    bind $data(w:frame)      <B1-Motion> "tixOpM::B1Motion $w %X %Y"

    bind $data(w:menu) <ButtonRelease-1> "tixOpM::Invoke $w"

    # Don't want the menu to behave as a tear-off menu
    #
    bind $data(w:menubutton) <2> {}
}

proc tixOpM::ButtonDown {w} {
    upvar #0 $w data

    $data(w:selector) config -relief sunken
}

proc tixOpM::ButtonUp {w rootX rootY} {
    upvar #0 $w data

    $data(w:selector) config -relief raised

    if [tixOpM::WithinFrame $w $rootX $rootY] {
	tixOpM::Post $w
    }
}

proc tixOpM::B1Motion {w rootX rootY} {
    upvar #0 $w data

    if [tixOpM::WithinFrame $w $rootX $rootY] {
	$data(w:selector) config -relief sunken
    } else {
	$data(w:selector) config -relief raised
    }
}

proc tixOpM::WithinFrame {w rootX rootY} {
    upvar #0 $w data

    set cw [winfo containing $rootX $rootY]
    if {$cw ==  $data(w:frame)} {
	return 1
    } elseif {[string match $data(w:frame).* $cw]} {
	return 1
    } else {
	puts $cw
	return 0
    }
}

proc tixOpM::Post {w} {
    upvar #0 $w data

    set rootx [winfo rootx $data(w:frame)]
    set rooty [winfo rooty $data(w:frame)]

    # adjust for the border of the menu and frame
    #
    incr rootx [lindex [$data(w:menu)  config -border] 4]
    incr rooty [lindex [$data(w:frame) config -border] 4]
    incr rooty [lindex [$data(w:menu)  config -border] 4]

    set value $data(-value)
    set y [$data(w:menu) yposition $data($value,index)]

    $data(w:menu) post $rootx [expr $rooty - $y]
    $data(w:menu) activate $data($value,index)
    grab $data(w:menu)
}

proc tixOpM::Invoke {w} {
    upvar #0 $w data

    $data(w:menu) unpost
    grab release $data(w:menu)

    set i [$data(w:menu) index active]
    if {$i != "none"} {
	set name $data($i,name)
	set data(-value) $name
	$data(w:menubutton) config -text $data($name,label)

	if {$data(-command) != {}} {
	    eval $data(-command) $name
	}
    }
}


proc tixOpM::config-value {w value} {
    upvar #0 $w data

    set name $value

    if [info exists data($name,index)] {
	$data(w:menubutton) config -text $data($name,label)
    } else {
	error "item \"$value\" does not exist"
    }
}


proc tixOpM::add {w type name args} {
    upvar #0 $w data

    if [info exists data($name,index)] {
	error "item $name already exists in the option menu $w"
    }

    case $type {
	"command" {
	    set validOptions {
		-accelerator -label -underline
	    }
	    set opt(-accelerator)	{}
	    set opt(-label)		$name
	    set opt(-underline)		{}

	    tixHandleOptions opt $validOptions $args

	    # Create a new item inside the menu
	    #
	    $data(w:menu) add command -label $opt(-label)
	    set index $data(nItems)
	    if {$opt(-accelerator) != {}} {
		$data(w:menu) entryconfig $index \
		    -accelerator $opt(-accelerator)
	    }
	    if {$opt(-underline) != {}} {
		$data(w:menu) entryconfig $index \
		    -underline $opt(-underline)
	    }

	    # Store info about this item
	    #
	    set data($index,name) $name
	    set data($name,label) $opt(-label)
	    set data($name,index) $index

	    if {$index == 0} {
		$data(w:menubutton) config -text $data($name,label)
		set data(-value) $name
	    }

	    incr data(nItems)
	}
	"separator" {
	    $data(w:menu) add separator
	    incr data(nItems)
	}
	default {
	    error "only types \"separator\" and \"command\" are allowed"
	}
    }
}
