tixWidgetClass tixPanedWindow {
    -superclass tixPrimitive
    -classname  TixPanedWindow
    -shortname  tixPnw
    -method {
	add pane panes set update
    }
    -flag {
	-command -handleactivebg -handlebg -orientation 
	-separatoractivebg -separatorbg
    }
    -static {
	-orientation
    }
    -configspec {
	{-command command Command {}}
	{-handleactivebg handleActiveBg HandleActiveBg bisque2}
	{-handlebg handleBg Background #ffe4c4}
	{-orientation orientation Orientation vertical}
	{-separatoractivebg separatorActiveBg SeparatorActiveBg red}
	{-separatorbg separatorBg Background #ffe4c4}
    }
    -alias {}
}

#----------------------------------------------------------------------
# ClassInitialization:
#----------------------------------------------------------------------

proc tixPnw::InitWidgetRec {w} {
    upvar #0 $w data

    tixChainMethod $w InitWidgetRec

    set data(items)       {}
    set data(nItems)      0
    set data(totalsize)   0
    set data(movePending) 0
}

proc tixPnw::ConstructWidget {w} {
    upvar #0 $w data

    tixChainMethod $w ConstructWidget

    # kludge. This can be removed when "pack recalculate" is introduced

    pack propagate $w 0
    $data(rootCmd) config -width  $data(-width)
    $data(rootCmd) config -height $data(-height)
}

proc tixPnw::SetBindings {w} {
    upvar #0 $w data

    tixChainMethod $w SetBindings

    bind $w <Configure> "tixPnw::HandleConfigure $w"
}

#----------------------------------------------------------------------
# ConfigOptions:
#----------------------------------------------------------------------
proc tixPnw::config-handlebg {w arg} {
    upvar #0 $w data

    for {set i 1} {$i < $data(nItems)} {incr i} {
	$data(btn,$i) config -bg $arg
    }
}

#----------------------------------------------------------------------
# PublicMethods:
#----------------------------------------------------------------------


# method: add
#
#    Adds a new pane into the PanedWindow.
#
# options -size -max -min -allowresize
#
proc tixPnw::add {w name args} {
    upvar #0 $w data


    # Step 1: Parse the options to get the children's size options

    # The default values
    #
    set option(-size)        0
    set option(-min)         0
    set option(-max)         100000
    set option(-allowresize) 1

    tixHandleOptions option {-size -max -min -allowresize} $args

    set data($name,size)        $option(-size)
    set data($name,min)         $option(-min)
    set data($name,max)         $option(-max)
    set data($name,allowresize) $option(-allowresize)

    # Step 2: Add the frame and the separator (if necessary)
    #
    frame $w.$name -bd 1 -relief raised

    lappend data(items) $name
    incr data(nItems)

    if {$data(nItems) > 1} {
	tixPnw::AddSeparator $w
    }

    # Step 3: Add the new frame and adjust the panedWindow
    #
    if {$data(-orientation) == "vertical"} {
	place $w.$name -x 0 -relwidth 1 \
	    -y $data(totalsize)
    } else {
	place $w.$name -y 0 -relheight 1 \
	    -x $data(totalsize)
    }

    incr data(totalsize) $data($name,size)

    return $w.$name
}

proc tixPnw::pane {w name args} {
    upvar #0 $w data

    if {$args == {}} {
	# return the pathname
	return $w.name
    } else {
	return [eval $w.$name $args]
    }
}

# return the name of all panes
proc tixPnw::panes {w} {
    upvar #0 $w data

    set ret {}
    foreach name $data(items) {
	lappend ret $w.$name
    }

    return $ret
}

proc tixPnw::set {w name args} {
    upvar #0 $w data

    set option(-size)        $data($name,size)
    set option(-min)         $data($name,min)
    set option(-max)         $data($name,max)
    set option(-allowresize) $data($name,allowresize)

    tixHandleOptions option {-size -max -min -allowresize} $args

    set data($name,size)        $option(-size)
    set data($name,min)         $option(-min)
    set data($name,max)         $option(-max)
    set data($name,allowresize) $option(-allowresize)

    tixPnw::HandleConfigure $w
}

proc tixPnw::update {w} {
    upvar #0 $w data

    # If the geometry was explicitely set, update() does not calculate
    # the natural size. Just return
    #
    if {$data(-geometry) != {}} {
	return
    }
    if {$data(-width) != 0 && $data(-height) != 0} {
	return
    }


    set w_and_h [tixPnw::GetNaturalSizes $w]

    if {$data(-width) == 0} {
	set data(-width) [lindex $w_and_h 0]
	$data(rootCmd) config -width $data(-width)
    }
    if {$data(-height) == 0} {
	set data(-height) [lindex $w_and_h 1]
	$data(rootCmd) config -height $data(-height)
    }

    tixPnw::UpdateSizes $w
    tixPnw::PlotHandles $w 0
}

#----------------------------------------------------------------------
# PrivateMethods:
#----------------------------------------------------------------------

proc tixPnw::AddSeparator {w} {
    upvar #0 $w data

    set n [expr "$data(nItems)-1"]

    if {$data(-orientation) == "vertical"} {
	set data(sep,$n) [frame $w.sep$n -relief sunken \
			  -bd 1 -height 2 -width 10000 -bg $data(-separatorbg)]
    } else {
	set data(sep,$n) [frame $w.sep$n -relief sunken \
			  -bd 1 -width 2 -height 10000 -bg $data(-separatorbg)]
    }

    set data(btn,$n) [frame $w.btn$n -relief raised \
		      -bd 1 -width 9 -height 9 -cursor crosshair \
		      -bg $data(-handlebg)]

    bind  $data(btn,$n) \
	<ButtonPress-1>   "tixPnw::BtnDown $w $n"
    bind  $data(btn,$n) \
	<ButtonRelease-1> "tixPnw::BtnUp   $w $n"
    bind  $data(btn,$n) \
	<Any-Enter>       "tixPnw::HighlightBtn $w $n"
    bind  $data(btn,$n) \
	<Any-Leave>       "tixPnw::DeHighlightBtn $w $n"

    if {$data(-orientation) == "vertical"} {
	bind  $data(btn,$n) <B1-Motion> \
	    "tixPnw::BtnMove $w $n %Y"
    } else {
	bind  $data(btn,$n) <B1-Motion> \
	    "tixPnw::BtnMove $w $n %X"
    }

    if {$data(-orientation) == "vertical"} {
	place $data(btn,$n) -relx 0.90 -y [expr "$data(totalsize)-5"]
	place $data(sep,$n) -x 0 -y [expr "$data(totalsize)-1"] -relwidth 1
    } else {
	place $data(btn,$n) -rely 0.90 -x [expr "$data(totalsize)-5"]
	place $data(sep,$n) -y 0 -x [expr "$data(totalsize)-1"] -relheight 1
    }

    for {set i 1} {$i < $data(nItems)} {incr i} {
	raise $data(sep,$i)
	raise $data(btn,$i)
    }
}

proc tixPnw::BtnDown {w item} {
    upvar #0 $w data

    if {$data(-orientation) == "vertical"} {
	set spec -height
    } else {
	set spec -width
    }

    for {set i 1} {$i < $data(nItems)} {incr i} {
	$data(sep,$i) config -bg $data(-separatoractivebg) $spec 1
    }

    $data(btn,$item) config -relief sunken
    tixPnw::GetMotionLimit $w $item
    grab $data(btn,$item)
    set data(movePending) 0
}

proc tixPnw::Min2 {a b} {
    if {$a < $b} {
	return $a
    } else {
	return $b
    }
}

proc tixPnw::GetMotionLimit {w item} {
    upvar #0 $w data

    set curBefore 0
    set minBefore 0
    set maxBefore 0

    for {set i 0} {$i < $item} {incr i} {
	set name [lindex $data(items) $i]
	incr curBefore $data($name,size)
	incr minBefore $data($name,min)
	incr maxBefore $data($name,max)
    }

    set curAfter 0
    set minAfter 0
    set maxAfter 0
    while {$i < $data(nItems)} {
	set name [lindex $data(items) $i]
	incr curAfter $data($name,size)
	incr minAfter $data($name,min)
	incr maxAfter $data($name,max)
	incr i
    }

    set beforeToGo  \
     [tixPnw::Min2 [expr "$curBefore-$minBefore"] [expr "$maxAfter-$curAfter"]]
    set afterToGo  \
     [tixPnw::Min2 [expr "$curAfter-$minAfter"] [expr "$maxBefore-$curBefore"]]

    set data(beforeLimit) [expr "$curBefore-$beforeToGo"]
    set data(afterLimit)  [expr "$curBefore+$afterToGo"]
    set data(curSize)     $curBefore

    tixPnw::PlotHandles $w 1
}

# Compress the motion so that update is quick even on slow machines
#
# rootp = root position (either rootx or rooty)
proc tixPnw::BtnMove {w item rootp} {
    upvar #0 $w data

    set data(rootp) $rootp
    if {$data(movePending) == 0} {
	after 2 tixPnw::BtnMoveCompressed $w $item
	set data(movePending) 1
    }
}

proc tixPnw::BtnMoveCompressed {w item} {
    upvar #0 $w data

    if {$data(-orientation) == "vertical"} {
	set p [expr $data(rootp)-[winfo rooty $w]]
    } else {
	set p [expr $data(rootp)-[winfo rootx $w]]
    }

    if {$p == $data(curSize)} {
	set data(movePending) 0
	return
    }

    if {$p < $data(beforeLimit)} {
	set p $data(beforeLimit)
    }
    if {$p >= $data(afterLimit)} {
	set p $data(afterLimit)
    }
    tixPnw::CalculateChange $w $item $p

    # Force the redraw to happen
    #
    update idletasks
    set data(movePending) 0
}

# Calculate the change in response to mouse motions
#
proc tixPnw::CalculateChange {w item p} {
    upvar #0 $w data

    if {$p < $data(curSize)} {
	tixPnw::MoveBefore $w $item $p
    } elseif {$p > $data(curSize)} {
	tixPnw::MoveAfter $w $item $p
    }

    tixPnw::PlotHandles $w 1
}

proc tixPnw::MoveBefore {w item p} {
    upvar #0 $w data

    set n [expr "$data(curSize)-$p"]

    # Shrink the frames before
    #
    set from [expr $item-1]
    set to   0
    tixPnw::Iterate $w $from $to tixPnw::Shrink $n

    # Adjust the frames after
    #
    set from $item
    set to   [expr "$data(nItems)-1"]
    tixPnw::Iterate $w $from $to tixPnw::Grow $n

    set data(curSize) $p
}

proc tixPnw::MoveAfter {w item p} {
    upvar #0 $w data

    set n    [expr "$p-$data(curSize)"]

    # Shrink the frames after
    #
    set from $item
    set to   [expr "$data(nItems)-1"]
    tixPnw::Iterate $w $from $to tixPnw::Shrink $n

    # Graw the frame before
    #
    set from [expr $item-1]
    set to   0
    tixPnw::Iterate $w $from $to tixPnw::Grow $n

    set data(curSize) $p
}


proc tixPnw::PlotHandles {w showBorders} {
    upvar #0 $w data

    set totalsize 0
    set i 0

    if {$data(-orientation) == "vertical"} {
	set btnp [expr [winfo width $w]-13]
    } else {
	set h [winfo height $w]
	if {$h > 18} {
	    set btnp 9
	} else {
	    set btnp [expr $h-9]
	}
    }

    set firstpane [lindex $data(items) 0]
    set totalsize $data($firstpane,size)

    for {set i 1} {$i < $data(nItems)} {incr i} {

	if {$data(-orientation) == "vertical"} {
	    place $data(btn,$i) -x $btnp -y [expr "$totalsize-4"]
	    place $data(sep,$i) -x 0 -y [expr "$totalsize-1"] -relwidth 1
	} else {
	    place $data(btn,$i) -y $btnp -x [expr "$totalsize-5"]
	    place $data(sep,$i) -y 0 -x [expr "$totalsize-1"] -relheight 1
	}

	set name [lindex $data(items) $i]
	incr totalsize $data($name,size)
    }
}

proc tixPnw::BtnUp {w item} {
    upvar #0 $w data

    tixPnw::UpdateSizes $w
    $data(btn,$item) config -relief raised
    grab release $data(btn,$item)
}


proc tixPnw::UpdateSizes {w} {
    upvar #0 $w data

    set data(totalsize) 0

    for {set i 0} {$i < $data(nItems)} {incr i} {
	set name [lindex $data(items) $i]
	if {$data(-orientation) == "vertical"} {
	    place $w.$name -x 0 -relwidth 1 -y $data(totalsize) \
		-height $data($name,size)
	} else {
	    place $w.$name -y 0 -relheight 1 -x $data(totalsize) \
		-width $data($name,size)
	}
	incr data(totalsize) $data($name,size)
    }

    # Reset the color and width of the separator

    if {$data(-orientation) == "vertical"} {
	set spec -height
    } else {
	set spec -width
    }

    for {set i 1} {$i < $data(nItems)} {incr i} {
	$data(sep,$i) config -bg $data(-separatorbg) $spec 2
    }

    # Invoke the callback command
    #
    if {$data(-command) != {}} {
	set sizes {}
	foreach item $data(items) {
	    lappend sizes $data($item,size)
	}
	eval $data(-command) [list $sizes]
    }
}

proc tixPnw::HighlightBtn {w item} {
    upvar #0 $w data

    $data(btn,$item) config -background $data(-handleactivebg)
}

proc tixPnw::DeHighlightBtn {w item} {
    upvar #0 $w data

    $data(btn,$item) config -background $data(-handlebg)
}

proc tixPnw::GetNaturalSizes {w} {
    upvar #0 $w data

    set data(totalsize) 0

    if {$data(-orientation) == "vertical"} {
	set majorspec height
	set minorspec width
    } else {
	set majorspec width
	set minorspec height
    }

    # update() will cause the packer to recalculate the reqwidth and
    # reqheight of all the panes. Then GetNaturalSizes will return their
    # sizes correctly.
    #
    update idletasks

    set minorsize 0
    foreach name $data(items) {
	# set the minor size
	#
	set req_minor [winfo req$minorspec $w.$name]
	if {$req_minor > $minorsize} {
	    set minorsize $req_minor
	}

	# Check the natural size against the max, min requirements.
	# Change the natural size if necessary
	#
	if {$data($name,size) == 0} {
	    set data($name,size) [winfo req$majorspec $w.$name]
	}
	if {$data($name,size) < $data($name,min)} {
	    set data($name,size) $data($name,min)
	}
	if {$data($name,size) > $data($name,max)} {
	    set data($name,size) $data($name,max)
	}

	# Add up the total size
	#
	incr data(totalsize) $data($name,size)
    }

    if {$data(-orientation) == "vertical"} {
	return "$minorsize $data(totalsize)"
    } else {
	return "$data(totalsize) $minorsize"
    }
}

#--------------------------------------------------
# Handling resize
#--------------------------------------------------

# This monitor the sizes of the child windows
#
proc tixPnw::HandleConfigure {w} {
    upvar #0 $w data

    tixPnw::GetNaturalSizes $w

    if {$data(-orientation) == "vertical"} {
	set newSize [winfo height $w]
    } else {
	set newSize [winfo width $w]
    }

    if {$newSize > $data(totalsize)} {
	# Grow from bottom
	set n    [expr "$newSize-$data(totalsize)"]
	set from [expr "$data(nItems)-1"]
	set to   0
	tixPnw::Iterate $w $from $to tixPnw::Grow $n
	tixPnw::UpdateSizes $w

    } elseif {$newSize < $data(totalsize)} {
	# Shrink from bottom
	set n    [expr "$data(totalsize)-$newSize"]
	set from [expr "$data(nItems)-1"]
	set to   0
	tixPnw::Iterate $w $from $to tixPnw::Shrink $n
	tixPnw::UpdateSizes $w
    }

    tixPnw::PlotHandles $w 0
}

#--------------------------------------------------
# Shrink and grow items
#--------------------------------------------------
proc tixPnw::Shrink {w name n} {
    upvar #0 $w data

    set canShrink [expr "$data($name,size) - $data($name,min)"]

    if {$canShrink > $n} {
	incr data($name,size) -$n
	return 0
    } elseif {$canShrink > 0} {
	set data($name,size) $data($name,min)
	incr n -$canShrink
    }
    return $n
}

proc tixPnw::Grow {w name n} {
    upvar #0 $w data

    set canGrow [expr "$data($name,max) - $data($name,size)"]

    if {$canGrow > $n} {
	incr data($name,size) $n
	return 0
    } elseif {$canGrow > 0} {
	set data($name,size) $data($name,max)
	incr n -$canGrow
    }
    return $n
}

proc tixPnw::Iterate {w from to proc n} {
    upvar #0 $w data

    if {$from <= $to} {
	for {set i $from} {$i <= $to} {incr i} {
	    set n [$proc $w [lindex $data(items) $i] $n]
	    if {$n == 0} {
		break
	    }
	}
    } else {
	for {set i $from} {$i >= $to} {incr i -1} {
	    set n [$proc $w [lindex $data(items) $i] $n]
	    if {$n == 0} {
		break
	    }
	}
    }
}
