tixWidgetClass tixResizeHandle {
    -superclass tixVResize
    -classname  TixResizeHandle
    -shortname  tixReH
    -method {
	hide show
    }
    -flag {
	-command -cursorfg -cursorbg -handlesize -hintcolor -hintwidth -x -y
    }
    -configspec {
	{-command command Command {}}
	{-cursorfg cursorFg CursorColor white}
	{-cursorbg cursorBg CursorColor red}
	{-handlesize handleSize HandleSize 6}
	{-hintcolor hintColor HintColor red}
	{-hintwidth hintWidth HintWidth 1}
	{-x x X 0}
	{-y y Y 0}
    }
}

proc tixReH::InitWidgetRec {w} {
    upvar #0 $w data

    tixChainMethod $w InitWidgetRec

    set data(shown) 0
}

proc tixReH::ConstructWidget {w} {
    upvar #0 $w data

    tixChainMethod $w ConstructWidget

    # Create the hints
    #
    set data(w_ht) $w:tix_priv_ht
    set data(w_hb) $w:tix_priv_hb
    set data(w_hl) $w:tix_priv_hl
    set data(w_hr) $w:tix_priv_hr

    frame $data(w_ht) -height $data(-hintwidth) -bg $data(-background)
    frame $data(w_hb) -height $data(-hintwidth) -bg $data(-background)
    frame $data(w_hl) -width  $data(-hintwidth) -bg $data(-background)
    frame $data(w_hr) -width  $data(-hintwidth) -bg $data(-background)

    # Create the corner resize handles
    #
    set data(w_r00) $w

    $data(rootCmd) config\
	-cursor "top_left_corner $data(-cursorbg) $data(-cursorfg)"

    set data(w_r01) $w:tix_priv_01
    set data(w_r10) $w:tix_priv_10
    set data(w_r11) $w:tix_priv_11

    frame $data(w_r01) -relief $data(-relief) -bd $data(-borderwidth) \
	-cursor "bottom_left_corner $data(-cursorbg) $data(-cursorfg)"\
	-bg $data(-background)
    frame $data(w_r10) -relief $data(-relief) -bd $data(-borderwidth) \
	-cursor "top_right_corner $data(-cursorbg) $data(-cursorfg)"\
	-bg $data(-background)
    frame $data(w_r11) -relief $data(-relief) -bd $data(-borderwidth) \
	-cursor "bottom_right_corner $data(-cursorbg) $data(-cursorfg)"\
	-bg $data(-background)

    # Create the border resize handles
    #
    set data(w_bt)  $w:tix_priv_bt
    set data(w_bb)  $w:tix_priv_bb
    set data(w_bl)  $w:tix_priv_bl
    set data(w_br)  $w:tix_priv_br

    frame $data(w_bt) -relief $data(-relief) -bd $data(-borderwidth) \
	-cursor "top_side $data(-cursorbg) $data(-cursorfg)"\
	-bg $data(-background)
    frame $data(w_bb) -relief $data(-relief) -bd $data(-borderwidth) \
	-cursor "bottom_side $data(-cursorbg) $data(-cursorfg)"\
	-bg $data(-background)
    frame $data(w_bl) -relief $data(-relief) -bd $data(-borderwidth) \
	-cursor "left_side $data(-cursorbg) $data(-cursorfg)"\
	-bg $data(-background)
    frame $data(w_br) -relief $data(-relief) -bd $data(-borderwidth) \
	-cursor "right_side $data(-cursorbg) $data(-cursorfg)"\
	-bg $data(-background)
}

proc tixReH::SetBindings {w} {
    upvar #0 $w data

    tixChainMethod $w SetBindings

    bind $data(w_r00)   <1> \
	"tixReH::dragstart $w $data(w_r00)   1 %X %Y  {1  1 -1 -1}"
    bind $data(w_r01)   <1> \
	"tixReH::dragstart $w $data(w_r01)   1 %X %Y  {1  0 -1  1}"
    bind $data(w_r10)   <1> \
	"tixReH::dragstart $w $data(w_r10)   1 %X %Y  {0  1  1 -1}"
    bind $data(w_r11)   <1> \
	"tixReH::dragstart $w $data(w_r11)   1 %X %Y  {0  0  1  1}"
    bind $data(w_bt)    <1> \
	"tixReH::dragstart $w $data(w_bt)    1 %X %Y  {0  1  0 -1}"
    bind $data(w_bb)    <1> \
	"tixReH::dragstart $w $data(w_bb)    1 %X %Y  {0  0  0  1}"
    bind $data(w_bl)    <1> \
	"tixReH::dragstart $w $data(w_bl)    1 %X %Y  {1  0 -1  0}"
    bind $data(w_br)    <1> \
	"tixReH::dragstart $w $data(w_br)    1 %X %Y  {0  0  1  0}"

    foreach win [list \
		 $data(w_r00)\
		 $data(w_r01)\
		 $data(w_r10)\
		 $data(w_r11)\
		 $data(w_bt)\
		 $data(w_bb)\
		 $data(w_bl)\
		 $data(w_br)\
		 ] {
	bind $win <B1-Motion>       "tixVRe::drag    $w %X %Y"
	bind $win <ButtonRelease-1> "tixVRe::dragend $w $win 0 %X %Y"
	bind $win <Any-Escape>      "tixVRe::dragend $w $win 1  0  0"
    }
}

proc tixReH::dragstart {w win depress rootx rooty mrect} {
    upvar #0 $w data

    set wx $data(-x)
    set wy $data(-y)
    set ww $data(-width)
    set wh $data(-height)

    tixVRe::dragstart $w $win $depress $rootx $rooty \
	[list $wx $wy $ww $wh] $mrect
}

# Place the hint frame to indicate the changes
#
proc tixReH::SetHintFrame {w x y width height} {
    upvar #0 $w data

    # The four sides of the window
    #
    set x2 [expr "$x+$width"]
    set y2 [expr "$y+$height"]

    place $data(w_ht) -x $x  -y $y  -width  $width  -bordermode outside
    place $data(w_hb) -x $x  -y $y2 -width  $width  -bordermode outside
    place $data(w_hl) -x $x  -y $y  -height $height -bordermode outside
    place $data(w_hr) -x $x2 -y $y  -height $height -bordermode outside

    # This ensures that the hint frames appear as quickly as possible
    #
    update idletasks
}

proc tixReH::ShowHintFrame {w} {
    upvar #0 $w data

    $data(w_ht) config -bg $data(-hintcolor)
    $data(w_hb) config -bg $data(-hintcolor)
    $data(w_hl) config -bg $data(-hintcolor)
    $data(w_hr) config -bg $data(-hintcolor)
}

proc tixReH::HideHintFrame {w} {
    upvar #0 $w data

    # Hide the hint frame
    #
    $data(w_ht) config -bg $data(-background)
    $data(w_hb) config -bg $data(-background)
    $data(w_hl) config -bg $data(-background)
    $data(w_hr) config -bg $data(-background)
}

proc tixReH::UpdateSize {w x y width height} {
    upvar #0 $w data

    set data(-x)      $x
    set data(-y)      $y
    set data(-width)  $width
    set data(-height) $height

    tixReH::ComposeWindow $w

    if {$data(-command) != {}} {
	eval $data(-command) $x $y $width $height
    }
}

proc tixReH::ComposeWindow {w} {
    upvar #0 $w data

    set px $data(-x)
    set py $data(-y)
    set pw $data(-width)
    set ph $data(-height)

    # Set the hint frames
    #
    tixReH::SetHintFrame $w $px $py $pw $ph

    # Set the four corner resize handles
    #
    set sz_2 [expr $data(-handlesize)/2]

    set x1 [expr "$px - $sz_2"]
    set y1 [expr "$py - $sz_2"]
    set x2 [expr "$px - $sz_2" + $pw]
    set y2 [expr "$py - $sz_2" + $ph]

    place $data(w_r00) -x $x1 -y $y1 \
	-width $data(-handlesize) -height $data(-handlesize)
    place $data(w_r01) -x $x1 -y $y2\
	-width $data(-handlesize) -height $data(-handlesize)
    place $data(w_r10) -x $x2 -y $y1\
	-width $data(-handlesize) -height $data(-handlesize)
    place $data(w_r11) -x $x2 -y $y2\
	-width $data(-handlesize) -height $data(-handlesize)


    # Set the four border resize handles
    #
    set mx [expr "$px + $pw/2 - $sz_2"]
    set my [expr "$py + $ph/2 - $sz_2"]

    place $data(w_bt) -x $mx -y $y1 \
	-width $data(-handlesize) -height $data(-handlesize)
    place $data(w_bb) -x $mx -y $y2 \
	-width $data(-handlesize) -height $data(-handlesize)
    place $data(w_bl) -x $x1 -y $my \
	-width $data(-handlesize) -height $data(-handlesize)
    place $data(w_br) -x $x2 -y $my \
	-width $data(-handlesize) -height $data(-handlesize)
}

proc tixReH::config-width {w value} {
    tixDoWhenIdle tixReH::ComposeWindow $w
}

proc tixReH::config-height {w value} {
    tixDoWhenIdle tixReH::ComposeWindow $w
}

proc tixReH::config-x {w value} {
    tixDoWhenIdle tixReH::ComposeWindow $w
}

proc tixReH::config-y {w value} {
    tixDoWhenIdle tixReH::ComposeWindow $w
}


proc tixReH::show {w} {
    upvar #0 $w data

    set data(shown) 1

    raise $data(w_ht)
    raise $data(w_hb)
    raise $data(w_hl)
    raise $data(w_hr)

    raise $data(w_r00)
    raise $data(w_r01)
    raise $data(w_r10)
    raise $data(w_r11)

    raise $data(w_bt)
    raise $data(w_bb)
    raise $data(w_bl)
    raise $data(w_br)

    tixCancleIdleTask tixReH::ComposeWindow $w
    tixReH::ComposeWindow $w
}


proc tixReH::hide {w} {
    upvar #0 $w data

    if {!$data(shown)} {
	return
    }

    set data(shown) 0

    place forget $data(w_r00)
    place forget $data(w_r01)
    place forget $data(w_r10)
    place forget $data(w_r11)

    place forget $data(w_bt)
    place forget $data(w_bb)
    place forget $data(w_bl)
    place forget $data(w_br)

    place forget $data(w_ht)
    place forget $data(w_hb)
    place forget $data(w_hl)
    place forget $data(w_hr)
}


