tixWidgetClass tixSelect {
    -superclass tixLabelWidget
    -classname  TixSelect
    -shortname  tixSel
    -method {
	add button invoke
    }
    -flag {
	-command
	-state -variable -value
	-allowzero -radio -orientation -orient
	-padx -pady -selectedbg -buttontype
    }
    -forcecall {
	-variable
    }
    -static {
	-orientation -radio -allowzero
    }
    -configspec {
	{-buttontype buttonType ButtonType button}
	{-command command Command {}}
	{-state state State normal}
	{-variable variable Variable {}}
	{-value value Value {}}
	{-radio radio Radio true }
	{-allowzero allowZero AllowZero false}
	{-orientation orientation Orientation horizontal}
	{-selectedbg selectedBg SelectedBg gray}
	{-padx padx Pad 0}
	{-pady pady Pad 0}
    }
    -alias {
	{-orient -orientation}
    }
}

proc tixSel::InitWidgetRec {w} {
    upvar #0 $w data

    tixChainMethod $w InitWidgetRec
    set data(nSelected) 0
    set data(items)    {}
    set data(buttonbg) {}
}


#----------------------------------------------------------------------
#                           CONFIG OPTIONS
#----------------------------------------------------------------------
proc tixSel::config-state {w arg} {
    upvar #0 $w data

    if {$arg == "disabled"} {
	foreach item $data(items) {
	    $data(w:$item) config -state disabled -relief raised \
		-bg $data(buttonbg)
	}
    } else {
	foreach item $data(items) {
	    if {$data(s:$item) == "selected"} {
		$data(w:$item) config -relief sunken -bg $data(-selectedbg) \
		    -state normal
	    } else {
		$data(w:$item) config -relief raised -bg $data(buttonbg) \
		    -command "$w invoke $item" -state normal
	    }
	}
    }
}

# %% obsolete
#proc tixSel::config-variable {w arg} {
#    upvar #0 $w data
#
#    if {$data(-variable) != {}} {
#	trace vdelete $data(-variable) w "tixSel::TraceProc $w"
#    }
#
#    if {$arg != {}} {
#	upvar #0 $arg variable
#	if [info exists $arg] {
#	    $w config -value $variable
#	} else {
#	    set variable $data(-value)
#	}
#	global $arg
#	trace variable $arg w "tixSel::TraceProc $w"
#    }
#}


proc tixSel::config-variable {w arg} {
    upvar #0 $w data

    if {$data(-variable) != {}} {
	upvar #0 $data(-variable) oldVar
	trace vdelete oldVar w "tixSel::TraceProc $w"
    }

    if {$arg != {}} {
	upvar #0 $arg newVar
	if [info exists $arg] {
	    $w config -value $newVar
	} else {
	    set variable $data(-value)
	}
	trace variable $newVar w "tixSel::TraceProc $w"
    }
}

proc tixSel::config-value {w arg} {
    upvar #0 $w data

    if {[lindex $arg 0] == "-nocallback"} {
	set docallback 0
	set arg [lrange $arg 1 end]
    } else {
	set docallback 1
    }

    # sanity checking
    foreach item $arg {
	if {[lsearch $data(items) $item] == "-1"} {
	    error "button subwidget \"$item\" does not exist"
	}
    }

    if {$data(-radio) == "true" && [llength $arg] > 1} {
	error "cannot choose more than one items in a radio box"
    }

    if {$data(-allowzero) == "false" && [llength $arg] == 0} {
	error "empty selection not allowed"
    }

    set data(-value) $arg
    tixSel::UpdateVariable $w

    # Reset all to be unselected
    foreach item $data(items) {
	if {[lsearch $arg $item] == "-1"} {
	    if {$data(s:$item) == "selected"} {
		# was selected but now unselected
		# -> popup the button, call command
		$data(w:$item) config -relief raised -bg $data(buttonbg)
		set data(s:$item) unselected
		incr data(nSelected) -1

		if {$docallback && $data(-command) != {}} {
		    eval $data(-command) $item 0
		}
	    }
	} else {
	    if {$data(s:$item) == "unselected"} {
		# was unselected but now selected
		# -> push down the button, call command
		$data(w:$item) config -relief sunken -bg $data(-selectedbg)
		set data(s:$item) selected
		incr data(nSelected)

		if {$docallback && $data(-command) != {}} {
		    eval $data(-command) $item 1
		}
	    }
	}
    }
}

#----------------------------------------------------------------------
#                     WIDGET COMMANDS
#----------------------------------------------------------------------
proc tixSel::add {w name args} {
    upvar #0 $w data

    set data(w:$name) [eval $data(-buttontype) $data(w:frame).$name -command \
		       [list "$w invoke $name"] $args]

    if {$data(-orientation) == "horizontal"} {
	pack $data(w:$name) -side left -expand yes -fill y\
	    -padx $data(-padx) -pady $data(-pady)
    } else {
	pack $data(w:$name) -side top -expand yes  -fill x\
	    -padx $data(-padx) -pady $data(-pady)
    }

    if {$data(-state) == "disabled"} {
	$data(w:name) config -relief raised -state disabled
    }

    # find out the background of the buttons
    #
    if {$data(buttonbg) == {}} {
	set data(buttonbg) [lindex [$data(w:$name) config -background] 4]
	
    }

    # Set the initial values
    #
    set data(s:$name) unselected

    lappend data(items) $name
}

proc tixSel::button {w name args} {
    upvar #0 $w data

    if {$args != {}} {
	return [eval $data(w:$name) $args]
    } else {
	return $w.name
    }
}

#call the command
#
proc tixSel::invoke {w name} {
    upvar #0 $w data

    if {$data(-state) != "normal"} {
	return
    }

    if {$data(s:$name) == "selected"} {
	if {$data(nSelected) > 1 || $data(-allowzero) == "true"} {
	    $data(w:$name) config -relief raised -bg $data(buttonbg)
	    set data(s:$name) unselected
	    incr data(nSelected) -1
	    tixSel::UpdateValue $w 

	    if {$data(-command) != {}} {
		eval $data(-command) $name 0
	    }
	}
    } else {
	if {$data(-radio) == "true"} {
	    set data(nSelected) 1
	    # cannot call Updatevalue because data(w:??) not set yet.
	    set data(-value) $name
	    tixSel::UpdateVariable $w

	    foreach item $data(items) {
		if {$item == $name} {
		    $data(w:$item) config -relief sunken -bg $data(-selectedbg)
		    set data(s:$item) selected

		    if {$data(-command) != {}} {
			eval $data(-command) $item 1
		    }
		} elseif {$data(s:$item) == "selected"} {
		    $data(w:$item) config -relief raised -bg $data(buttonbg)
		    set data(s:$item) unselected

		    if {$data(-command) != {}} {
			eval $data(-command) $item 0
		    }
		}
	    }
	} else {
	    $data(w:$name) config -relief sunken -bg $data(-selectedbg)
	    set data(s:$name) selected
	    incr data(nSelected)
	    tixSel::UpdateValue $w 

	    if {$data(-command) != {}} {
		eval $data(-command) $name 1
	    }
	}
    }
}

#----------------------------------------------------------------------
#                Private functions
#----------------------------------------------------------------------
proc tixSel::UpdateValue {w} {
    upvar #0 $w data

    set data(-value) {}

    foreach item $data(items) {
	if {$data(s:$item) == "selected"} {
	    lappend data(-value) $item
	}
    }

    tixSel::UpdateVariable $w
}


proc tixSel::UpdateVariable {w} {
    upvar #0 $w data

    if {$data(-variable) != {}} {
	upvar #0 $data(-variable) variable

	trace vdelete  variable w "tixSel::TraceProc $w"
	set variable $data(-value)
	trace variable variable w "tixSel::TraceProc $w"
    }
}

proc tixSel::TraceProc {w name1 name2 op} {
    upvar #0 $name1 var

    if {$name2 == {}} {
	$w config -value $var
    } else {
	$w config -value $var($name2)
    }
}
	
# requirement of -valiable
# 1) Always updated before -command is called
