# Stub function
#

proc tixInit {args} {
    if {[info commands tix] != {}} {
	return tix
    }
    return [eval tixAppContext tix $args]
}

#----------------------------------------------------------------------
# Tix application context class
#
#
tixClass tixAppContext {
    -superclass {}
    -classname  TixAppContext
    -shortname  tixApp
    -method {
	addbitmapdir setbitmap getbitmap
    }
    -flag {
	-autoscroll -beta -binding -debug -fontset
	-grabmode -haspixmap -libdir -scheme
    }
    -static {
	
    }
    -readonly {
	-haspixmap
    }
    -configspec {
	{-autoscroll 	true}
	{-beta       	true}
	{-binding    	TK}
	{-debug      	false}
	{-fontset    	TK}
	{-grabmode 	global}
	{-haspixmap 	0}
	{-libdir     	/usr/local/lib/tix}
	{-scheme     	TK}
    }
}

proc tixApp::Constructor {w} {
    upvar #0 $w data

    # Set up the AutoPath for some auxiliary packages
    #
    global auto_path tix_priv

    lappend auto_path $data(-libdir)/schemes
    lappend auto_path $data(-libdir)/bindings

    # Enable/Disable beta widgets
    #
    if {$data(-beta)} {
	lappend auto_path $data(-libdir)/beta
    }

    # Enable/Disable Intrinsics debugging
    #
    if {$data(-debug)} {
	set tix_priv(debug) 1
    } else {
	set tix_priv(debug) 0
    }

    tixApp::BitmapInit $w
    tixSetFontset::$data(-fontset)
    tixSetColorScheme::$data(-scheme)
    tixInitOptionDatabase
    tixSetBinding::$data(-binding)

    if {$data(-debug) == "true"} {
	# For widget programming, it is more convient to have the error
	# message printed on the terminal. For some extensive useage of
	# bindings, suce as in the case of tixBalloon, the default
	# therror just doesn't work.
	#
	proc tkerror {err} {
	    global errorInfo
	    puts $err
	    puts $errorInfo
	}
    }
}

proc tixApp::BitmapInit {w} {
    upvar #0 $w data

    # See whether we have pixmap extension
    #
    set data(-haspixmap) true

    # Dynamically set the bitmap directory
    #
    set data(bitmapdirs) $data(-libdir)/bitmaps
}

proc tixApp::addbitmapdir {w bmpdir} {
    upvar #0 $w data

    if {[lsearch $data(bitmapdirs) $bmpdir] == "-1"} {
	set data(bitmapdirs) [concat $bmpdir $data(bitmapdirs)]
    }
}


proc tixApp::getbitmap {w bitmapname} {
    upvar #0 $w data

    if {[info exists data(bmp:$bitmapname)]} {
	return $data(bmp:$bitmapname)
    } else {
	if {$data(-haspixmap)} {
	    set ext xpm
	} else {
	    set ext xbm
	}

	# This is the fallback value. If we can't find the bitmap in
	# the bitmap directories, then use the name of the bitmap
	# as the default value.
	#
	set data(bmp:$bitmapname) $bitmapname

	foreach dir $data(bitmapdirs) {
	    if {$data(-haspixmap)} {
		if [file exists $dir/$bitmapname.xpm] {
		    set data(bmp:$bitmapname) @$dir/$bitmapname.xpm
		    break
		}
		if [file exists $dir/$bitmapname.xbm] {
		    set data(bmp:$bitmapname) @$dir/$bitmapname.xbm
		    break
		}
	    } else {
		if [file exists $dir/$bitmapname.xbm] {
		    set data(bmp:$bitmapname) @$dir/$bitmapname.xbm
		    break
		}
	    }
	    if [file exists $dir/$bitmapname] {
		set data(bmp:$bitmapname) @$dir/$bitmapname
		break
	    }
	}

	return $data(bmp:$bitmapname)
    }
}

tixInit
