#----------------------------------------------------------------------
#
#
#           Package loading
#
#
#----------------------------------------------------------------------
proc setenv {name args} {
    global env

    if {[llength $args] == 1} {
        return [set env($name) [lindex $args 0]]
    } else {
        if {[info exists env($ename)] == 0} {
            tkerror "Error in setenv: "
                    "environment variable \"$name\" does not exist"
        } else {
            return $env($name)
        }
    }
}
#----------------------------------------------------------------------
#
#
#           Handling options in a command
#
#
#----------------------------------------------------------------------
proc tixHandleOptions {p_record validOptions arglist} {
    upvar $p_record data

    set len [expr [llength $arglist]-2 ]
    set i 0

    while {$i <= $len} {
	set option [lindex $arglist $i]
	incr i
	set arg [lindex $arglist $i]
	incr i

	if {[lsearch $validOptions $option] == "-1"} {
	    return -code error \
		"unknown option $option; must be one of $validOptions"
	}
	set data($option) $arg
    }
}

#
# kludge: should be able to handle all kinds of flags
#         now only handles "-flag value" pairs.
#
proc tixHandleArgv {p_argv p_options validFlags} {
    upvar $p_options opt
    upvar $p_argv    argv

    set old_argv $argv
    set argv {}

    tixForEach {flag value} $old_argv {
	if {[lsearch $validFlags $flag] != "-1"} {
	    # The caller will handle this option exclusively
	    # It won't be added back to the original arglist
	    #
	    eval $opt($flag,action) $value
	} else {
	    # The caller does not handle this option
	    #
	    lappend argv $flag
	    lappend argv $value
	}
    }
}


#----------------------------------------------------------------------
#
#
#           U T I L I T Y   F U N C T I O N S  F O R   T I X 
#
#
#----------------------------------------------------------------------

# RESET THE STRING IN THE ENTRY
proc tixSetEntry {entry string} {
    set oldstate [lindex [$entry config -state] 4]
    $entry config -state normal
    $entry delete 0 end
    $entry insert 0 $string
    $entry config -state $oldstate
}

# GET THE FIRST SELECTED ITEM IN A LIST
proc tixListGetSingle {lst} {
    set indices [$lst curselection]
    if {$indices != "" } {
	return [$lst get [lindex $indices 0]]
    } else {
	return ""
    }
}
#-------------------------------
# DISABLE EVERYTHING IN A WINDOW
#-------------------------------
proc tixDisableAll {w} {
    foreach x [tixDescendants $w] {
	catch {$x config -state disabled}
    }
}
#------------------------------
# ENSABLE EVERYTHING IN A WINDOW
#------------------------------
proc tixEnableAll {w} {
    foreach x [tixDescendants $w] {
	catch {$x config -state normal}
    }
}
#------------------------------
# RETURN EVERYTHING IN A WINDOW
#------------------------------
proc tixDescendants {parent} {
    set des {}
    lappend des $parent

    foreach w [winfo children $parent] {
	foreach x [tixDescendants $w] {
	    lappend des $x
	}
    }
    return $des
}
#----------------------------------------------------------------------
# RECORD A DIALOG'S POSITION AND RESTORE IT THE NEXT TIME IT IS OPENED
#----------------------------------------------------------------------
proc tixDialogRestore {w {flag -geometry}} {
    global tixDPos

    if [info exists tixDPos($w)] {
	if ![winfo ismapped $w] {
	    wm geometry $w $tixDPos($w)
	    wm deiconify $w
	}
    } elseif {$flag == "-geometry"} {
	update
	set tixDPos($w) [winfo geometry $w]
    } else {
	update
	set tixDPos($w) +[winfo rootx $w]+[winfo rooty $w]
    }
}
#----------------------------------------------------------------------
# RECORD A DIALOG'S POSITION AND RESTORE IT THE NEXT TIME IT IS OPENED
#----------------------------------------------------------------------
proc tixDialogWithdraw {w {flag -geometry}} {
    global tixDPos

    if [winfo ismapped $w] {
	if {$flag == "-geometry"} {
	    set tixDPos($w) [winfo geometry $w]
	} else {
	    set tixDPos($w) +[winfo rootx $w]+[winfo rooty $w]
	}
	wm withdraw $w
    }
}
#----------------------------------------------------------------------
# RECORD A DIALOG'S POSITION AND RESTORE IT THE NEXT TIME IT IS OPENED
#----------------------------------------------------------------------
proc tixDialogDestroy {w {flag -geometry}} {
    global tixDPos

    if [winfo ismapped $w] {
	if {$flag == "-geometry"} {
	    set tixDPos($w) [winfo geometry $w]
	} else {
	    set tixDPos($w) +[winfo rootx $w]+[winfo rooty $w]
	}
    }
    destroy $w
}

#----------------------------------------------------------------------
# Extension of foreach, can handle more than one names
#----------------------------------------------------------------------
proc tixForEach {names list body} {
    set len [llength $list]
    set i 0

    while {$i < $len} {
	foreach name $names {
	    uplevel 1 [list set $name [lindex $list $i]]
	    incr i
	}

	if {$i > $len} {
	    error "incorrect number of items in the list \{$list\}"
	}

	uplevel 1 $body
    }
}

# Obsolete
#
proc tixQueryAppResource {name class default} {

    set value [option get . $name $class]
    if {$value == ""} {
	return $default
    } else {
	return $value
    }    
}

# RESET THE STRING IN THE ENTRY
#
proc tixDoWhenIdle {command args} {
    global tix_priv

    if [info exists tix_priv(pending,$command:$args)] {
	return
    }

    set tix_priv(pending,$command:$args) 1
    after 0 _tixCallIdle $command $args
}

proc tixCancleIdleTask {command args} {
    global tix_priv

    if [info exists tix_priv(pending,$command:$args)] {
	unset tix_priv(pending,$command:$args)
    }
}

proc _tixCallIdle {command args} {
    global tix_priv

    if [info exists tix_priv(pending,$command:$args)] {
	unset tix_priv(pending,$command:$args)
	eval $command $args
    }
}
