# expand-mode
#
# When display area is bigger than request area, what to do?
#   adjust : keep the w:window to be the reqwidth/height
#   expand : expand w:window to fill the empty area
#   adjustexpand : first adjust, then expand
#
# Terminology:
#   "child window"
#       - the window that is being scrolled
#   "display window"
#       - the part of the child window that appears on the screen.
#       - i.e., the internal subwidget "int0"
#
# Scrollbar: none, x, y, x-yauto, y-xauto, auto, both
#
#

tixWidgetClass tixVScroll {
    -superclass tixPrimitive
    -classname  TixVScroll
    -shortname  tixVSW
    -method {
	setsize
    }
    -flag {
	-scrollbar -reqwidth -reqheight
    }
    -static {
	-scrollbar
    }
    -configspec {
	{-command command Command {}}
	{-expandmode expandMode ExpandMode adjust}
	{-scrollbar scrollbar Scrollbar both}
	{-reqwidth reqWidth ReqWidth 0}
	{-reqheight reqHeight ReqHeight 0}
    }
}

proc tixVSW::InitWidgetRec {w} {
    upvar #0 $w data

    tixChainMethod $w InitWidgetRec

    set data(type)          nil
    set data(settingscroll) 0
}

proc tixVSW::ConstructWidget {w} {
    upvar #0 $w data

    tixChainMethod $w ConstructWidget

    # Top frame
    frame $w.top -border 0

    # Frame inside top frame
    # Reason of having two int frame is: place would overlap the
    # border of int0 if there were only one frame.
    #
    frame $w.int0 -border 2 -relief sunken
    frame $w.int1 -border 0
    pack $w.int1 -in $w.int0 -expand yes -fill both

    # Bottom frame
    frame $w.bot -border 0

    scrollbar $w.hsb -orient horizontal

    scrollbar $w.vsb

    set scbWidth [lindex [$w.vsb config -width] 4]
    set scbBd    [lindex [$w.vsb config -bd]    4]

    frame $w.xframe -height $scbWidth -relief flat \
	-width [expr "$scbWidth + 2 * $scbBd"]

    set data(w_top)     $w.top
    set data(w_bot)     $w.bot
    set data(w_int0)    $w.int0
    set data(w_int1)    $w.int1
    set data(w:hsb)     $w.hsb
    set data(w:vsb)     $w.vsb
    set data(w:xframe)  $w.xframe

    set data(w:window) [tixCallMethod $w ConstructClientWindow $data(w_int1)]

    pack $data(w:window) -in $data(w_int1) -expand yes -fill both

    tixDoWhenIdle tixVSW::Repack $w
}

proc tixVSW::SetBindings {w} {
    upvar #0 $w data

    tixChainMethod $w SetBindings

    bind $w              <Configure> "tixVSW::MonitorSize $w"
    bind $data(w:window) <Map>       "tixVSW::Map $w"

    $data(w:hsb) config -command "tixVSW::View $w x"
    $data(w:vsb) config -command "tixVSW::View $w y"

    bind $w <Enter>     "focus $w"
    bind $w <Prior>     "tixVSW::RawPriorKey $w"
    bind $w <Next>      "tixVSW::RawNextKey  $w"
    bind $w <Control-u> "tixVSW::RawPriorKey $w"
    bind $w <Control-d> "tixVSW::RawNextKey  $w"
    bind $w <Up>        "tixVSW::RawUpKey    $w"
    bind $w <Down>      "tixVSW::RawDownKey  $w"
}

proc tixVSW::Repack {w} {
    upvar #0 $w data

    if {$data(type) == "nil"} {
	# It is not initialized
	#
	tixVSW::GetClientInfo $w
    }

    if {$data(-scrollbar) == "auto"} {
	set type [tixVSW::ScrollBarsNeeded $w]
    } else {
	set type $data(-scrollbar)
    }

    # Set the scrollbars
    # 
    tixVSW::View $w x [lindex $data(x,clntInfo) 2]
    tixVSW::View $w y [lindex $data(y,clntInfo) 2]

    if {$type != $data(type)} {
	pack forget $data(w_top)
	pack forget $data(w_bot)
	pack forget $data(w_int0)
	pack forget $data(w:hsb)
	pack forget $data(w:vsb)
	pack forget $data(w:xframe)

	set data(type) $type
	tixVSW::Pack $w
    }
}

proc tixVSW::Pack {w} {
    upvar #0 $w data

    case $data(type) {
	both {
	   pack $data(w:vsb)  -in $data(w_top) -side right -fill y
	   pack $data(w_int0) -in $data(w_top) -side left  -fill both -expand 1

	   pack $data(w:hsb)    -in $data(w_bot) -side left -fill x -expand 1
	   pack $data(w:xframe) -in $data(w_bot) -side left -fill y

	   pack $data(w_top) -side top -fill both -expand yes
	   pack $data(w_bot) -side bottom -fill x
	}
	x {
	    pack $data(w:hsb) -side bottom -fill x
	    pack $data(w_int0) -side top -fill both -expand yes
	}
	y {
	    pack $data(w:vsb) -side right -fill y
	    pack $data(w_int0) -side left -fill both -expand yes
	}
	none {
	    pack $data(w_int0) -fill both -expand yes
	}
    }
}

# Ask the client whhat kind of scrollbars are needed
#

# disp = display window
#

proc tixVSW::GetClientInfo {w} {
    upvar #0 $w data

    set dispW [winfo width  $data(w_int1)]
    set dispH [winfo height $data(w_int1)]
    set rootW [winfo width  $w]
    set rootH [winfo height $w]

    set clntInfo [eval tixCallMethod $w GetClientInfo \
		  [list [list $dispW $rootW 0]]\
		  [list [list $dispH $rootH 0]]]

    set data(x,clntInfo) [lindex $clntInfo 0]
    set data(y,clntInfo) [lindex $clntInfo 1]
}

proc tixVSW::ScrollBarsNeeded {w} {
    upvar #0 $w data

    set dispW [winfo width  $data(w_int1)]
    set dispH [winfo height $data(w_int1)]
    set rootW [winfo width  $w]
    set rootH [winfo height $w]

    set clntInfo [eval tixCallMethod $w GetClientInfo \
		  [list [list $dispW $rootW 0]]\
		  [list [list $dispH $rootH 0]]]

    set clntXInfo [lindex $clntInfo 0]
    set clntYInfo [lindex $clntInfo 1]

    set clntXTotal  [lindex $clntXInfo 0]
    set clntXWindow [lindex $clntXInfo 1]
    set clntXFirst  [lindex $clntXInfo 2]
    set clntXLast   [lindex $clntXInfo 3]

    set clntYTotal  [lindex $clntYInfo 0]
    set clntYWindow [lindex $clntYInfo 1]
    set clntYFirst  [lindex $clntYInfo 2]
    set clntYLast   [lindex $clntYInfo 3]

    if {$clntXWindow < $clntXTotal || $clntXFirst > 0} {
	set needX 1
    } else {
	set needX 0
    }
    if {$clntYWindow < $clntYTotal || $clntYFirst > 0} {
	set needY 1
    } else {
	set needY 0
    }

    set data(x,clntInfo) $clntXInfo
    set data(y,clntInfo) $clntYInfo

    if {$needX && $needY} {
	return both
    } elseif {$needX} {
	return x
    } elseif {$needY} {
	return y
    } else {
	return none
    }
}

# The client calls this function when it needs to:
# (1) change its size
# (2) change its view
#
proc tixVSW::ClientRequest {w clntXInfo clntYInfo} {
    upvar #0 $w data

    set data(x,clntInfo) $clntXInfo
    set data(y,clntInfo) $clntYInfo
    tixVSW::Repack $w
}

# Raw event handlers
#
proc tixVSW::RawUpKey {w} {
    tixCallMethod $w HandleLineUp
}

proc tixVSW::RawDownKey {w} {
    tixCallMethod $w HandleLineDown
}

proc tixVSW::RawPriorKey {w} {
    tixCallMethod $w HandlePageUp
}

proc tixVSW::RawNextKey {w} {
    tixCallMethod $w HandlePageDown
}

# Virtual event handlers: subclasses can derive from these functions
#

proc tixVSW::HandleLineUp {w} {
    upvar #0 $w data

    set first  [lindex $data(y,clntInfo) 2]

    tixVSW::View $w y [expr $first - 1]
}

proc tixVSW::HandleLineDown {w} {
    upvar #0 $w data

    set first  [lindex $data(y,clntInfo) 2]

    tixVSW::View $w y [expr $first + 1]
}

proc tixVSW::HandlePageUp {w} {
    upvar #0 $w data

    set window [lindex $data(y,clntInfo) 1]
    set first  [lindex $data(y,clntInfo) 2]

    tixVSW::View $w y [expr $first - $window]
}

proc tixVSW::HandlePageDown {w} {
    upvar #0 $w data

    set window [lindex $data(y,clntInfo) 1]
    set first  [lindex $data(y,clntInfo) 2]

    tixVSW::View $w y [expr $first + $window]
}

proc tixVSW::View {w which first} {
    upvar #0 $w data

    set data($which,clntInfo) \
	[tixCallMethod $w SetClientView $which $first]

    eval tixVSW::SetScrollBar $w $which $data($which,clntInfo)
}

proc tixVSW::Map {w args} {
}

proc tixVSW::MonitorSize {w args} {
    tixVSW::Repack $w
}

proc tixVSW::SetScrollBar {w which total window first last} {
    upvar #0 $w data

    if {$data(settingscroll)} {
        return
    }
    set data(settingscroll) 1

    case $which {
	"x" {
	    $data(w:hsb) set $total $window $first $last
	}
	"y" {
	    $data(w:vsb) set $total $window $first $last
	}
    }
    set data(settingscroll) 0
}
